#<profile4xi profile = "12" />
# =================================================================================================
#<program4xi program = "kpu2image4xi.py">
#<abstract4xi abstract = "KPU 2D image management"/>
#<refer4xi reference = "image2d4xi.py"/>
#<verify4xi verification = "10" target = "kpu2image4xi5test.py">
#<state4xi status = "ACK4review; ACK4tagging; ACK4moduletesting; ACK4systemtesting; [Rel 7.21]"/>
# ================================================================================================= 

# -------------------------------------------------------------------------------------------------
# Public API
# -------------------------------------------------------------------------------------------------
#<interface4xi interface = " 
__all__ = [ 'KPU2Image4Xi',                        # ACK 5.31  p76 
            
            'copy2image4kpu',                      # ACK 5.3.3 p88
            'copy2imagetobuffer4kpu',              # ACK 5.3.4 p90
            'copy2buffertoimage4kpu',              # ACK 5.3.4 p92
            'create2image4kpu',                    # ACK 5.31  p76            

            'get2imageinfo4kpu',                   # ACK 5.3.6 p97 nb: modernized so that you can also access only selected elements, if you want           
            'get2supportedimageformats4kpu',       # ACK 5.3.2 p82 nb: a separate function to get format whereas using get2imageinfo4kpu you can access any element by key such as 'FORMAT' 
                              
            'manoeuvre2copyimage4kpu',             # ACK modernized 
            'manoeuvre2copyimagetobuffer4kpu',     # ACK modernized        
            'manoeuvre2copybuffertoimage4kpu',     # ACK modernized
            'manoeuvre2mapimage4kpu',              # ACK modernized                                           
            'manoeuvre2readimage4kpu',             # ACK modernized
            'manoeuvre2writeimage4kpu',            # ACK modernized            
            'map2image4kpu',                       # ACK 5.3.5 p94 
            
            'read2image4kpu',                      # ACK 5.3.3 p84  
            'retain2image4kpu',                    # ACK 5.4.1 p99
            'release2image4kpu',                   # ACK 5.4.1 p99

            'set2imagecallback4kpu',               # ACK modernized
            'set2image4kpu',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    

            'trace2image4kpu',                     # ACK modernized 
                                    
            'wait2image4kpu',                      # ACK modernized nb: checks Queue object and thereby causes delay doing something useful
            'write2image4kpu'                      # ACK 5.3.3 p84                 
          ] 
#"
#/>


# -------------------------------------------------------------------------------------------------
# Standard Imports                          
# -------------------------------------------------------------------------------------------------
#<standardize4xi standard = "
import os
from platform import processor
import sys
import threading                
import _thread   
from decimal import *
#"
#/>


# -------------------------------------------------------------------------------------------------
# Scope4Xi imports                               
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
import version4xi

# -------------------------------------------------------------------------------------------------
# Buffer module                           
# -------------------------------------------------------------------------------------------------
from kpu2buffer4xi import KPU2Buffer4Xi
from kpu2buffer4xi import create2buffer4kpu
from kpu2buffer4xi import create2subbuffer4kpu
from kpu2buffer4xi import get2bufferinfo4kpu
from kpu2buffer4xi import release2buffer4kpu
from kpu2buffer4xi import retain2buffer4kpu
from kpu2buffer4xi import set2buffer4kpu
from kpu2buffer4xi import trace2buffer4kpu

# -------------------------------------------------------------------------------------------------
# Command queue module                           
# -------------------------------------------------------------------------------------------------
from kpu2commandqueue4xi import KPU2CommandQueue4Xi
from kpu2commandqueue4xi import create2commandqueue4kpu 
from kpu2commandqueue4xi import flush2commandqueue4kpu
from kpu2commandqueue4xi import finish2commandqueue4kpu
from kpu2commandqueue4xi import get2commandqueueinfo4kpu
from kpu2commandqueue4xi import release2commandqueue4kpu
from kpu2commandqueue4xi import retain2commandqueue4kpu
from kpu2commandqueue4xi import set2commandqueue4kpu
from kpu2commandqueue4xi import trace2commandqueue4kpu

# -------------------------------------------------------------------------------------------------
# Context module                           
# -------------------------------------------------------------------------------------------------
from kpu2context4xi import KPU2Context4Xi 
from kpu2context4xi import create2context4kpu
from kpu2context4xi import get2context4kpu
from kpu2context4xi import release2context4kpu
from kpu2context4xi import retain2context4kpu
from kpu2context4xi import set2context4kpu


# -------------------------------------------------------------------------------------------------
# Device module                        
# -------------------------------------------------------------------------------------------------
from kpu2device4xi import KPU4Xi
from kpu2device4xi import init2kpu4xi 
from kpu2device4xi import set2kpu4xi
from kpu2device4xi import get2kpu4xi

# -------------------------------------------------------------------------------------------------
# Event module                           
# -------------------------------------------------------------------------------------------------
from kpu2event4xi import KPU2Event4Xi
from kpu2event4xi import create2event4kpu
from kpu2event4xi import get2eventinfo4kpu
from kpu2event4xi import release2event4kpu
from kpu2event4xi import retain2event4kpu
from kpu2event4xi import set2eventcallback4kpu 
from kpu2event4xi import set2event4kpu
from kpu2event4xi import trace2event4kpu
from kpu2event4xi import wait2event4kpu


# -------------------------------------------------------------------------------------------------
# Platform module                           
# -------------------------------------------------------------------------------------------------
from platformids4xi  import get2platformids4xi
from platforminfo4xi import PlatformInfo4Xi                                                                        
from platforminfo4xi import init2platforminfo4xi
from platforminfo4xi import set2platforminfo4xi
from platforminfo4xi import get2platforminfo4xi
#"
#/>

# -------------------------------------------------------------------------------------------------
# Multiprocessing Imports                          
# -------------------------------------------------------------------------------------------------
#<process4xi process = "
import _multiprocessing
#"
#/>


# -------------------------------------------------------------------------------------------------
# DragonXi imports                              
# -------------------------------------------------------------------------------------------------
#<dragonize4xi dragonization = "
from ace_object4xi import ace_object4xi
from arena4xi import Arena4Xi
from cpu_corecount4xi import cpu_corecount4xi
from dummyprocess4xi import DummyProcess4Xi
from print4xi import print4xi
from pipe4xi import Pipe4Xi
from queue4xi import Queue4Xi  
from time4xi import time4xi
from reason_object4xi import reason_object4xi
from sense_object4xi import sense_object4xi


#"
#/>

# -------------------------------------------------------------------------------------------------
# Global constants                            
# ------------------------------------------------------------------------------------------------
#<globalize4xi globalization = "
global kpu2image4xi                                                     # 2D-IMAGES FOR KPU

global kpu2trace4xi                                                     # TRACE

global kpu2buffer4xi                                                    # BUFFER FOR KPU

global control2readimage4kpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH KPU  
global state2read4kpu                                                   # STATE FOR COMMAND AND CONTROL
global max2readstate4kpu                                                # MAX FOR STATE
global ack2readinfo4kpu                                                 # ACK FOR IMAGE INFO
global message2readget4kpu                                              # QUEUE GET
global message2readput4kpu                                              # QUEUE PUT
global message2readreceive4kpu                                          # PIPE RECEIVE
global message2readsend4kpu                                             # PIPE SEND

global control2writeimage4kpu                                           # COMMAND AND CONTROL TO WRITE 2D IMAGES WITH KPU 
global state2write4kpu                                                  # STATE FOR COMMAND AND CONTROL
global max2writestate4kpu                                               # MAX FOR STATE
global ack2writeinfo4kpu                                                # ACK FOR IMAGE INFO
global message2writeget4kpu                                             # QUEUE GET
global message2writeput4kpu                                             # QUEUE PUT
global message2writereceive4kpu                                         # PIPE RECEIVE
global message2writesend4kpu                                            # PIPE SEND

global control2copyimage4kpu                                            # COMMAND AND CONTROL TO COPY 2D IMAGES WITH KPU 
global state2copyimage4kpu                                              # STATE FOR COMMAND AND CONTROL
global max2copyimagestate4kpu                                           # MAX FOR STATE
global ack2copyimageinfo4kpu                                            # ACK FOR IMAGE INFO
global message2copyimageget4kpu                                         # QUEUE GET
global message2copyimageput4kpu                                         # QUEUE PUT
global message2copyimagereceive4kpu                                     # PIPE RECEIVE
global message2copyimagesend4kpu                                        # PIPE SEND

global control2copyimagetobuffer4kpu                                    # COMMAND AND CONTROL TO COPY 2D IMAGES TO BUFFER WITH KPU 
global state2copyimagetobuffer4kpu                                      # STATE FOR COMMAND AND CONTROL
global max2copyimagetobufferstate4kpu                                   # MAX FOR STATE
global ack2copyimagetobufferinfo4kpu                                    # ACK FOR IMAGE INFO
global message2copyimagetobufferget4kpu                                 # QUEUE GET
global message2copyimagetobufferput4kpu                                 # QUEUE PUT
global message2copyimagetobufferreceive4kpu                             # PIPE RECEIVE
global message2copyimagetobuffersend4kpu                                # PIPE SEND

global control2copybuffertoimage4kpu                                    # COMMAND AND CONTROL TO COPY BUFFER TO 2D IMAGE WITH KPU 
global state2copybuffertoimage4kpu                                      # STATE FOR COMMAND AND CONTROL
global max2copybuffertoimagestate4kpu                                   # MAX FOR STATE
global ack2copybuffertoimageinfo4kpu                                    # ACK FOR IMAGE INFO
global message2copybuffertoimageget4kpu                                 # QUEUE GET
global message2copybuffertoimageput4kpu                                 # QUEUE PUT
global message2copybuffertoimagereceive4kpu                             # PIPE RECEIVE
global message2copybuffertoimagesend4kpu                                # PIPE SEND                      

global control2mapimage4kpu                                            # COMMAND AND CONTROL TO READ  2D IMAGES WITH KPU  
global state2map4kpu                                                   # STATE FOR COMMAND AND CONTROL
global max2mapstate4kpu                                                # MAX FOR STATE
global ack2mapinfo4kpu                                                 # ACK FOR IMAGE INFO
global message2mapget4kpu                                              # QUEUE GET
global message2mapput4kpu                                              # QUEUE PUT
global message2mapreceive4kpu                                          # PIPE RECEIVE
global message2mapsend4kpu                                             # PIPE SEND
            
#"
#/>

# -------------------------------------------------------------------------------------------------
# Settings
# -------------------------------------------------------------------------------------------------
#<set4xi setting = "

#"
#"/>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "trace2image4kpu" usage = "
#
#   print('=' *33)          
#   print('(Step 15A/25) OF KPU2IMAGE4XI5TEST - TRYING TO TRACE KPU: ', time4xi())      
#   print('=' *33)      
#   trace2kpu = None
#   try:
#       trace2kpu = trace2image4kpu() 
#       print('(Step 15B/25) OF KPU2IMAGE4XI5TEST - TRACE2KPU: ', trace2kpu)  
#       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4kpu', 'METHOD': 'trace2image4kpu'}
#   except:
#       print('(Step 15Z1/25) OF KPU2IMAGE4XI5TEST *** FAILED TO TRACE KPU: ', trace2kpu)       
#">

# -------------------------------------------------------------------------------------------------   
def trace2image4kpu(task4trace2kpu=None):  

    '''
    creates callback to kpu2image4xi
    '''
    
    global kpu2trace4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)         
    # -----------------------------------------------------------------------------------------          
    class2print = "trace2image4kpu" 
    method2print = "trace2image4kpu"

    message2print = "{(Step 1A/33) started" + "; trace2image4kpu-started: " + str(trace2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                              
    # <function trace2image4kpu at 0x0000000003AF0348> 
  
             
    message2print = "{(Step 1B/33) passed" + "; task4trace2kpu-to-trace2image4kpu: " + str(task4trace2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              # 
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    kpu2trace4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
      
    if task4trace2kpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY KPUIZED CALLBACK !
        # -----------------------------------------------------------------------------------------    
        task4trace2kpu = {}                                                                                            
            
        task4trace2kpu['CLASS']   = "trace2image4kpu"                                                               # KPUIZE ZPU TRACE!
        task4trace2kpu['METHOD']  = "trace2image4kpu" 
        task4trace2kpu['MESSAGE'] = "{(Step 2Z0/33) failed" + "; trace2image4kpu-failed " + str(kpu2trace4xi)  + "; time: " + time4xi() + "}"               
  
        
        message2print = "{(Step 2A/33) set" + "; task4trace2kpu-set-by-trace2image4kpu to: " + str(task4trace2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4kpu',
        #  'METHOD': 'trace2image4kpu'}
        #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}
 
    
        code2ack = code2ack + '2A-'            


    # -----------------------------------------------------------------------------------------
    # PARAMETER GIVEN BY KPUIZED CALLBACK !
    # -----------------------------------------------------------------------------------------          
    message2print = "{(Step 3A/33) try" + "; task4trace2kpu-try-by-trace2image4kpu: " + str(task4trace2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)    
    #  { trace2image4kpu : trace2image4kpu ; (Step 3A/33) try; task4trace2kpu-try-by-trace2image4kpu: 
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 08:14:33 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'trace2image4kpu',
    #  'METHOD': 'trace2image4kpu'}
    #    ; time: 08:14:33 09/06/11 Pacific Daylight Time}

    try:
        kpu2trace4xi ['CLASS']  = task4trace2kpu['CLASS']                                                      # KPUIZE KPU TRACE!
        kpu2trace4xi ['METHOD']  = task4trace2kpu['METHOD']
        kpu2trace4xi ['MESSAGE'] = task4trace2kpu['MESSAGE']
        
        message2print = "{(Step 3B/33) set" + "; kpu2trace4xi-set-by-trace2image4kpu to: " + str(kpu2trace4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4kpu : trace2image4kpu ; (Step 3B/33) set; kpu2trace4xi-set-by-trace2image4kpu to: 
        #                     {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}',
        #                      'CLASS': 'trace2image4kpu',
        #                      'METHOD': 'trace2image4kpu'}
        #                     ; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3B-'         
    except:
        message2print = "{(Step 3Z1/33) failed" + "; kpu-trace-failed-by-trace2image4kpu: " + str(task4trace2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z1-' 
        
    try:
        # =================================
        # SIMULATE KPU TRACE
        # =================================
        class2print   = task4trace2kpu['CLASS']                                                                # KPUIZE KPU TRACE!
        method2print  = task4trace2kpu['METHOD']
        
        message2print = task4trace2kpu['MESSAGE']
        print4xi(class2print, method2print, message2print)                                                     # KPUIZED TRACE FOR OPENCL CALLBACK!
        # { trace2image4kpu : trace2image4kpu ; (Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}
        
        code2ack = code2ack + '3C-'         
    except:
        class2print = "trace2image4kpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4kpu"   
        message2print = "{(Step 3Z2/33) failed" + "; simulation-of-kpu-trace-failed-by-trace2image4kpu: " + str(task4trace2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z2-' 
        
    try:
        # =================================
        # NORMAL TRACE
        # =================================                     
        class2print = "trace2image4kpu"                                                                        # SET TRACE BACK TO THIS FUNCTION!
        method2print = "trace2image4kpu"                
        message2print = "{(Step 3D/33) performed" + "; kpu-trace-performed-by-trace2image4kpu: " + str(task4trace2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # { trace2image4kpu : trace2image4kpu ; (Step 3D/33) performed; kpu-trace-performed-by-trace2image4kpu: 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
        # 'CLASS': 'trace2image4kpu', 'METHOD': 'trace2image4kpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

        code2ack = code2ack + '3D-' 
    except:
        message2print = "{(Step 3Z3/33) failed" + "; kpu-trace-failed-by-trace2image4kpu: " + str(task4trace2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                    
        code2nack = code2nack + '3Z3-' 

    # -----------------------------------------------------------------------------------------
    # (Step 33/3)           
    # -----------------------------------------------------------------------------------------
    message2print = "{(Step 33/33) returning" + "; kpu2trace4xi-returning-by-trace2image4kpu: " + str(kpu2trace4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 23:41:37 09/06/11 Pacific Daylight Time}', 
    #  'CLASS': 'trace2image4kpu', 
    #  'METHOD': 'trace2image4kpu'}; time: 23:41:37 09/06/11 Pacific Daylight Time}

    
    return kpu2trace4xi
            
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2imagecallback4kpu" usage = "
#
#   print('(Step 14A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE CALLBACK OF KPU DEVICE ', time4xi())  
#   try:                               
#       response4setcallback2image = set2imagecallback4kpu()                                                                                                 
#       print('(Step 14B/700) OF KPU2IMAGE4XI5TEST - SET 2D IMAGE CALLBACK OF KPU DEVICE TO: ', response4setcallback2image)   
#       # <function trace2image4kpu at 0x0000000004E2CAC8>
#   except:
#       print('(Step 14Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE CALLBACK OF KPU DEVICE ', time4xi())  
# 
#">

# ------------------------------------------------------------------------------------------------- 
 
def set2imagecallback4kpu(task4set2imagecallback=None):

    '''
    registers callback function to kpu2image4xi
    '''
    global kpu2image4xi
    
    class2print = "set2imagecallback4kpu" 
    method2print = "set2imagecallback4kpu"
    
    message2print = "{(Step 1A/70) started" + "; set2imagecallback4kpu-started: " + str(set2imagecallback4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function set2imagecallback4kpu at 0x000000000390B748>
    
    try:
    
        kpu2image4xi ['CALLBACK'] = trace2image4kpu
        
        message2print = "{(Step 1B/70) set callback2kpu" + "; imagecallback4kpu-set-by-set2imagecallback4kpu: " + str(kpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4kpu at 0x0000000003A9E348> 
        
        message2print = "{(Step 1C/70) returning" + "; callback-of-kpu2image4xi-returning-by-set2imagecallback4kpu: " + str(kpu2image4xi ['CALLBACK']) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4kpu at 0x0000000003864A48> 
        
        return kpu2image4xi ['CALLBACK']      
        
    except:
        message2print = "{(Step 1Z1/70) failed" + "; callback2image-failed-by-set2imagecallback4kpu: " + str(callback2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         

        message2print = "{(Step 1Z2/70) returning" + "; trace2image4kpu-returning-by-set2imagecallback4kpu: " + str(trace2image4kpu) + "; time: " + time4xi() + "}"  
        print4xi(class2print, method2print, message2print)      
            
        return trace2image4kpu
            
#</function4xi> 




# -------------------------------------------------------------------------------------------------
#<function4xi function = "create2image4kpu" usage = "
#   #1     
#   print('=' *33)          
#   print('(Step 2A/700) OF KPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE4KPU ', time4xi())      
#   print('=' *33)        
#   # 
#   try:
#       response4create2Dimage = create2image4kpu()                                                                 
#       print('(Step 2B/700) OF KPU2IMAGE4XI5TEST - CREATED 2D IMAGE4KPU ', time4xi())
#       keys2response = response4create2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 2Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE4KPU *** ', time4xi())
#   #2
#   print('(Step 123A/700) OF KPU2IMAGE4XI5TEST - TRYING TO CREATE 2D IMAGE for KPU ', time4xi()) 
#   # 
#   try:
#       xi2kpu = KPU2Image4Xi()                            
#       response4create2Dimage = xi2kpu.create2image()                                  
#       keys2response = response4create2Dimage.keys()
#       print('(Step 122B/700) OF KPU2IMAGE4XI5TEST - CREATED 2D IMAGE ', time4xi())  
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4create2Dimage[key2response])
#           # 
#   except:
#       print('(Step 123Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO CREATE 2D IMAGE for KPU ', time4xi())    
#">

# -------------------------------------------------------------------------------------------------   
def create2image4kpu(task4imagecreate2kpu=None):

    '''
    creates 2D image for KPU
    '''
    
    global kpu2image4xi                                                                                              
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "create2image4kpu" 
    method2print = "create2image4kpu"

    message2print = "{(Step 1A/70) started" + "; create2image4kpu-started: " + str(create2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # <function create2image4kpu at 0x00000000038D97C8> 
  
             
    message2print = "{(Step 1B/70) passed" + "; task4imagecreate2kpu-to-create2image4kpu: " + str(task4imagecreate2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None 

    code2ack  = ' '
    code2nack = ' '    
    
    kpu2image4xi = {}                                                                                               # INITIALIZES TO EMPTY DICTIONARY
               
    try:             
        if task4imagecreate2kpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR CREATION - BUILD TASK TO KPU! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecreate2kpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1C/70) initialized" + "; task4imagecreate2kpu-created-by-create2image4kpu to: " + str(task4imagecreate2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {}
            
            function2callback4kpu = trace2image4kpu                                                                 # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 1D/70) initialized" + "; function2callback4kpu-created-by-create2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4kpu at 0x00000000038D9348>
            
            code2ack  = code2ack + '1D-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "create2image4kpu"                                                          # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "create2image4kpu"                                                          # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; create2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1E/70) initialized" + "; args2callback-created-by-create2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4kpu',
            #  'METHOD': 'create2image4kpu'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

            code2ack  = code2ack + '1E-'
            
            # ---------------------------------------------------------------------------------------------
            # (1) CALLBACKS - EXTRA                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecreate2kpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecreate2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            code2ack  = code2ack + '2C-'
            message2print = "{(Step 1F/70) initialized" + "; task4imagecreate2kpu-created-by-create2image4kpu to: " + str(task4imagecreate2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4kpu at 0x00000000038D9348>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
            #  'CLASS': 'create2image4kpu',
            #  'METHOD': 'create2image4kpu'}}; time: 00:54:15 09/06/11 Pacific Daylight Time}
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - EXTRA                      
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-create2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-create2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 00:54:15 09/06/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-create2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecreate2kpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2kpu-trying-by-create2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2kpu-got-by-create2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 00:54:15 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2context4kpu',
                #  'METHOD': 'create2context4kpu'},
                #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x00000000037D0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0, 'PROPERTIES': None}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2kpu-failed-by-create2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecreate2kpu ['CONTEXT'] = context2kpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - EXTRA                            
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2kpu-trying-by-create2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2kpu-got-by-create2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318,
                #  'ID2PLATFORM': '0000000001509568',
                #  'PROFILING2QUEUE': 'ACK',
                #  'EXECUTE2KERNELS4OPENCL': 'ACK',
                #  'NATIVE2WIDTHLONG4VECTOR': 2,
                #  'MAX2UNITS4COMPUTE': 4,
                #  'SIZE2MEMORY4GLOBAL': 3221225472,
                #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
                #  'MAX2SIZE4WORKGROUP': 1024,
                #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
                #  'SIZE2CACHE': 65536,
                #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
                #  'TYPE4CACHE': 'Read/Write',
                #  'PREFERRED2WIDTHINT4VECTOR': 4,
                #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
                #  'NATIVE2WIDTHINT4VECTOR': 4,
                #  'SIZE2BUFFER4CONSTANT': 65536,
                #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
                #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
                #  'MAX2NUMBER4CONSTANTARGS': 8,
                #  'VERSION2DRIVER': '2.0',
                #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics cl_khr_global_int32_extended_atomics cl_khr_local_int32_base_atomics 
              
                #  'AVAILABLE2COMPILER': 'ACK',
                #  'MAX2DIMENSIONS4WORKITEMS': 3,
                #  'SUPPORT2CORRECTION4ERROR': 0,
                #  'BITS2ADDRESS': 64,
                #  'AVAILABLE': 'ACK',
                #  'SIZE2MEMORY4LOCAL': 32768,
                #  'VENDOR': 'AuthenticAMD',
                #  'SUPPORT2IMAGE': 'NACK',
                #  'MAX2SIZE4KERNELARGUMENT': 4096,
                #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-
                #  'OUT-OF-ORDER2QUEUE': 'NACK',
                #  'ALIGNMENT2ADDRESS4BASE': 1024,
                #  'PROFILE': 'FULL_PROFILE',
                #  'TYPE2MEMORY4LOCAL': 'Global',
                #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
                #  'PREFERRED2WIDTHLONG4VECTOR': 2,
                #  'MAX2FREQUENCY4CLOCK': 3210,
                #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
                #  'ID': 4033,
                #  'MAX2WORKITEM2': 1024,
                #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
                #  'ROUND2ZERO': 'ACK',
                #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
                #  'MAX2WORKITEM1': 1024,
                #  'MAX2WORKITEM0': 1024,
                #  'SIZE2LINE4CACHE': 64,
                #  'MAX2ALLOCATION4MEMORY': 1073741824,
                #  'NATIVE2WIDTHCHAR4VECTOR': 16,
                #  'QUIET2NANS': 'ACK',
                #  'NATIVE2WIDTHSHORT4VECTOR': 8,
                #  'ROUND2NEAREST4EVEN': 'ACK',
                #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
                #  'NACK2RETURN': ' ',
                #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
                #  'ENDIANESS2DEVICE': 'Little',
                #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
                #  'ROUND2+VE3INFINITY': 'ACK',
                #  'TYPE': 'CL_DEVICE_TYPE_KPU',
                #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 00:54:15 09/06/11 Pacific Daylight Time}

                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2kpu-failed-by-create2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
                
            task4imagecreate2kpu ['DEVICE'] = device2kpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) ERROR CODE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            task4imagecreate2kpu ['CODE2ERROR'] = 0             
            code2ack  = code2ack + '5B-'              

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - EXTRA                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2kpu-trying-by-create2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2kpu-got-by-create2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003991B70> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2kpu-failed-by-create2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagecreate2kpu ['QUEUE'] = queue2kpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - EXTRA                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2kpu-trying-by-create2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2kpu-got-by-create2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 388>, <read-write PipeConnection, handle 392>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2kpu-failed-by-create2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecreate2kpu ['PIPE'] = pipe2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2kpu-trying-by-create2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2kpu-got-by-create2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4kpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #  'CLASS': 'create2commandqueue4kpu',
                #  'METHOD': 'create2commandqueue4kpu'},
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4kpu at 0x000000000372C848>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
                #  'INFO2PRIVATE': None,
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A7A6D8>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0 ... 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4kpu', 
                #  'METHOD': 'create2context4kpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2kpu-failed-by-create2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagecreate2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2kpu-trying-by-create2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                                   # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2kpu-got-by-create2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'COUNT2REFERENCE': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2buffer4kpu',
                #                    'METHOD': 'create2buffer4kpu'},
                #  'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x0000000003B2B348>,
                #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003ADD2F0>,
                #  'FLAGS': 0,
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 05:04:51 09/06/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4kpu', 
                #  'METHOD': 'create2context4kpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x00000000036E0148>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, ...
                
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2kpu-failed-by-create2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecreate2kpu ['BUFFER'] = bufferinfo2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT INFO - EXTRA                              
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2kpu-trying-by-create2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                     # SYSTEM CREATES EVENT IF NECESSAY   
                message2print = "{(Step 10B/70) got" + "; eventinfo2kpu-got-by-create2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4kpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}',
                #                    'CLASS': 'create2event4kpu',
                #                    'METHOD': 'create2event4kpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4kpu at 0x0000000003747D48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003A02898>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 05:10:51 09/06/11 Pacific Daylight Time}', 
                #                                                      'CLASS': 'create2context4kpu', 'METHOD': 'create2context4kpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x00000000036F0148>,
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
                #  'INFO2PRIVATE': None,
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ',
                #  'SIZE2PRIVATE': 0,
                #  'PROPERTIES': None},
                #  'COMMANDQUEUE': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
                
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2kpu-failed-by-create2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecreate2kpu ['EVENT'] = eventinfo2kpu             
            
            # ---------------------------------------------------------------------------------------------
            # (11) FLAGS - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            flags2kpu = 0
           
            message2print = "{(Step 11A/70) trying" + "; flags2kpu-trying-by-create2image4kpu: " + str(flags2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2kpu = 0                                                              
                message2print = "{(Step 11B/70) set" + "; flags2kpu-set-by-create2image4kpu to: " + str(flags2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; flags2kpu-failed-by-create2image4kpu: " + str(flags2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecreate2kpu ['FLAGS'] = flags2kpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) WIDTH - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            width2kpu = 0

            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI ETC...
            width2layer = 10                                                                       # SCALED TO 10 EACH LAYER            
            
            message2print = "{(Step 14A/70) trying" + "; width2kpu-trying-by-create2image4kpu: " + str(width2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                width2kpu = int(width2layer * (quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra))
                width2kpu = 2 * width2kpu                                                          # POS and NEG X  2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 14B/70) set" + "; width2kpu-set-by-create2image4kpu to: " + str(width2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; width2kpu-failed-by-create2image4kpu: " + str(width2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecreate2kpu ['WIDTH'] = width2kpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) HEIGHT - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            height2kpu = 0
            quantity2core = 1                                                                      # ORIGIN 
            quantity2skeleton = 1                                                                  # SKELETON WITH DISTANCE 1
            quantity2layers = 33                                                                   # LAYERS 10, 100 ... 900, 1000 ... 3300
            quantity2frame = 1                                                                     # FRAME
            quantity2extra = 4                                                                     # CONTROLS FOR GUI
            
            quantity2total = quantity2core + quantity2skeleton + quantity2layers + quantity2frame + quantity2extra
            height2layer = 10                                                                      # SCALED TO 10 
                       
            message2print = "{(Step 15A/70) trying" + "; height2kpu-trying-by-create2image4kpu: " + str(height2kpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
                        
            try:
                height2kpu = height2layer * quantity2total
                
                height2kpu = 2 * height2kpu                                                        # POS and NEG Y   2*400 = 800 PIXELS
                                                                                                                                         
                message2print = "{(Step 15B2/70) set" + "; height2kpu-set-by-create2image4kpu to: " + str(height2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; height2kpu-failed-by-create2image4kpu: " + str(height2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '15Z-'
             
                        
            task4imagecreate2kpu ['HEIGHT'] = height2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2kpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2kpu-trying-by-create2image4kpu: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2kpu = int(width2kpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2kpu-set-by-create2image4kpu to: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                             
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2kpu-failed-by-create2image4kpu: " + str(rowpitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagecreate2kpu ['ROWPITCH'] = rowpitch2kpu     
            
            # ---------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED                           
            # ---------------------------------------------------------------------------------------------             
            pointer2kpu = None
            size2kpu = int(width2kpu * height2kpu)                                                                  # CALCULATED MEMORY
            if size2kpu == 0:
                size2kpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2kpu-trying-by-create2image4kpu: " + str(pointer2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2kpu = Arena4Xi(size2kpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B1/70) got" + "; arena2kpu-got-by-create2image4kpu: " + str(arena2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041C0D68>
                
                size2arena = arena2kpu.size                                                                         # 
                              
                message2print = "{(Step 17B2/70) got" + "; size2arena-got-by-create2buffer4kpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2kpu.buffer
                
                message2print = "{(Step 17B3/70) got" + "; buffer2arena-got-by-create2buffer4kpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x0000000003AAF298> 
                
                code2ack  = code2ack + '17B-' 
                
                if size2arena < size2kpu:
                    size2kpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17C/70) set" + "; size2kpu-set-by-create2buffer4kpu to: " + str(size2kpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17C-'
                else:
                    pass
                    
                pointer2kpu = buffer2arena
                message2print = "{(Step 17D/70) set" + "; pointer2kpu-set-by-create2buffer4kpu to: " + str(pointer2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x0000000003AAF298>
                
                code2ack  = code2ack + '17D-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2kpu-failed-by-create2image4kpu: " + str(pointer2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagecreate2kpu ['POINTER'] = pointer2kpu                            
            task4imagecreate2kpu ['SIZE']    = size2kpu                                                             # EXTRA
            
            
            # ---------------------------------------------------------------------------------------------
            # (18) FORMAT - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            format2kpu = {}
           
            message2print = "{(Step 18A/70) trying" + "; format2kpu-trying-by-create2image4kpu: " + str(format2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                format2kpu['ORDER'] = 'CL_RGBA'                                                              
                message2print = "{(Step 18B/70) set" + "; format2kpu-set-by-create2image4kpu to: " + str(format2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'ORDER': 'CL_RGBA'}
                 
                code2ack  = code2ack + '18B-'
                
                format2kpu['TYPE'] = 'CL_UNORM_INT8'                                                              
                message2print = "{(Step 18C/70) set" + "; format2kpu-set-by-create2image4kpu to: " + str(format2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
                
                code2ack  = code2ack + '18C-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; format2kpu-failed-by-create2image4kpu: " + str(format2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
        
            task4imagecreate2kpu ['FORMAT'] = format2kpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) DEPTH - MUST BE 0                               
            # ---------------------------------------------------------------------------------------------             
            depth2kpu = 0
            message2print = "{(Step 19A/70) set" + "; depth2kpu-set-by-create2image4kpu to: " + str(depth2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #  
            code2ack  = code2ack + '19A-' 
            task4imagecreate2kpu ['DEPTH'] = depth2kpu
                                                                                                                                       
        else:
            pass                                                                                                    # PARAMETER DATA RECEIVED                       
    except: 
        message2print = "{(Step 1Z1/70) yielded" + "; task4imagecreate2kpu-yielded-by-create2image4kpu: " + str(task4imagecreate2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '1Z1-' 

    # ---------------------------------------------------------------------------------------------
    # TASK EITHER GENERATED BY THIS PROGRAM (NONE GIVEN) OR GIVEN BY USER    
    # ---------------------------------------------------------------------------------------------
    keys2task = task4imagecreate2kpu.keys()        
    message2print = "{(Step 19A/70) using" + "; keys2task-using-by-create2image4kpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # ['BUFFER',
    #  'CODE2ERROR',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'FLAGS',
    #  'FORMAT',
    #  'FUNCTION2CALLBACK',
    #  'HEIGHT',
    #  'PIPE',
    #  'PLATFORM',
    #  'POINTER',
    #  'QUEUE',
    #  'ROWPITCH',
    #  'SIZE',
    #  'WIDTH']; time: 00:53:55 10/05/11 Pacific Daylight Time}
  
    
    # ---------------------------------------------------------------------------------------------
    # (Step 20A/70) - CREATE 2D IMAGE FOR KPU! 
    # ---------------------------------------------------------------------------------------------    
    kpu2image4xi = {}                                                                                                                                                                               
    message2print = "{(Step 20A/70) initialized" + "; kpu2image4xi-initialized-by-create2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # {} 
    
    message2print = "{(Step 21A0/70) try" + "; data2callback-try-by-create2image4kpu: {}" + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)      
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA 
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['DATA2CALLBACK'] = task4imagecreate2kpu['DATA2CALLBACK']
        
        message2print = "{(Step 21A1/70) initialized" + "; data2callback-created-by-create2image4kpu to: " + str(kpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 05:56:13 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4kpu',
        #  'METHOD': 'create2image4kpu'}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['FUNCTION2CALLBACK'] = task4imagecreate2kpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) initialized" + "; function2callback-created-by-create2image4kpu to: " + str(kpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # <function trace2image4kpu at 0x0000000003B04348>  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform-try-by-create2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['PLATFORM'] = task4imagecreate2kpu['PLATFORM']
        
        message2print = "{(Step 22C1/70) initialized" + "; platform-created-by-create2image4kpu to: " + str(kpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 05:56:13 09/06/11 Pacific Daylight Time}
  
        code2ack  = code2ack + '22C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2kpu-try-by-create2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['CONTEXT'] = task4imagecreate2kpu['CONTEXT']
        
        message2print = "{(Step 23B/70) initialized" + "; context-created-by-create2image4kpu to: " + str(kpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2context4kpu',
        #  'METHOD': 'create2context4kpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x00000000036B0148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
        #  'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 05:56:13 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-create2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['DEVICE'] = task4imagecreate2kpu['DEVICE']
        
        message2print = "{(Step 24B/70) initialized" + "; device-created-by-create2image4kpu to: " + str(kpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0', 'EXTENSIONS': 'cl_amd_fp64 ...
        
        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['CODE2ERROR'] = task4imagecreate2kpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) initialized" + "; code2error-created-by-create2image4kpu to: " + str(kpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['QUEUE'] = task4imagecreate2kpu['QUEUE']
        
        message2print = "{(Step 26B/70) initialized" + "; queue-created-by-create2image4kpu to: " + str(kpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003ED99E8>  
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['PIPE'] = task4imagecreate2kpu['PIPE']
        
        message2print = "{(Step 27B/70) initialized" + "; pipe-created-by-create2image4kpu to: " + str(kpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 376>, <read-write PipeConnection, handle 380>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-kpu2image4xi-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-create2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['COMMANDQUEUE']  = task4imagecreate2kpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) initialized" + "; commandqueue-created-by-create2image4kpu to: " + str(kpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4kpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #  'CLASS': 'create2commandqueue4kpu',
        #  'METHOD': 'create2commandqueue4kpu'},
        #  'FUNCTION2CALLBACK': <function trace2commandqueue4kpu at 0x00000000037BC848>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
        #  'INFO2PRIVATE': None,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000432B3C8>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
              
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-create2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['BUFFER']  = task4imagecreate2kpu['BUFFER']  
 
        message2print = "{(Step 29B/70) initialized" + "; bufferinfo-created-by-create2image4kpu to: " + str(kpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2buffer4kpu',
        #  'METHOD': 'create2buffer4kpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x0000000003AFC348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003AAF2F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ...
        
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-create2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['EVENT']  = task4imagecreate2kpu['EVENT']  
 
        message2print = "{(Step 30B/70) initialized" + "; eventinfo-created-by-create2image4kpu to: " + str(kpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4kpu-failed ; time: 05:56:12 09/06/11 Pacific Daylight Time}',
        #                     'CLASS': 'create2event4kpu',
        #                     'METHOD': 'create2event4kpu'},
        #  'FUNCTION2CALLBACK': <function trace2event4kpu at 0x0000000003707D48>,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000039C3860>,
        #  'NACK2RETURN': ' ',
        #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['FLAGS']  = task4imagecreate2kpu['FLAGS']  
 
        message2print = "{(Step 31B/70) initialized" + "; flags-created-by-create2image4kpu to: " + str(kpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        kpu2image4xi['WIDTH']  = task4imagecreate2kpu['WIDTH']  
 
        message2print = "{(Step 34H/70) initialized" + "; width-created-by-create2image4kpu to: " + str(kpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        kpu2image4xi['HEIGHT']  = task4imagecreate2kpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) initialized" + "; height-created-by-create2image4kpu to: " + str(kpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800 
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['ROWPITCH']  = task4imagecreate2kpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) initialized" + "; rowpitch-created-by-create2image4kpu to: " + str(kpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 800
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-create2image4kpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['POINTER']  = task4imagecreate2kpu['POINTER']  
 
        message2print = "{(Step 37B/70) initialized" + "; pointer-created-by-create2image4kpu to: " + str(kpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <mmap.mmap object at 0x0000000003C050E0>
         
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['FORMAT']  = task4imagecreate2kpu['FORMAT']  
 
        message2print = "{(Step 38B/70) initialized" + "; format-created-by-create2image4kpu to: " + str(kpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '38Z1-'

    message2print = "{(Step 39A/70) try" + "; depth-try-by-create2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (39) DEPTH - MUST BE 0                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['DEPTH']  = task4imagecreate2kpu['DEPTH']  
 
        message2print = "{(Step 39B/70) initialized" + "; depth-created-by-create2image4kpu to: " + str(kpu2image4xi['DEPTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '39B-'
    except: 
        message2print = "{(Step 39Z1/70) yielded" + "; depth-yielded-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '39Z1-'
                                                                                                               
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    kpu2image4xi ['ACK2RETURN']  = code2ack
    kpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; kpu2image4xi-returning-by-create2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    # 'DEPTH'
    # 'WIDTH'
    # 'HEIGHT'
    return kpu2image4xi
            
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "set2image4kpu" usage = "
#    
#   print('(Step 4B/33) OF KPU2IMAGE4XI5TEST - TRYING TO GET PLATFORMIDS4XI', time4xi())
#   platform2xi = {}
#   try:
#       platform2xi = get2platformids4xi()
#       print('(Step 4B/33) of CONTEXT4KPU5TEST - GOT PLATFORMIDS4XI', platform2xi) 
#   except:
#       print('(Step 4Z/33) of CONTEXT4KPU5TEST *** FAILED TO GET PLATFORMIDS4XI', time4xi())     
#   
#   print('(Step 2A/33) OF DEVICE4KPU5TEST - TRYING TO INITIALIZE DEVICE4KPU ', time4xi())   
#   print('=' *33)        
#   try:
#       device2kpu = init2kpu4xi()                                                                                
#       print('(Step 2B/33) OF DEVICE4KPU5TEST - INITIALIZED DEVICE4KPU TO: ', device2kpu)        
#   except:
#       print('(Step 2Z/33) OF DEVICE4KPU5TEST *** FAILED TO INITIALIZE DEVICE4KPU', time4xi()) 
#   print('(Step 3A/33) OF KPU2IMAGE4XI5TEST - TRYING TO CREATE CONTEXT4KPU ', time4xi())    
#   print('=' *33)        
#   try:
#       context2kpu = create2context4kpu()                                                                               
#       print('(Step 3B/33) OF KPU2IMAGE4XI5TEST - CREATED CONTEXT4KPU: ', context2kpu)                                  
#   except:
#       print('(Step 3Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO CREATE CONTEXT4KPU', time4xi()) 
#          
#   task4set2image = {}
#   task4set2image  ['CONTEXT'] = context2kpu  
# 
#   ... other data for set operation ...
#    
#        
#   response4set2Dimage = {} 
#   print('(Step 11A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SET 2D IMAGE4KPU ', time4xi())   
#   try:                               
#       response4set2Dimage = set2image4kpu(task4set2image)                                                                                                         
#                                                                                     
#       print('(Step 11B/700) OF KPU2IMAGE4XI5TEST - SET 2D IMAGE4KPU ', time4xi())
#       keys2response = response4set2Dimage.keys()
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4set2Dimage[key2response])
#           # 
#   except:
#       print('(Step 11Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SET 2D IMAGE4KPU *** ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def set2image4kpu(task4imageset2kpu=None):

    '''
    sets 2D images for KPU
    '''
    global kpu2image4xi  
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "set2image4kpu"
    method2print = "set2image4kpu"

    message2print = "{(Step 1A/33) started" + "; set2image4kpu-started: " + str(set2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function set2image4kpu at 0x0000000003A62848> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageset2kpu-to-set2image4kpu: " + str(task4imageset2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'CODE2ERROR': 0,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003D4A710>,
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4kpu',
    #              'METHOD': 'create2context4kpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4kpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #  'COMMANDQUEUE': {'CODE2ERROR': 0,
    #              'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4kpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2commandqueue4kpu',
    #              'METHOD': 'create2commandqueue4kpu'},
    #  'FUNCTION2CALLBACK': <function trace2commandqueue4kpu at 0x00000000037BC848>,
    #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-8A-9B-10B-9B-10B-11B-12B-13B-14B-15B-16B-17B-18B-',
    #  'INFO2PRIVATE': None,
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003EDA780>,
    #  'NACK2RETURN': ' ',
    #  'CONTEXT': {'COUNT2REFERENCE': 0,
    #              'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
    #              'CLASS': 'create2context4kpu',
    #              'METHOD': 'create2context4kpu'},
    #              'FUNCTION2CALLBACK': <function trace2context4kpu at 0x0000000003770148>,
    #              'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
    #              'INFO2PRIVATE': None,
    #              'PLATFORM': {'list2platforms': ['DragonXi'],
    #                           'ack2return': ' 2C-2E-2F-2G-',
    #                           'nack2return': ' ', 'available2platform': 1, 'max2entries': 1},
    #              'NACK2RETURN': ' ',
    #              'SIZE2PRIVATE': 0,
    #              'PROPERTIES': None},
    #              'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318,
    #                          'ID2PLATFORM': '0000000001509568',
    #  'PROPERTIES': None,
    #  'PIPE': (<read-write PipeConnection, handle 552>, <read-write PipeConnection, handle 556>)},
    #  'DEVICE': {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', 'EXECUTE2KERNELS4OPENCL': 'ACK',
    #              'NATIVE2WIDTHLONG4VECTOR': 2, 'MAX2UNITS4COMPUTE': 4, 'SIZE2MEMORY4GLOBAL': 3221225472, 'NATIVE2WIDTHFLOAT4VECTOR': 4, 'MAX2SIZE4WORKGROUP': 1024, 'PREFERRED2WIDTHCHAR4VECTOR': 16, 'SIZE2CACHE': 65536, 
    #              'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 'TYPE4CACHE': 'Read/Write', 'PREFERRED2WIDTHINT4VECTOR': 4, 'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 'NATIVE2WIDTHINT4VECTOR': 4, 
    #              'SIZE2BUFFER4CONSTANT': 65536, 'EXECUTE2FUNCTION4NATIVE': 'ACK', 'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 'MAX2NUMBER4CONSTANTARGS': 8, 'VERSION2DRIVER': '2.0', 
            
    code2ack  = ' '
    code2nack = ' '
    # -----------------------------------------------------------------------------------------
    # CREATE IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                              # INITIALIZE IMAGE FOR KPU 
            message2print = "{(Step 1C/70) set" + "; kpu2image4xi-set-by-set2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4kpu-skipped-by-set2image4kpu: " + str(create2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function create2image4kpu at 0x0000000003A627C8>;                                  
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 1E/70) set" + "; kpu2image4xi-set-by-set2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
                
    # ---------------------------------------------------------------------------------------------
    # SET            
    # ---------------------------------------------------------------------------------------------     
    message2print = "{(Step 22A0/70) try" + "; data2callback-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    #     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 21A/70) - DATA2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['DATA2CALLBACK'] = task4imageset2kpu['DATA2CALLBACK']
                                        
        message2print = "{(Step 21A1/70) set" + "; data2callback-set-by-set2image4kpu to: " + str(kpu2image4xi['DATA2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        
        code2ack  = code2ack + '21A1-'
    except: 
        message2print = "{(Step 21Z1/70) yielded" + "; data2callback-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '21Z1-'
    
    message2print = "{(Step 22B0/70) try" + "; function2callback-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22B/70) - FUNCTION2CALLBACK - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['FUNCTION2CALLBACK'] = task4imageset2kpu['FUNCTION2CALLBACK']
        
        message2print = "{(Step 22B1/70) set" + "; function2callback-set-by-set2image4kpu to: " + str(kpu2image4xi['FUNCTION2CALLBACK'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #  
        code2ack  = code2ack + '22B1-'
    except: 
        message2print = "{(Step 22Z2/70) yielded" + "; function2callback-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '22Z2-' 
    
    message2print = "{(Step 22C0/70) try" + "; platform2callback-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    # 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 22C/70) - PLATFORM - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['PLATFORM'] = task4imageset2kpu['PLATFORM']
        
        message2print = "{(Step 2C1/70) set" + "; platform-set-by-set2image4kpu to: " + str(kpu2image4xi['PLATFORM'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # yielded  
        code2ack  = code2ack + '2C1-'
    except: 
        message2print = "{(Step 22Z3/70) yielded" + "; platform-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '22Z3-' 
        
        
      
    message2print = "{(Step 23A/70) try" + "; context2kpu-try-by-set2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)       
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 23/70) - CONTEXT - REQUIRED
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['CONTEXT'] = task4imageset2kpu['CONTEXT']
        
        message2print = "{(Step 23B/70) set" + "; context-set-by-set2image4kpu to: " + str(kpu2image4xi['CONTEXT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'COUNT2REFERENCE': 0,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 07:13:59 09/06/11 Pacific Daylight Time}',
        #                    'CLASS': 'create2context4kpu',
        #                    'METHOD': 'create2context4kpu'},
        #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x0000000003770148>,
        #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-',
        #  'INFO2PRIVATE': None,
        #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 'NACK2RETURN': ' ',
        #  'SIZE2PRIVATE': 0,
        #  'PROPERTIES': None}; time: 07:13:59 09/06/11 Pacific Daylight Time} 

        
        code2ack  = code2ack + '23B-'
    except: 
        message2print = "{(Step 23Z1/70) yielded" + "; context-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
    
        code2nack  = code2nack + '23Z1-' 
        
    message2print = "{(Step 24A/70) try" + "; device-try-by-set2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 24/70) - DEVICE - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['DEVICE'] = task4imageset2kpu['DEVICE']
        
        message2print = "{(Step 24B/70) set" + "; device-set-by-set2image4kpu to: " + str(kpu2image4xi['DEVICE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'RESOLUTION2TIMER4PROFILING': 318,
        #  'ID2PLATFORM': '0000000001509568',
        #  'PROFILING2QUEUE': 'ACK',
        #  'EXECUTE2KERNELS4OPENCL': 'ACK',
        #  'NATIVE2WIDTHLONG4VECTOR': 2,
        #  'MAX2UNITS4COMPUTE': 4,
        #  'SIZE2MEMORY4GLOBAL': 3221225472,
        #  'NATIVE2WIDTHFLOAT4VECTOR': 4,
        #  'MAX2SIZE4WORKGROUP': 1024,
        #  'PREFERRED2WIDTHCHAR4VECTOR': 16,
        #  'SIZE2CACHE': 65536,
        #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK',
        #  'TYPE4CACHE': 'Read/Write',
        #  'PREFERRED2WIDTHINT4VECTOR': 4,
        #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1,
        #  'NATIVE2WIDTHINT4VECTOR': 4,
        #  'SIZE2BUFFER4CONSTANT': 65536,
        #  'EXECUTE2FUNCTION4NATIVE': 'ACK',
        #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0,
        #  'MAX2NUMBER4CONSTANTARGS': 8,
        #  'VERSION2DRIVER': '2.0',
        #  'EXTENSIONS': 'cl_amd_fp64 cl_khr_global_int32_base_atomics ...
        #  'AVAILABLE2COMPILER': 'ACK',
        #  'MAX2DIMENSIONS4WORKITEMS': 3,
        #  'SUPPORT2CORRECTION4ERROR': 0,
        #  'BITS2ADDRESS': 64,
        #  'AVAILABLE': 'ACK',
        #  'SIZE2MEMORY4LOCAL': 32768,
        #  'VENDOR': 'AuthenticAMD',
        #  'SUPPORT2IMAGE': 'NACK',
        #  'MAX2SIZE4KERNELARGUMENT': 4096,
        #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-3A-4A-5A-6A-7A-8A-9A-10A-11A-...
        #  'OUT-OF-ORDER2QUEUE': 'NACK',
        #  'ALIGNMENT2ADDRESS4BASE': 1024,
        #  'PROFILE': 'FULL_PROFILE',
        #  'TYPE2MEMORY4LOCAL': 'Global',
        #  'PREFERRED2WIDTHFLOAT4VECTOR': 4,
        #  'PREFERRED2WIDTHLONG4VECTOR': 2,
        #  'MAX2FREQUENCY4CLOCK': 3210,
        #  'PREFERRED2WIDTHSHORT4VECTOR': 8,
        #  'ID': 4033,
        #  'MAX2WORKITEM2': 1024,
        #  'NATIVE2WIDTHDOUBLE4VECTOR': 0,
        #  'ROUND2ZERO': 'ACK',
        #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, AuthenticAMD',
        #  'MAX2WORKITEM1': 1024,
        #  'MAX2WORKITEM0': 1024,
        #  'SIZE2LINE4CACHE': 64,
        #  'MAX2ALLOCATION4MEMORY': 1073741824,
        #  'NATIVE2WIDTHCHAR4VECTOR': 16,
        #  'QUIET2NANS': 'ACK',
        #  'NATIVE2WIDTHSHORT4VECTOR': 8,
        #  'ROUND2NEAREST4EVEN': 'ACK',
        #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)',
        #  'NACK2RETURN': ' ',
        #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1,
        #  'ENDIANESS2DEVICE': 'Little',
        #  'MINIMUM2ALIGNMENT4DATATYPE': 128,
        #  'ROUND2+VE3INFINITY': 'ACK',
        #  'TYPE': 'CL_DEVICE_TYPE_KPU',
        #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 07:13:59 09/06/11 Pacific Daylight Time}

        
        code2ack  = code2ack + '24B-'
    except: 
        message2print = "{(Step 24Z1/70) yielded" + "; device-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '24Z1-'
                 
    message2print = "{(Step 25A/70) try" + "; code2error-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 25/70) - ERROR CODE - REQUIRED
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['CODE2ERROR'] = task4imageset2kpu['CODE2ERROR']
        
        message2print = "{(Step 25B/70) set" + "; code2error-set-by-set2image4kpu to: " + str(kpu2image4xi['CODE2ERROR'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0 
        code2ack  = code2ack + '25B-'
    except: 
        message2print = "{(Step 25Z1/70) yielded" + "; code2error-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '25Z1-'                
                
          
    message2print = "{(Step 26A/70) try" + "; queue-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 26/70) - QUEUE - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['QUEUE'] = task4imageset2kpu['QUEUE']
        
        message2print = "{(Step 26B/70) set" + "; queue-set-by-set2image4kpu to: " + str(kpu2image4xi['QUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <queue4xi.Queue4Xi object at 0x0000000003D4A710> 
        code2ack  = code2ack + '26B-'
    except: 
        message2print = "{(Step 26Z1/70) yielded" + "; queue-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '26Z1-' 
                                
    message2print = "{(Step 27A/70) try" + "; pipe-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                     
    try:
        # -----------------------------------------------------------------------------------------
        # (Step 27/70) - PIPE - EXTRA
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['PIPE'] = task4imageset2kpu['PIPE']
        
        message2print = "{(Step 27B/70) set" + "; pipe-set-by-set2image4kpu to: " + str(kpu2image4xi['PIPE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>)  
        
        code2ack  = code2ack + '27B-'
    except: 
        message2print = "{(Step 27Z1/70) yielded" + "; pipe-kpu2image4xi-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '27Z1-'   

    message2print = "{(Step 28A/70) try" + "; commandqueue-try-by-set2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['COMMANDQUEUE']  = task4imageset2kpu['COMMANDQUEUE']  
 
        message2print = "{(Step 28B/70) set" + "; commandqueue-set-by-set2image4kpu to: " + str(kpu2image4xi['COMMANDQUEUE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
        
        code2ack  = code2ack + '28B-'
    except: 
        message2print = "{(Step 28Z1/70) yielded" + "; commandqueue-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '28Z1-'    

    message2print = "{(Step 29A/70) try" + "; bufferinfo-try-by-set2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (29) BUFFER INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['BUFFER']  = task4imageset2kpu['BUFFER']  
 
        message2print = "{(Step 29B/70) set" + "; bufferinfo-set-by-set2image4kpu to: " + str(kpu2image4xi['BUFFER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '29B-'
    except: 
        message2print = "{(Step 29Z1/70) yielded" + "; bufferinfo-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        code2nack  = code2nack + '29Z1-'            

    message2print = "{(Step 30A/70) try" + "; eventinfo-try-by-set2image4kpu: {}" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (30) EVENT INFO - EXTRA                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['EVENT']  = task4imageset2kpu['EVENT']  
 
        message2print = "{(Step 30B/70) set" + "; eventinfo-set-by-set2image4kpu to: " + str(kpu2image4xi['EVENT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '30B-'
    except: 
        message2print = "{(Step 30Z1/70) yielded" + "; eventinfo-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '30Z1-'            
                

    message2print = "{(Step 31A/70) try" + "; flags-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (31) FLAGS - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['FLAGS']  = task4imageset2kpu['FLAGS']  
 
        message2print = "{(Step 31B/70) set" + "; flags-set-by-set2image4kpu to: " + str(kpu2image4xi['FLAGS'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '31B-'
    except: 
        message2print = "{(Step 31Z1/70) yielded" + "; flags-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded
        
        code2nack  = code2nack + '31Z1-'     

    message2print = "{(Step 34A/70) try" + "; width-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (34) WIDTH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        kpu2image4xi['WIDTH']  = task4imageset2kpu['WIDTH']  
 
        message2print = "{(Step 34H/70) set" + "; width-set-by-set2image4kpu to: " + str(kpu2image4xi['WIDTH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '34H-'
    except: 
        message2print = "{(Step 34Z1/70) yielded" + "; width-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        code2nack  = code2nack + '34Z1-'   


    message2print = "{(Step 35A/70) try" + "; height-try-by-set2image4kpu: None"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (35) HEIGHT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------      
        kpu2image4xi['HEIGHT']  = task4imageset2kpu['HEIGHT']  
 
        message2print = "{(Step 35H/70) set" + "; height-set-by-set2image4kpu to: " + str(kpu2image4xi['HEIGHT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        #  
        code2ack  = code2ack + '35H-'
    except: 
        message2print = "{(Step 35Z1/70) yielded" + "; height-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '35Z1-'   
 

    message2print = "{(Step 36A/70) try" + "; rowpitch-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (36) ROWPITCH - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['ROWPITCH']  = task4imageset2kpu['ROWPITCH']  
 
        message2print = "{(Step 36B/70) set" + "; rowpitch-set-by-set2image4kpu to: " + str(kpu2image4xi['ROWPITCH'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '36B-'
    except: 
        message2print = "{(Step 36Z1/70) yielded" + "; rowpitch-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '36Z1-'

    message2print = "{(Step 37A/70) try" + "; pointer-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (37) POINTER - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['POINTER']  = task4imageset2kpu['POINTER']  
 
        message2print = "{(Step 37B/70) set" + "; pointer-set-by-set2image4kpu to: " + str(kpu2image4xi['POINTER'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '37B-'
    except: 
        message2print = "{(Step 37Z1/70) yielded" + "; pointer-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '37Z1-'

    message2print = "{(Step 38A/70) try" + "; format-try-by-set2image4kpu: None" + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)        
    try:            
        # ---------------------------------------------------------------------------------------------
        # (38) FORMAT - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
                        
        kpu2image4xi['FORMAT']  = task4imageset2kpu['FORMAT']  
 
        message2print = "{(Step 38B/70) set" + "; format-set-by-set2image4kpu to: " + str(kpu2image4xi['FORMAT'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        code2ack  = code2ack + '38B-'
    except: 
        message2print = "{(Step 38Z1/70) yielded" + "; format-yielded-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # yielded 
        code2nack  = code2nack + '38Z1-'
                        
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    kpu2image4xi ['ACK2RETURN']  = code2ack
    kpu2image4xi ['NACK2RETURN'] = code2nack 
    
    message2print = "{(Step 70/70) returning" + "; kpu2image4xi-returning-by-set2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-', 'ROWPITCH': 800, 'DATA2CALLBACK': ...
    
    
    return kpu2image4xi

           
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2image4kpu" usage = "
#                                                                                                              
#   # -----------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4KPU                      
#   # -----------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4KPU ', time4xi())    
#   response4copy2image = {} 
#   task4copy2image = None
#   try:                               
#       response4copy2image = copy2image4kpu(task4copy2image)                                                                                                          
#       print('(Step 16B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGE4KPU : ', response4copy2image)
#       queue2copy4kpu      = response4copy2image['QUEUE']
#       channel2copypublish4kpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copyconsume4kpu = response4copy2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGE4KPU ', time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4KPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4KPU ', time4xi())     
#   try:                               
#       message2copyimagesend4kpu = "<copy4xi>copy2copy4kpu</copy4xi>"
#       channel2copypublish4kpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4kpu.send(message2copyimagesend4kpu)
#       print("(Step 17B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2copypublish4kpu, " MESSAGE2SEND4KPU: " , message2copyimagesend4kpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4KPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4KPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2copyimage4kpu(queue2copy4kpu, channel2copyconsume4kpu, channel2copypublish4kpu)
#       print("(Step 18D/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2copyimage4kpu)
#   except:
#       print("(Step 18Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # -----------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4KPU ', time4xi())     
#   try:
#       channel2copyconsume4kpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4kpu = channel2copyconsume4kpu.recv()
#       print("(Step 19B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copyconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                  
#       # <roger4xi>roger2copy4kpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4KPU ', time4xi())         
#   # -----------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4KPU TO RECEIVE                
#   # -----------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4KPU ', time4xi())     
#   try:                               
#       message2copyimagesend4kpu = "<copy4xi>copy2copy4kpu</copy4xi>"
#       channel2copypublish4kpu = response4copy2image['PUBLISHERCHANNEL']
#       channel2copypublish4kpu.send(message2copyimagesend4kpu)
#       print("(Step 20B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2copypublish4kpu, " MESSAGE2SEND4KPU: " , message2copyimagesend4kpu)                                                                                                
#       # <copy4xi>copy2copy4kpu</copy4xi> 
#   except:
#        print('(Step 20Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4KPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4KPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2copyimage4kpu(queue2copy4kpu, channel2copyconsume4kpu, channel2copypublish4kpu)
#       print("(Step 21B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2copyimage4kpu)
#   except:
#       print("(Step 21Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGE4KPU ', time4xi())     
#   try:
#       channel2copyconsume4kpu = response4copy2image['CONSUMERCHANNEL']                              
#       message2copyimagereceive4kpu = channel2copyconsume4kpu.recv()
#       print("(Step 22B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copyconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                  
#       # <copy4xi>copy2copy4kpu</copy4xi> 
#   except:
#       print('(Step 22Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGE4KPU ', time4xi())          
#   # ----------------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # ----------------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copy4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # -----------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4KPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2copyimage4kpu(queue2copy4kpu, channel2copyconsume4kpu, channel2copypublish4kpu)
#       print("(Step 25B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2copyimage4kpu)
#   except:
#       print("(Step 25Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4kpu = channel2copyconsume4kpu.recv()
#       print("(Step 26D/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copyconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4kpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -----------------------------------------------------------------------------------------------
#   (27) LET MANOEUVRE2IMAGE4KPU GO TO STATE 0               
#   # -----------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2copyimage4kpu(queue2copy4kpu, channel2copyconsume4kpu, channel2copypublish4kpu)
#       print("(Step 27B/700) OF KPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4KPU ", response2manoeuvre4kpu)
#   except:
#       print("(Step 27Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2image4kpu(task4imagecopy2kpu=None):

    '''
    copies 2D images for KPU
    '''
    global kpu2image4xi                                                                                             # 2D-IMAGES FOR KPU  
    global control2copyimage4kpu                                                                                    # COMMAND AND CONTROL FOR KPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2image4kpu"
    method2print = "copy2image4kpu"

    message2print = "{(Step 0A/33) started" + "; copy2image4kpu-started: " + str(copy2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                               
    # <function copy2image4kpu at 0x00000000038538C8>
    
    message2print = "{(Step 0B/33) passed" + "; task4imagecopy2kpu-to-copy2image4kpu: " + str(task4imagecopy2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                       # INITIALIZE IMAGE FOR KPU 
            message2print = "{(Step 0C/70) created" + "; kpu2image4xi-created-by-copy2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        else:
            message2print = "{(Step 0D/70) skipped" + "; create2image4kpu-skipped-by-copy2image4kpu: " + str(copy2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # <function copy2image4kpu at 0x00000000038538C8>                                 
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 0E/70) set" + "; kpu2image4xi-set-by-copy2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2image4xi = {}
            message2print = "{(Step 0Z/70) yielded" + "; kpu2image4xi-yielded-by-copy2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '0Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagecopy2kpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagecopy2kpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagecopy2kpu-created-by-copy2image4kpu to: " + str(task4imagecopy2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {} 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                                 # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4kpu-created-by-copy2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4kpu at 0x000000000384E9C8> 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4kpu"                                                            # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4kpu"                                                            # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4kpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4kpu',
            #  'METHOD': 'copy2image4kpu'}; time: 07:01:44 10/04/11 Pacific Daylight Time}


            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2kpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagecopy2kpu-created-by-copy2image4kpu to: " + str(task4imagecopy2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4kpu at 0x000000000384E9C8>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2image4kpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
            #  'CLASS': 'copy2image4kpu',
            #  'METHOD': 'copy2image4kpu'}}; time: 07:01:44 10/04/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 07:01:44 10/04/11 Pacific Daylight Time}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagecopy2kpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2kpu-trying-by-copy2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2kpu-got-by-copy2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) ...
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2kpu-failed-by-copy2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagecopy2kpu ['CONTEXT'] = context2kpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2kpu-trying-by-copy2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2kpu-got-by-copy2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK', ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2kpu-failed-by-copy2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagecopy2kpu ['DEVICE'] = device2kpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE SOURCE OBJECT - REQUIRED                         
            # ---------------------------------------------------------------------------------------------         
            imagesourceinfo2kpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2kpu-trying-by-copy2image4kpu: " + str(imagesourceinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagesourceinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imagesourceinfo2kpu-got-by-copy2image4kpu: " + str(imagesourceinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2kpu-failed-by-copy2image4kpu: " + str(imagesourceinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagecopy2kpu ['IMAGESOURCE'] = imagesourceinfo2kpu         
                   
            # ---------------------------------------------------------------------------------------------
            # (6) IMAGE DESTINATION OBJECT - REQUIRED                        
            # ---------------------------------------------------------------------------------------------                         
            imagedestinationinfo2kpu = {}
             
            message2print = "{(Step 6A/70) trying" + "; imagedestinationinfo2kpu-trying-by-copy2image4kpu: " + str(imagedestinationinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imagedestinationinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 6B/70) got" + "; imagedestinationinfo2kpu-got-by-copy2image4kpu: " + str(imagedestinationinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004472BE0>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                # note: source and destination will be the same !             
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; imagedestinationinfo2kpu-failed-by-copy2image4kpu: " + str(imagedestinationinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'          
                      
            task4imagecopy2kpu ['IMAGEDESTINATION'] = imagedestinationinfo2kpu     
            
            # ---------------------------------------------------------------------------------------------
            # (7) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 7A/70) trying" + "; queue2kpu-trying-by-copy2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 7B/70) got" + "; queue2kpu-got-by-copy2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x000000000453EB38>
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; queue2kpu-failed-by-copy2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagecopy2kpu ['QUEUE'] = queue2kpu

            
            # ---------------------------------------------------------------------------------------------
            # (8) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 8A/70) trying" + "; pipe2kpu-trying-by-copy2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 8B/70) got" + "; pipe2kpu-got-by-copy2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 612>, <read-write PipeConnection, handle 616>)
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; pipe2kpu-failed-by-copy2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
                
            task4imagecopy2kpu ['PIPE'] = pipe2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (9) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2kpu-trying-by-copy2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 9B/70) got" + "; commandqueueinfo2kpu-got-by-copy2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4kpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4kpu', 
                #  'METHOD': 'create2commandqueue4kpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4kpu at 0x00000000037EDA48> ...
                
                code2ack  = code2ack + '9B-'  
            except:
                message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2kpu-failed-by-copy2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagecopy2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (10) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; bufferinfo2kpu-trying-by-copy2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                               # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 10B/70) got" + "; bufferinfo2kpu-got-by-copy2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE' ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; bufferinfo2kpu-failed-by-copy2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagecopy2kpu ['BUFFER'] = bufferinfo2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventinfo2kpu-trying-by-copy2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 11B/70) got" + "; eventinfo2kpu-got-by-copy2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0,
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4kpu-failed ; time: 07:01:44 10/04/11 Pacific Daylight Time}',
                #  'CLASS': 'create2event4kpu',
                #  'METHOD': 'create2event4kpu'},
                #  'FUNCTION2CALLBACK': <function trace2event4kpu at 0x0000000003803F48>,
                #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-6B-7B-8B-9B-10B-11B-12B-14B-15B-16B-',
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000423B668>,
                #  'NACK2RETURN': ' ',
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventinfo2kpu-failed-by-copy2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagecopy2kpu ['EVENT'] = eventinfo2kpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlist2kpu-trying-by-copy2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlist2kpu-set-by-copy2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2kpu-failed-by-copy2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagecopy2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (13) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
            
            message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-copy2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1    
                
                message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2kpu-set-by-copy2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0  
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-copy2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagecopy2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN FOR SOURCE - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin4source2kpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin4source2kpu-trying-by-copy2image4kpu: " + str(origin4source2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4source2kpu['X'] = 0 
                origin4source2kpu['Y'] = 0  
                origin4source2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin4source2kpu-set-by-copy2image4kpu to: " + str(origin4source2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin4source2kpu-failed-by-copy2image4kpu: " + str(origin4source2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagecopy2kpu ['SOURCEORIGIN'] = origin4source2kpu
            
            # ---------------------------------------------------------------------------------------------
            # (15) ORIGIN FOR DESTINATION - REQUIRED                             
            # ---------------------------------------------------------------------------------------------              
            origin4destination2kpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; origin4destination2kpu-trying-by-copy2image4kpu: " + str(origin4destination2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin4destination2kpu['X'] = 100 
                origin4destination2kpu['Y'] = 100  
                origin4destination2kpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; origin4destination2kpu-set-by-copy2image4kpu to: " + str(origin4destination2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; origin4destination2kpu-failed-by-copy2image4kpu: " + str(origin4destination2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagecopy2kpu ['DESTINATIONORIGIN'] = origin4destination2kpu
                        
            
            
            # --------------------------------------------------------------------------------------------------------
            # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
            # --------------------------------------------------------------------------------------------------------             
            region4copy2kpu = {}                                                                                                         
           
            message2print = "{(Step 16A/70) trying" + "; region4copy2kpu-trying-by-copy2image4kpu: " + str(region4copy2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                region4copy2kpu['X'] = 10 
                region4copy2kpu['Y'] = 10  
                region4copy2kpu['Z'] = 10                                                                           
                message2print = "{(Step 16B/70) set" + "; region4copy2kpu-set-by-copy2image4kpu to: " + str(region4copy2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 

                code2ack  = code2ack + '16B-'                
                 
            except:
                message2print = "{(Step 16Z/70) failed" + "; region4copy2kpu-failed-by-copy2image4kpu: " + str(region4copy2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '16Z-'
            
            task4imagecopy2kpu ['REGION'] = region4copy2kpu              
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2kpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-copy2image4kpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:21:58 10/05/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                            # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4kpu-created-by-copy2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                 # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2image4kpu"                                                        # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2image4kpu"                                                        # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    
            
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagecopy2kpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagecopy2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
                
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagecopy2kpu-created-by-copy2image4kpu to: " + str(task4imagecopy2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
                 
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagecopy2kpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2kpu-trying-by-copy2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2kpu-got-by-copy2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2kpu-failed-by-copy2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagecopy2kpu ['CONTEXT'] = context2kpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2kpu-trying-by-copy2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2kpu-got-by-copy2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2kpu-failed-by-copy2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagecopy2kpu ['DEVICE'] = device2kpu
        
        if 'IMAGESOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) SOURCE IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourceimageinfo2kpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2kpu-trying-by-copy2image4kpu: " + str(sourceimageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourceimageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; sourceimageinfo2kpu-got-by-copy2image4kpu: " + str(sourceimageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2kpu-failed-by-copy2image4kpu: " + str(sourceimageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagecopy2kpu ['IMAGESOURCE'] = sourceimageinfo2kpu         
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2kpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; destinationimageinfo2kpu-trying-by-copy2image4kpu: " + str(destinationimageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; destinationimageinfo2kpu-got-by-copy2image4kpu: " + str(destinationimageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; destinationimageinfo2kpu-failed-by-copy2image4kpu: " + str(destinationimageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4imagecopy2kpu ['IMAGEDESTINATION'] = destinationimageinfo2kpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2kpu-trying-by-copy2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2kpu-got-by-copy2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2kpu-failed-by-copy2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagecopy2kpu ['QUEUE'] = queue2kpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2kpu-trying-by-copy2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2kpu-got-by-copy2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2kpu-failed-by-copy2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4imagecopy2kpu ['PIPE'] = pipe2kpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2kpu-trying-by-copy2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2kpu-got-by-copy2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2kpu-failed-by-copy2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagecopy2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; bufferinfo2kpu-trying-by-copy2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                           # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 30B/70) got" + "; bufferinfo2kpu-got-by-copy2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; bufferinfo2kpu-failed-by-copy2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagecopy2kpu ['BUFFER'] = bufferinfo2kpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2kpu-trying-by-copy2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2kpu-got-by-copy2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2kpu-failed-by-copy2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagecopy2kpu ['EVENT'] = eventinfo2kpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2kpu-trying-by-copy2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2kpu-set-by-copy2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2kpu-failed-by-copy2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagecopy2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------
            eventwaitlistquantity2kpu = 0
            
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-copy2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1    
                
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2kpu-set-by-copy2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-copy2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
                    
            task4imagecopy2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                             
                    
        if 'SOURCEORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) SOURCEORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            source4origin2kpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; source4origin2kpu-trying-by-copy2image4kpu: " + str(source4origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                source4origin2kpu['X'] = 0 
                source4origin2kpu['Y'] = 0  
                source4origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; source4origin2kpu-set-by-copy2image4kpu to: " + str(source4origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; source4origin2kpu-failed-by-copy2image4kpu: " + str(source4origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagecopy2kpu ['SOURCEORIGIN'] = source4origin2kpu
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (35) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2kpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; destination4origin2kpu-trying-by-copy2image4kpu: " + str(destination4origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2kpu['X'] = 100 
                destination4origin2kpu['Y'] = 100  
                destination4origin2kpu['Z'] = 100                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; destination4origin2kpu-set-by-copy2image4kpu to: " + str(destination4origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; destination4origin2kpu-failed-by-copy2image4kpu: " + str(destination4origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagecopy2kpu ['DESTINATIONSOURCEORIGIN'] = destination4origin2kpu                
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2kpu-trying-by-copy2image4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2kpu-set-by-copy2image4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2kpu-failed-by-copy2image4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4imagecopy2kpu ['REGION'] = region2kpu
        
        # ---------------------------------------------------------------------------------------------
        #  EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagecopy2kpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2image4kpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'IMAGESOURCE',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagecopy2kpu-yielded-by-copy2image4kpu: " + str(task4imagecopy2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimage4kpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2kpu = False
    channels2kpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2kpu-trying-by-copy2image4kpu: " + str(channels2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2kpu, publisherchannel2kpu = task4imagecopy2kpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2kpu-got-by-copy2image4kpu: " + str(consumerchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 612> 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2kpu-got-by-copy2image4kpu: " + str(publisherchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 616>
                              
        ack4channels2kpu = True
        
        control2copyimage4kpu['CONSUMERCHANNEL']  = consumerchannel2kpu
        control2copyimage4kpu['PUBLISHERCHANNEL'] = publisherchannel2kpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2kpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2kpu = False
    queue2kpu = None
    message2print = "{(Step 41A/70) trying" + "queue2kpu-trying-by-copy2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2kpu = task4imagecopy2kpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2kpu-got-by-copy2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004721CF8>
              
        ack4queue2kpu = True
        control2copyimage4kpu['QUEUE']  = queue2kpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2kpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4kpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4kpu-trying-by-copy2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4kpu = DummyProcess4Xi(target=manoeuvre2copyimage4kpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2kpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2kpu, publisherchannel2kpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4kpu-created-by-copy2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2image4kpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2image4kpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimage4kpu-returning-by-copy2image4kpu: " + str(control2copyimage4kpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004721CF8>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 616>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 612>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
   
    return control2copyimage4kpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2imagetobuffer4kpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (60) ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4KPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 60A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4KPU ', time4xi())    
#   response4copy2imagetobuffer = {} 
#   task4copy2imagetobuffer = None
#   try:                               
#       response4copy2imagetobuffer = copy2imagetobuffer4kpu(task4copy2imagetobuffer)                                                                                                          
#       print('(Step 60B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4KPU : ', response4copy2imagetobuffer)
#       queue2copyimagetobuffer4kpu      = response4copy2imagetobuffer['QUEUE']
#       channel2copyimagetobufferpublish4kpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferconsume4kpu = response4copy2imagetobuffer['CONSUMERCHANNEL']
#   except:
#       print('(Step 60Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPY2IMAGETOBUFFER4KPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (61) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 61A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4KPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4kpu = "<copy4xi>copy2copyimagetobuffer4kpu</copy4xi>"
#       channel2copyimagetobufferpublish4kpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4kpu.send(message2copyimagetobuffersend4kpu)
#       print("(Step 61B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2copyimagetobufferpublish4kpu, " MESSAGE2SEND4KPU: " , message2copyimagetobuffersend4kpu)                                                                                                
#   except:
#       print('(Step 61Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4KPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (62) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 62A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copyimagetobuffer4kpu(queue2copyimagetobuffer4kpu, channel2copyimagetobufferconsume4kpu, channel2copyimagetobufferpublish4kpu)
#       print("(Step 62B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", manoeuvre2copyimagetobuffer4kpu)
#   except:
#       print("(Step 62Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (63) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 63A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4KPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4kpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4kpu = channel2copyimagetobufferconsume4kpu.recv()
#       print("(Step 63B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copyimagetobufferconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                  
#       # 
#   except:
#       print('(Step 63Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4KPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (64) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 64A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4KPU ', time4xi())     
#   try:                               
#       message2copyimagetobuffersend4kpu = "<copy4xi>copy2copyimagetobuffer4kpu</copy4xi>"
#       channel2copyimagetobufferpublish4kpu = response4copy2imagetobuffer['PUBLISHERCHANNEL']
#       channel2copyimagetobufferpublish4kpu.send(message2copyimagetobuffersend4kpu)
#       print("(Step 64B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2copyimagetobufferpublish4kpu, " MESSAGE2SEND4KPU: " , message2copyimagetobuffersend4kpu)                                                                                                
#       #  
#   except:
#       print('(Step 64Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4KPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (65) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 65A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copyimagetobuffer4kpu(queue2copyimagetobuffer4kpu, channel2copyimagetobufferconsume4kpu, channel2copyimagetobufferpublish4kpu)
#       print("(Step 65B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", manoeuvre2copyimagetobuffer4kpu)
#   except:
#       print("(Step 65Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (66) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 66A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPY2IMAGETOBUFFER4KPU ', time4xi())     
#   try:
#       channel2copyimagetobufferconsume4kpu = response4copy2imagetobuffer['CONSUMERCHANNEL']                              
#       message2copyimagereceive4kpu = channel2copyimagetobufferconsume4kpu.recv()
#       print("(Step 66B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copyimagetobufferconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                  
#       # 
#   except:
#       print('(Step 66Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPY2IMAGETOBUFFER4KPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (67) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 67A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copyimagetobuffer4dragon.put(path2dragon)             
#       print("(Step 67B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 67Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (68) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 68A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copyimagetobuffer4kpu(queue2copyimagetobuffer4kpu, channel2copyimagetobufferconsume4kpu, channel2copyimagetobufferpublish4kpu)
#       print("(Step 68B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", manoeuvre2copyimagetobuffer4kpu)
#   except:
#       print("(Step 68Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (69) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 69A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4kpu = channel2copyimagetobufferconsume4kpu.recv()
#       print("(Step 69D/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copyimagetobufferconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                   
#   except:
#       print('(Step 69Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (70) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 70A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copyimagetobuffer4kpu(queue2copyimagetobuffer4kpu, channel2copyimagetobufferconsume4kpu, channel2copyimagetobufferpublish4kpu)
#       print("(Step 70B/700) OF KPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4KPU ", response2manoeuvreimagecopytobuffer4kpu)
#   except:
#       print("(Step 70Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2imagetobuffer4kpu(task4copyimagetobuffer2kpu=None):

    '''
    copies 2D images to buffer for KPU
    '''
    global kpu2image4xi                                                                                             # 2D-IMAGES FOR KPU  
    global kpu2buffer4xi                                                                                            # BUFFER FOR KPU
    
    global control2copyimagetobuffer4kpu                                                                            # COMMAND AND CONTROL FOR KPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2imagetobuffer4kpu"
    method2print = "copy2imagetobuffer4kpu"

    message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer4kpu-started: " + str(copy2imagetobuffer4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                 
    # <function copy2imagetobuffer4kpu at 0x000000000390B948>
    
    message2print = "{(Step 1B/33) passed" + "; task4copyimagetobuffer2kpu-to-copy2imagetobuffer4kpu: " + str(task4copyimagetobuffer2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                              # INITIALIZE IMAGE FOR KPU 
            message2print = "{(Step 1C/70) created" + "; kpu2image4xi-created-by-copy2imagetobuffer4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4kpu-skipped-by-copy2imagetobuffer4kpu: " + str(copy2imagetobuffer4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4kpu-skipped-by-copy2imagetobuffer4kpu                                 
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 1E/70) set" + "; kpu2image4xi-set-by-copy2imagetobuffer4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; kpu2image4xi-yielded-by-copy2imagetobuffer4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2buffer4xi is None:
            kpu2buffer4xi = create2buffer4kpu()                                                                     # INITIALIZE BUFFER FOR KPU 
            message2print = "{(Step 1F/70) created" + "; kpu2buffer4xi-created-by-copy2imagetobuffer4kpu to: " + str(kpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4kpu-skipped-by-copy2imagetobuffer4kpu: " + str(copy2imagetobuffer4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                  
    except:
        try:
            kpu2buffer4xi = create2buffer4kpu()
            message2print = "{(Step 1H/70) set" + "; kpu2buffer4xi-set-by-copy2imagetobuffer4kpu to: " + str(kpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0,
            #  'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}',
            #   'CLASS': 'create2buffer4kpu',
            #   'METHOD': 'create2buffer4kpu'},
            #   'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x0000000004782B48>,
            #   'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>,
            #   'FLAGS': 0,
            #   'CONTEXT': {'COUNT2REFERENCE': 0,
            #   'DATA2CALLBACK': ...
            
        except:
            kpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; kpu2buffer4xi-yielded-by-copy2imagetobuffer4kpu: " + str(kpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASK          
    # -----------------------------------------------------------------------------------------          
              
    if task4copyimagetobuffer2kpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
            
        task4copyimagetobuffer2kpu = {}                                                                                                                                                                                       
            
        message2print = "{(Step 1A/70) initialized" + "; task4copyimagetobuffer2kpu-created-by-copy2imagetobuffer4kpu to: " + str(task4copyimagetobuffer2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
            
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4kpu = trace2image4kpu                                                                 # CALLBACK FUNCTION FOR KPU
            
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4kpu-created-by-copy2imagetobuffer4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4kpu at 0x0000000004797548> 
            
        code2ack  = code2ack + '1B1-'
           
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4kpu"                                                            # MODULE OF KPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4kpu"                                                            # FUNCTION OF KPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4kpu-failed; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4kpu', 
        #  'METHOD': 'copy2imagetobuffer4kpu'}; time: 03:40:47 10/04/11 Pacific Daylight Time}


        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2kpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copyimagetobuffer2kpu-created-by-copy2imagetobuffer4kpu to: " + str(task4copyimagetobuffer2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4kpu at 0x0000000004797548>,
        #   'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2imagetobuffer4kpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2imagetobuffer4kpu', 
        #  'METHOD': 'copy2imagetobuffer4kpu'}}; time: 03:40:47 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copyimagetobuffer2kpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2kpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2kpu-trying-by-copy2imagetobuffer4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2kpu-got-by-copy2imagetobuffer4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4kpu', 
            #  'METHOD': 'create2context4kpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x000000000394F348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}; time: 03:40:47 10/04/11 Pacific Daylight Time}
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2kpu-failed-by-copy2imagetobuffer4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
            
        task4copyimagetobuffer2kpu ['CONTEXT'] = context2kpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2kpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2kpu-trying-by-copy2imagetobuffer4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2kpu-got-by-copy2imagetobuffer4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, 
            #  'NATIVE2WIDTHFLOAT4VECTOR': 4, 
            #  'MAX2SIZE4WORKGROUP': 1024, 
            #  'PREFERRED2WIDTHCHAR4VECTOR': 16, 
            #  'SIZE2CACHE': 65536, 
            #  'IEEE754-2008FUSED2MULTIPLY-ADD': 'NACK', 
            #  'TYPE4CACHE': 'Read/Write', 
            #  'PREFERRED2WIDTHINT4VECTOR': 4, 
            #  'SIZE2PREFERREDWORKGROUPMULTIPLE4KERNEL': 1, 
            #  'NATIVE2WIDTHINT4VECTOR': 4, 
            #  'SIZE2BUFFER4CONSTANT': 65536, 
            #  'EXECUTE2FUNCTION4NATIVE': 'ACK', 
            #  'PREFERRED2WIDTHDOUBLE4VECTOR': 0, 
            #  'MAX2NUMBER4CONSTANTARGS': 8, 
            #  'VERSION2DRIVER': '2.0', 
            #  'EXTENSIONS': 'cl_amd_fp64 ...
            #  'AVAILABLE2COMPILER': 'ACK', 
            #  'MAX2DIMENSIONS4WORKITEMS': 3, 
            #  'SUPPORT2CORRECTION4ERROR': 0, 
            #  'BITS2ADDRESS': 64, 
            #  'AVAILABLE': 'ACK', 
            #  'SIZE2MEMORY4LOCAL': 32768, 
            #  'VENDOR': 'AuthenticAMD', 
            #  'SUPPORT2IMAGE': 'NACK', 
            #  'MAX2SIZE4KERNELARGUMENT': 4096, 
            #  'ACK2RETURN': ' 1C-1D-1E-1F-1D-2A-...
            #  'OUT-OF-ORDER2QUEUE': 'NACK', 
            #  'ALIGNMENT2ADDRESS4BASE': 1024, 
            #  'PROFILE': 'FULL_PROFILE', 
            #  'TYPE2MEMORY4LOCAL': 'Global', 
            #  'PREFERRED2WIDTHFLOAT4VECTOR': 4, 
            #  'PREFERRED2WIDTHLONG4VECTOR': 2, 
            #  'MAX2FREQUENCY4CLOCK': 3210, 
            #  'PREFERRED2WIDTHSHORT4VECTOR': 8, 
            #  'ID': 4033, 
            #  'MAX2WORKITEM2': 1024, 
            #  'NATIVE2WIDTHDOUBLE4VECTOR': 0, 
            #  'ROUND2ZERO': 'ACK', 
            #  'NAME': 'AMD64 Family 16 Model 4 Stepping 2, 
            #   AuthenticAMD', 
            #  'MAX2WORKITEM1': 1024, 
            #  'MAX2WORKITEM0': 1024, 
            #  'SIZE2LINE4CACHE': 64, 
            #  'MAX2ALLOCATION4MEMORY': 1073741824, 
            #  'NATIVE2WIDTHCHAR4VECTOR': 16, 
            #  'QUIET2NANS': 'ACK', 
            #  'NATIVE2WIDTHSHORT4VECTOR': 8, 
            #  'ROUND2NEAREST4EVEN': 'ACK', 
            #  'VERSION': 'OpenCL 1.1 ATI-Stream-v2.3 (451)', 
            #  'NACK2RETURN': ' ', 
            #  'MEMORY2UNIFIED4HOSTANDDEVICE': 1, 
            #  'ENDIANESS2DEVICE': 'Little', 
            #  'MINIMUM2ALIGNMENT4DATATYPE': 128, 
            #  'ROUND2+VE3INFINITY': 'ACK', 
            #  'TYPE': 'CL_DEVICE_TYPE_KPU', 
            #  'DENORMS2SINGLEPRECISION4FLOATINGPOINTCAPABILITY': 'ACK'}; time: 03:40:47 10/04/11 Pacific Daylight Time}
            
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2kpu-failed-by-copy2imagetobuffer4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
        
        task4copyimagetobuffer2kpu ['DEVICE'] = device2kpu
        
               
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE SOURCE OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagesourceinfo2kpu = {}
         
        message2print = "{(Step 5A/70) trying" + "; imagesourceinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(imagesourceinfo2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagesourceinfo2kpu = get2imageinfo4kpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagesourceinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(imagesourceinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000042514E0>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': ...

            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagesourceinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(imagesourceinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copyimagetobuffer2kpu ['IMAGESOURCE'] = imagesourceinfo2kpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER DESTINATION OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        bufferdestinationinfo2kpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; bufferdestinationinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(bufferdestinationinfo2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            bufferdestinationinfo2kpu = get2bufferinfo4kpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; bufferdestinationinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(bufferdestinationinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4kpu', 
            #  'METHOD': 'create2buffer4kpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x0000000004782B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x000000000423BD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; bufferdestinationinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(bufferdestinationinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copyimagetobuffer2kpu ['BUFFERDESTINATION'] = bufferdestinationinfo2kpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY KPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4kpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copyimagetobuffer4kpu-trying-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4kpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copyimagetobuffer4kpu-got-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x00000000042B2208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copyimagetobuffer4kpu-failed-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copyimagetobuffer2kpu ['QUEUE'] = queue2copyimagetobuffer4kpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY KPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2kpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2kpu-trying-by-copy2imagetobuffer4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2kpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2kpu-got-by-copy2imagetobuffer4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 660>, <read-write PipeConnection, handle 664>)
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2kpu-failed-by-copy2imagetobuffer4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copyimagetobuffer2kpu ['PIPE'] = pipe2kpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2kpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4kpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2commandqueue4kpu', 
            #  'METHOD': 'create2commandqueue4kpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2commandqueue4kpu at 0x000000000395EA48>, 
            #  'ACK2RETURN': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copyimagetobuffer2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2kpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2kpu = get2eventinfo4kpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4kpu-failed ; time: 03:40:47 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2event4kpu', 
            #  'METHOD': 'create2event4kpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2event4kpu at 0x0000000003971F48>, 
            #  'ACK2RETURN': ' 1D-1E-...
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copyimagetobuffer2kpu ['EVENT'] = eventinfo2kpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2kpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2kpu-trying-by-copy2imagetobuffer4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2kpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2kpu-set-by-copy2imagetobuffer4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2kpu-failed-by-copy2imagetobuffer4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copyimagetobuffer2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2kpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-copy2imagetobuffer4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2kpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                      
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2kpu-set-by-copy2imagetobuffer4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-copy2imagetobuffer4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copyimagetobuffer2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (14) ORIGIN FOR SOURCE - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        origin4source2kpu = {}
        
        message2print = "{(Step 14A/70) trying" + "; origin4source2kpu-trying-by-copy2imagetobuffer4kpu: " + str(origin4source2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4source2kpu['X'] = 0 
            origin4source2kpu['Y'] = 0  
            origin4source2kpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 14B/70) set" + "; origin4source2kpu-set-by-copy2imagetobuffer4kpu to: " + str(origin4source2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 0, 'X': 0, 'Z': 0} 
            code2ack  = code2ack + '14B-' 
        except:
            message2print = "{(Step 14Z/70) failed" + "; origin4source2kpu-failed-by-copy2imagetobuffer4kpu: " + str(origin4source2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '14Z-'
    
        task4copyimagetobuffer2kpu ['SOURCEORIGIN'] = origin4source2kpu
        
       
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2kpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2kpu-trying-by-copy2imagetobuffer4kpu: " + str(region4copy2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2kpu['X'] = 10 
            region4copy2kpu['Y'] = 10  
            region4copy2kpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2kpu-set-by-copy2imagetobuffer4kpu to: " + str(region4copy2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10} 
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2kpu-failed-by-copy2imagetobuffer4kpu: " + str(region4copy2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copyimagetobuffer2kpu ['REGION'] = region4copy2kpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER                     
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2kpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2kpu-set-by-copy2imagetobuffer4kpu to: " + str(offset4copy2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        code2ack  = code2ack + '17A-'                
        task4copyimagetobuffer2kpu ['OFFSET'] = offset4copy2kpu            
                    
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
    else:
        pass
                    
    # ---------------------------------------------------------------------------------------------
    # ANALYZE TASK - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2kpu.keys()
    message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2imagetobuffer4kpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}


    if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
        pass
    else:                
        # ---------------------------------------------------------------------------------------------
        # (21B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4kpu = trace2image4kpu                                                             # CALLBACK FUNCTION FOR KPU
        
        message2print = "{(Step 21B1/70) initialized" + "; function2callback4kpu-created-by-copy2imagetobuffer4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
            
        code2ack  = code2ack + '21B1-'
           
        args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2imagetobuffer4kpu"                                                        # MODULE OF KPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2imagetobuffer4kpu"                                                        # FUNCTION OF KPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2imagetobuffer4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

        message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2imagetobuffer4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 

        code2ack  = code2ack + '21B2-'
        
        # ---------------------------------------------------------------------------------------------
        # (21C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copyimagetobuffer2kpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copyimagetobuffer2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
        
        code2ack  = code2ack + '21C-'
        message2print = "{(Step 21C/70) initialized" + "; task4copyimagetobuffer2kpu-created-by-copy2imagetobuffer4kpu to: " + str(task4copyimagetobuffer2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #
             
    if 'PLATFORM' in keys2task:
        pass
    else:  
        # ---------------------------------------------------------------------------------------------
        # (22) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2imagetobuffer4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        try:
            platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2imagetobuffer4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
                 
            code2ack  = code2ack + '22B-'
        except:
            message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2imagetobuffer4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '22Z-'
                 
        task4copyimagetobuffer2kpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
    if 'CONTEXT' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (23) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2kpu = {}
        message2print = "{(Step 23A/70) trying" + "; context2kpu-trying-by-copy2imagetobuffer4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2kpu = get2context4kpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
            
            message2print = "{(Step 23B/70) got" + "; context2kpu-got-by-copy2imagetobuffer4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '23B-'         
        except:
            message2print = "{(Step 23Z/70) failed" + "; context2kpu-failed-by-copy2imagetobuffer4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '23Z-'
            
        task4copyimagetobuffer2kpu ['CONTEXT'] = context2kpu           
    
    if 'DEVICE' in keys2task:
        pass
    else:                   
        # ---------------------------------------------------------------------------------------------
        # (24) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2kpu = {}
        
        message2print = "{(Step 24A/70) trying" + "; device2kpu-trying-by-copy2imagetobuffer4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2kpu = get2kpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 24B/70) got" + "; device2kpu-got-by-copy2imagetobuffer4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
                
            code2ack  = code2ack + '24B-'  
        except:
            message2print = "{(Step 24Z/70) failed" + "; device2kpu-failed-by-copy2imagetobuffer4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '24Z-'
        
        task4copyimagetobuffer2kpu ['DEVICE'] = device2kpu
    
    if 'IMAGESOURCE' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (25) SOURCE IMAGE - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        sourceimageinfo2kpu = {}
         
        message2print = "{(Step 25A/70) trying" + "; sourceimageinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(sourceimageinfo2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            sourceimageinfo2kpu = get2imageinfo4kpu()                                                                         
            
            message2print = "{(Step 25B/70) got" + "; sourceimageinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(sourceimageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '25B-'  
        except:
            message2print = "{(Step 25Z/70) failed" + "; sourceimageinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(sourceimageinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '25Z-'          
                  
        task4copyimagetobuffer2kpu ['IMAGESOURCE'] = sourceimageinfo2kpu         
    
    if 'BUFFERDESTINATION' in keys2task:
        pass
    else:                      
        # ---------------------------------------------------------------------------------------------
        # (26) DESTINATION BUFFER - REQUIRED                            
        # ---------------------------------------------------------------------------------------------         
        destinationbufferinfo2kpu = {}
         
        message2print = "{(Step 26A/70) trying" + "; destinationbufferinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(destinationbufferinfo2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            destinationbufferinfo2kpu = get2bufferinfo4kpu()                                                                         
            
            message2print = "{(Step 26B/70) got" + "; destinationbufferinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(destinationbufferinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            #                 
            code2ack  = code2ack + '26B-'  
        except:
            message2print = "{(Step 26Z/70) failed" + "; destinationbufferinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(destinationbufferinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '26Z-'          
                  
        task4copyimagetobuffer2kpu ['BUFFERDESTINATION'] = destinationbufferinfo2kpu    
                                
    if 'QUEUE' in keys2task:
        pass
    else:   
        # ---------------------------------------------------------------------------------------------
        # (27) QUEUE - REQUIRED BY KPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copyimagetobuffer4kpu = None
        
        message2print = "{(Step 27A/70) trying" + "; queue2copyimagetobuffer4kpu-trying-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copyimagetobuffer4kpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 27B/70) got" + "; queue2copyimagetobuffer4kpu-got-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '27B-'  
        except:
            message2print = "{(Step 27Z/70) failed" + "; queue2copyimagetobuffer4kpu-failed-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '27Z-'
            
        task4copyimagetobuffer2kpu ['QUEUE'] = queue2copyimagetobuffer4kpu
        
    if 'PIPE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) PIPE - REQUIRED BY KPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2kpu = None
       
        message2print = "{(Step 28A/70) trying" + "; pipe2kpu-trying-by-copy2imagetobuffer4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2kpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 28B/70) got" + "; pipe2kpu-got-by-copy2imagetobuffer4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # 
            
            code2ack  = code2ack + '28B-'  
        except:
            message2print = "{(Step 28Z/70) failed" + "; pipe2kpu-failed-by-copy2imagetobuffer4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '28Z-'
            
        task4copyimagetobuffer2kpu ['PIPE'] = pipe2kpu 
        
    if 'COMMANDQUEUE' in keys2task:
        pass
    else:              
        # ---------------------------------------------------------------------------------------------
        # (28) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2kpu = {}
       
        message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 29B/70) got" + "; commandqueueinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '29B-'  
        except:
            message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '29Z-'
    
        task4copyimagetobuffer2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
    
         
    if 'EVENT' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (31) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2kpu = {}
       
        message2print = "{(Step 31A/70) trying" + "; eventinfo2kpu-trying-by-copy2imagetobuffer4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2kpu = get2eventinfo4kpu()                                                                    
            message2print = "{(Step 31B/70) got" + "; eventinfo2kpu-got-by-copy2imagetobuffer4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '31B-' 
        except:
            message2print = "{(Step 31Z/70) failed" + "; eventinfo2kpu-failed-by-copy2imagetobuffer4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '31Z-'
    
        task4copyimagetobuffer2kpu ['EVENT'] = eventinfo2kpu 
    
         
    if 'EVENTWAITLIST' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (32) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2kpu = {}
       
        message2print = "{(Step 32A/70) trying" + "; eventwaitlist2kpu-trying-by-copy2imagetobuffer4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2kpu = {}                                                                   
            message2print = "{(Step 32B/70) set" + "; eventwaitlist2kpu-set-by-copy2imagetobuffer4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            
            code2ack  = code2ack + '32B-' 
        except:
            message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2kpu-failed-by-copy2imagetobuffer4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '32Z-'
    
        task4copyimagetobuffer2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     
          
    if 'EVENTWAITLISTQUANTITY' in keys2task:
        pass
    else:               
        # ---------------------------------------------------------------------------------------------
        # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2kpu = 0
       
        message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-copy2imagetobuffer4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2kpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                   
            message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2kpu-set-by-copy2imagetobuffer4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '33B-' 
        except:
            message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-copy2imagetobuffer4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '33Z-'
    
        task4copyimagetobuffer2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                         
                
    if 'SOURCEORIGIN' in keys2task:
        pass
    else:                    
        # ---------------------------------------------------------------------------------------------
        # (34) SOURCEORIGIN - REQUIRED                             
        # ---------------------------------------------------------------------------------------------             
        source4origin2kpu = {}
        
        message2print = "{(Step 34A/70) trying" + "; source4origin2kpu-trying-by-copy2imagetobuffer4kpu: " + str(source4origin2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            source4origin2kpu['X'] = 0 
            source4origin2kpu['Y'] = 0  
            source4origin2kpu['Z'] = 0                                                                                                                          
            message2print = "{(Step 34B/70) set" + "; source4origin2kpu-set-by-copy2imagetobuffer4kpu to: " + str(source4origin2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '34B-' 
        except:
            message2print = "{(Step 34Z/70) failed" + "; source4origin2kpu-failed-by-copy2imagetobuffer4kpu: " + str(source4origin2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '34Z-'
    
        task4copyimagetobuffer2kpu ['SOURCEORIGIN'] = source4origin2kpu
             
   
             
    if 'REGION' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (36) REGION - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        region2kpu = {}
        
        message2print = "{(Step 36A/70) trying" + "; region2kpu-trying-by-copy2imagetobuffer4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
            region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
            region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
            message2print = "{(Step 36B/70) set" + "; region2kpu-set-by-copy2imagetobuffer4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            code2ack  = code2ack + '36B-' 
        except:
            message2print = "{(Step 36Z/70) failed" + "; region2kpu-failed-by-copy2imagetobuffer4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '36Z-'
    
        task4copyimagetobuffer2kpu ['REGION'] = region2kpu
    
         
    if 'OFFSET' in keys2task:
        pass
    else:            
        # ---------------------------------------------------------------------------------------------
        # (37) OFFSET - WHERE TO BEGIN COPYING DATA INTO DESTINATION BUFFER - REQUIRED                         
        # ---------------------------------------------------------------------------------------------             
        offset2kpu = 0
        message2print = "{(Step 37A/70) set" + "; offset2kpu-set-by-copy2imagetobuffer4kpu to: " + str(offset2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 
        task4copyimagetobuffer2kpu ['OFFSET'] = offset2kpu        
        code2ack  = code2ack + '37A-' 
        
    # ---------------------------------------------------------------------------------------------
    # EOF TASK
    # ---------------------------------------------------------------------------------------------
    keys2task = task4copyimagetobuffer2kpu.keys()
    message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2imagetobuffer4kpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)            
    # ['BUFFERDESTINATION',
    #  'COMMANDQUEUE',
    #  'CONTEXT',
    #  'DATA2CALLBACK',
    #  'DEVICE',
    #  'EVENT',
    #  'EVENTWAITLIST',
    #  'EVENTWAITLISTQUANTITY',
    #  'FUNCTION2CALLBACK',
    #  'IMAGESOURCE',
    #  'OFFSET',
    #  'PIPE',
    #  'PLATFORM',
    #  'QUEUE',
    #  'REGION',
    #  'SOURCEORIGIN']; time: 01:28:44 10/05/11 Pacific Daylight Time}
               


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copyimagetobuffer4kpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2kpu = False
    channels2kpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2kpu-trying-by-copy2imagetobuffer4kpu: " + str(channels2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copyimagetobuffer4kpu, publisherchannel2copyimagetobuffer4kpu = task4copyimagetobuffer2kpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copyimagetobuffer4kpu-got-by-copy2imagetobuffer4kpu: " + str(consumerchannel2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 660>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copyimagetobuffer4kpu-got-by-copy2imagetobuffer4kpu: " + str(publisherchannel2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 664>                     
        ack4channels2kpu = True
        
        control2copyimagetobuffer4kpu['CONSUMERCHANNEL']  = consumerchannel2copyimagetobuffer4kpu
        
        
        control2copyimagetobuffer4kpu['PUBLISHERCHANNEL'] = publisherchannel2copyimagetobuffer4kpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2kpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copyimagetobuffer4kpu = False
    queue2copyimagetobuffer4kpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copyimagetobuffer4kpu-trying-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copyimagetobuffer4kpu = task4copyimagetobuffer2kpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copyimagetobuffer4kpu-got-by-copy2imagetobuffer4kpu: " + str(queue2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004465E48>
              
        ack4queue2copyimagetobuffer4kpu = True
        control2copyimagetobuffer4kpu['QUEUE']  = queue2copyimagetobuffer4kpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copyimagetobuffer4kpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4kpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4kpu-trying-by-copy2imagetobuffer4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4kpu = DummyProcess4Xi(target=manoeuvre2copyimagetobuffer4kpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copyimagetobuffer4kpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copyimagetobuffer4kpu, publisherchannel2copyimagetobuffer4kpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4kpu-created-by-copy2imagetobuffer4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2imagetobuffer4kpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-14B-16B-17A-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2imagetobuffer4kpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2copyimagetobuffer4kpu-returning-by-copy2imagetobuffer4kpu: " + str(control2copyimagetobuffer4kpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004465E48>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 664>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 660>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2copyimagetobuffer4kpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "copy2buffertoimage4kpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (80) ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4KPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 80A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4KPU ', time4xi())    
#   response4copy2buffertoimage = {} 
#   task4copy2buffertoimage = None
#   try:                               
#       response4copy2buffertoimage = copy2buffertoimage4kpu(task4copy2buffertoimage)                                                                                                          
#       print('(Step 80B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4KPU : ', response4copy2buffertoimage)
#       queue2copybuffertoimage4kpu      = response4copy2buffertoimage['QUEUE']
#       channel2copybuffertoimagepublish4kpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimageconsume4kpu = response4copy2buffertoimage['CONSUMERCHANNEL']
#   except:
#       print('(Step 80Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH COPYBUFFERTOIMAGE4KPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (81) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 81A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4KPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4kpu = "<copy4xi>copy2copybuffertoimage4kpu</copy4xi>"
#       channel2copybuffertoimagepublish4kpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4kpu.send(message2copybuffertoimagesend4kpu)
#       print("(Step 81B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2copybuffertoimagepublish4kpu, " MESSAGE2SEND4KPU: " , message2copybuffertoimagesend4kpu)                                                                                                
#   except:
#       print('(Step 81Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4KPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (82) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 82C/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copybuffertoimage4kpu(queue2copybuffertoimage4kpu, channel2copybuffertoimageconsume4kpu, channel2copybuffertoimagepublish4kpu)
#       print("(Step 82D/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", manoeuvre2copybuffertoimage4kpu)
#   except:
#       print("(Step 82Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (83) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 83A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4KPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4kpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4kpu = channel2copybuffertoimageconsume4kpu.recv()
#       print("(Step 83B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copybuffertoimageconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                  
#       # 
#   except:
#       print('(Step 83Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4KPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (84) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 84A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4KPU ', time4xi())     
#   try:                               
#       message2copybuffertoimagesend4kpu = "<copy4xi>copy2copybuffertoimage4kpu</copy4xi>"
#       channel2copybuffertoimagepublish4kpu = response4copy2buffertoimage['PUBLISHERCHANNEL']
#       channel2copybuffertoimagepublish4kpu.send(message2copybuffertoimagesend4kpu)
#       print("(Step 84B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2copybuffertoimagepublish4kpu, " MESSAGE2SEND4KPU: " , message2copybuffertoimagesend4kpu)                                                                                                
#       #  
#   except:
#        print('(Step 84Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4KPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (85) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 85A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copybuffertoimage4kpu(queue2copybuffertoimage4kpu, channel2copybuffertoimageconsume4kpu, channel2copybuffertoimagepublish4kpu)
#       print("(Step 85B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", manoeuvre2copybuffertoimage4kpu)
#   except:
#       print("(Step 85Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (86) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 86A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH COPYBUFFERTOIMAGE4KPU ', time4xi())     
#   try:
#       channel2copybuffertoimageconsume4kpu = response4copy2buffertoimage['CONSUMERCHANNEL']                              
#       message2copyimagereceive4kpu = channel2copybuffertoimageconsume4kpu.recv()
#       print("(Step 86B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copybuffertoimageconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                  
#       # 
#   except:
#       print('(Step 86Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH COPYBUFFERTOIMAGE4KPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (87) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 87A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2copybuffertoimage4dragon.put(path2dragon)             
#       print("(Step 87B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 87Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (88) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 88A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copybuffertoimage4kpu(queue2copybuffertoimage4kpu, channel2copybuffertoimageconsume4kpu, channel2copybuffertoimagepublish4kpu)
#       print("(Step 88B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", manoeuvre2copybuffertoimage4kpu)
#   except:
#       print("(Step 88Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (89) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 89A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2copyimagereceive4kpu = channel2copybuffertoimageconsume4kpu.recv()
#       print("(Step 89D/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2copybuffertoimageconsume4kpu, " CONTROL & COMMAND: ", message2copyimagereceive4kpu)                                                                                                   
#   except:
#       print('(Step 89Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (90) LET MANOEUVRE2COPYIMAGETOBUFFER4KPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 90A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi()) 
#   try:        
#       response2manoeuvreimagecopytobuffer4kpu = manoeuvre2copybuffertoimage4kpu(queue2copybuffertoimage4kpu, channel2copybuffertoimageconsume4kpu, channel2copybuffertoimagepublish4kpu)
#       print("(Step 90B/700) OF KPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4KPU ", response2manoeuvreimagecopytobuffer4kpu)
#   except:
#       print("(Step 90Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGETOBUFFER4KPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def copy2buffertoimage4kpu(task4copybuffertoimage2kpu=None):

    '''
    copies buffer to image for KPU
    '''
    global kpu2image4xi                                                                                             # 2D-IMAGES FOR KPU  
    global kpu2buffer4xi                                                                                            # BUFFER FOR KPU
    
    global control2copybuffertoimage4kpu                                                                            # COMMAND AND CONTROL FOR KPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "copy2buffertoimage4kpu"
    method2print = "copy2buffertoimage4kpu"

    message2print = "{(Step 1A/33) started" + "; copy2buffertoimage4kpu-started: " + str(copy2buffertoimage4kpu)  + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                                
    # <function copy2buffertoimage4kpu at 0x0000000003869548>
    
    message2print = "{(Step 1B/33) passed" + "; task4copybuffertoimage2kpu-to-copy2buffertoimage4kpu: " + str(task4copybuffertoimage2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                              # INITIALIZE IMAGE FOR KPU 
            message2print = "{(Step 1C/70) created" + "; kpu2image4xi-created-by-copy2buffertoimage4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4kpu-skipped-by-copy2buffertoimage4kpu: " + str(copy2buffertoimage4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 1E/70) set" + "; kpu2image4xi-set-by-copy2buffertoimage4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2image4xi = {}
            message2print = "{(Step 1Z1/70) yielded" + "; kpu2image4xi-yielded-by-copy2buffertoimage4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z1-' 
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2BUFFER4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2buffer4xi is None:
            kpu2buffer4xi = create2buffer4kpu()                                                                     # INITIALIZE BUFFER FOR KPU 
            message2print = "{(Step 1F/70) created" + "; kpu2buffer4xi-created-by-copy2buffertoimage4kpu to: " + str(kpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1G/70) skipped" + "; create2buffer4kpu-skipped-by-copy2buffertoimage4kpu: " + str(copy2buffertoimage4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                  
    except:
        try:
            kpu2buffer4xi = create2buffer4kpu()
            message2print = "{(Step 1H/70) set" + "; kpu2buffer4xi-set-by-copy2buffertoimage4kpu to: " + str(kpu2buffer4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2buffer4xi = {}
            message2print = "{(Step 1Z2/70) yielded" + "; kpu2buffer4xi-yielded-by-copy2buffertoimage4kpu: " + str(kpu2buffer4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z2-' 
                            
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - ANALYZE TASKS           
    # -----------------------------------------------------------------------------------------          
    if task4copybuffertoimage2kpu is None:
        # -----------------------------------------------------------------------------------------
        # NO PARAMETERS GIVEN BY CALLER FOR COPY - BUILD TASK! 
        # -----------------------------------------------------------------------------------------    
        
        task4copybuffertoimage2kpu = {}                                                                                                                                                                                       
        
        message2print = "{(Step 1A/70) initialized" + "; task4copybuffertoimage2kpu-created-by-copy2buffertoimage4kpu to: " + str(task4copybuffertoimage2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # 
        
        # ---------------------------------------------------------------------------------------------
        # (1B) TRACE - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        function2callback4kpu = trace2image4kpu                                                                 # CALLBACK FUNCTION FOR KPU
        
        message2print = "{(Step 1B1/70) initialized" + "; function2callback4kpu-created-by-copy2buffertoimage4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <function trace2image4kpu at 0x0000000004645548>
        
        code2ack  = code2ack + '1B1-'
       
        args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
        args2callback['CLASS']   = "copy2buffertoimage4kpu"                                                     # MODULE OF KPUIZED OPENCL MODULE
        args2callback['METHOD']  = "copy2buffertoimage4kpu"                                                     # FUNCTION OF KPUIZED OPENCL MODULE
        args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

        message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)               
        # {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4kpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}',
        #  'CLASS': 'copy2buffertoimage4kpu', 
        #  'METHOD': 'copy2buffertoimage4kpu'}; time: 04:41:15 10/04/11 Pacific Daylight Time}

        code2ack  = code2ack + '1B2-'
            
        # ---------------------------------------------------------------------------------------------
        # (1C) CALLBACK - AUTOMATIC                      
        # ---------------------------------------------------------------------------------------------              
        task4copybuffertoimage2kpu['DATA2CALLBACK'] = args2callback                                             # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
        task4copybuffertoimage2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                 # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
        code2ack  = code2ack + '1C-'
        message2print = "{(Step 1C/70) initialized" + "; task4copybuffertoimage2kpu-created-by-copy2buffertoimage4kpu to: " + str(task4copybuffertoimage2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # {'FUNCTION2CALLBACK': <function trace2image4kpu at 0x0000000004645548>, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; copy2buffertoimage4kpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
        #  'CLASS': 'copy2buffertoimage4kpu', 
        #  'METHOD': 'copy2buffertoimage4kpu'}}; time: 04:41:15 10/04/11 Pacific Daylight Time}

            
        # ---------------------------------------------------------------------------------------------
        # (2) PLATFORM - AUTOMATIC                   
        # ---------------------------------------------------------------------------------------------             
        platform2xi = {}
        message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # {}
            
        try:
            platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
            message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-copy2buffertoimage4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'list2platforms': ['DragonXi'], 
            #  'ack2return': ' 2C-2E-2F-2G-', 
            #  'nack2return': ' ', 
            #  'available2platform': 1, 
            #  'max2entries': 1}; time: 04:41:15 10/04/11 Pacific Daylight Time}
                 
            code2ack  = code2ack + '2B-'
        except:
            message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                
            code2nack  = code2nack + '2Z-'
                 
        task4copybuffertoimage2kpu['PLATFORM'] = platform2xi                                                    # PLATFORM OF SYSTEM 
              
            
        # ---------------------------------------------------------------------------------------------
        # (3) CONTEXT - AUTOMATIC                 
        # ---------------------------------------------------------------------------------------------    
        context2kpu = {}
        message2print = "{(Step 3A/70) trying" + "; context2kpu-trying-by-copy2buffertoimage4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
            message2print = "{(Step 3B/70) got" + "; context2kpu-got-by-copy2buffertoimage4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
            code2ack  = code2ack + '3B-'         
        except:
            message2print = "{(Step 3Z/70) failed" + "; context2kpu-failed-by-copy2buffertoimage4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z-'
                
        task4copybuffertoimage2kpu ['CONTEXT'] = context2kpu           
            
        # ---------------------------------------------------------------------------------------------
        # (4) DEVICE - AUTOMATIC                       
        # ---------------------------------------------------------------------------------------------    
        device2kpu = {}
            
        message2print = "{(Step 4A/70) trying" + "; device2kpu-trying-by-copy2buffertoimage4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
            
            message2print = "{(Step 4B/70) got" + "; device2kpu-got-by-copy2buffertoimage4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'RESOLUTION2TIMER4PROFILING': 318, 
            #  'ID2PLATFORM': '0000000001509568', 
            #  'PROFILING2QUEUE': 'ACK', 
            #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
            #  'NATIVE2WIDTHLONG4VECTOR': 2, 
            #  'MAX2UNITS4COMPUTE': 4, 
            #  'SIZE2MEMORY4GLOBAL': 3221225472, ...
               
            code2ack  = code2ack + '4B-'  
        except:
            message2print = "{(Step 4Z/70) failed" + "; device2kpu-failed-by-copy2buffertoimage4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z-'
            
        task4copybuffertoimage2kpu ['DEVICE'] = device2kpu
            
                   
        # ---------------------------------------------------------------------------------------------
        # (5) IMAGE DESTINATION OBJECT - REQUIRED                         
        # ---------------------------------------------------------------------------------------------         
        imagedestinationinfo2kpu = {}
             
        message2print = "{(Step 5A/70) trying" + "; imagedestinationinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(imagedestinationinfo2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            imagedestinationinfo2kpu = get2imageinfo4kpu()                                                                         
            
            message2print = "{(Step 5B/70) got" + "; imagedestinationinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(imagedestinationinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'SIZE2QUEUE': 2147483646, 
            #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
            #  'ORDER': 'CL_RGBA'}, 
            #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004105358>, 
            #  'WIDTH': 800, 
            #  'CONTEXT': {'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2context4kpu', 
            #  'METHOD': 'create2context4kpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x00000000037FF348>, 
            #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
            #  'INFO2PRIVATE': None, 
            #  'PLATFORM': {'list2platforms': ['DragonXi'],'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
            #  'NACK2RETURN': ' ', 
            #  'SIZE2PRIVATE': 0, 
            #  'PROPERTIES': None}, 
            #  'PIPE': (<read-write PipeConnection, handle 524>, <read-write PipeConnection, handle 528>), 
            #  'COUNT2REFERENCE': 1, 
            #  'FUNCTION2CALLBACK': <function trace2image4kpu at 0x0000000004645548>, 
            #  'COMMANDQUEUE': ...
            
            code2ack  = code2ack + '5B-'  
        except:
            message2print = "{(Step 5Z/70) failed" + "; imagedestinationinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(imagedestinationinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '5Z-'          
                  
        task4copybuffertoimage2kpu ['IMAGEDESTINATION'] = imagedestinationinfo2kpu         
               
        # ---------------------------------------------------------------------------------------------
        # (6) BUFFER SOURCE OBJECT - REQUIRED                        
        # ---------------------------------------------------------------------------------------------                         
        buffersourceinfo2kpu = {}
         
        message2print = "{(Step 6A/70) trying" + "; buffersourceinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(buffersourceinfo2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        try:
            buffersourceinfo2kpu = get2bufferinfo4kpu()                                                                         
            
            message2print = "{(Step 6B/70) got" + "; buffersourceinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(buffersourceinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # {'CODE2ERROR': 0, 
            #  'COUNT2REFERENCE': 0, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 04:41:15 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2buffer4kpu', 
            #  'METHOD': 'create2buffer4kpu'}, 
            #  'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x0000000004631B48>, 
            #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000040EBD98>, 
            #  'FLAGS': 0, 
            #  'CONTEXT': ...
            
            code2ack  = code2ack + '6B-'  
        except:
            message2print = "{(Step 6Z/70) failed" + "; buffersourceinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(buffersourceinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '6Z-'          
                  
        task4copybuffertoimage2kpu ['BUFFERSOURCE'] = buffersourceinfo2kpu     
        
        # ---------------------------------------------------------------------------------------------
        # (7) QUEUE - REQUIRED BY KPUXI                          
        # ---------------------------------------------------------------------------------------------         
        queue2copybuffertoimage4kpu = None
        
        message2print = "{(Step 7A/70) trying" + "; queue2copybuffertoimage4kpu-trying-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            queue2copybuffertoimage4kpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
            
            message2print = "{(Step 7B/70) got" + "; queue2copybuffertoimage4kpu-got-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # <queue4xi.Queue4Xi object at 0x000000000492B208> 
            
            code2ack  = code2ack + '7B-'  
        except:
            message2print = "{(Step 7Z/70) failed" + "; queue2copybuffertoimage4kpu-failed-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '7Z-'
            
        task4copybuffertoimage2kpu ['QUEUE'] = queue2copybuffertoimage4kpu
         
        # ---------------------------------------------------------------------------------------------
        # (8) PIPE - REQUIRED BY KPUXI                              
        # ---------------------------------------------------------------------------------------------         
        pipe2kpu = None
       
        message2print = "{(Step 8A/70) trying" + "; pipe2kpu-trying-by-copy2buffertoimage4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        try:
            pipe2kpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
            
            message2print = "{(Step 8B/70) got" + "; pipe2kpu-got-by-copy2buffertoimage4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                  
            # (<read-write PipeConnection, handle 708>, <read-write PipeConnection, handle 712>) 
            
            code2ack  = code2ack + '8B-'  
        except:
            message2print = "{(Step 8Z/70) failed" + "; pipe2kpu-failed-by-copy2buffertoimage4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '8Z-'
            
        task4copybuffertoimage2kpu ['PIPE'] = pipe2kpu 
        
        # ---------------------------------------------------------------------------------------------
        # (9) COMMAND QUEUE - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        commandqueueinfo2kpu = {}
       
        message2print = "{(Step 9A/70) trying" + "; commandqueueinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)  
                    
        try:
            commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
            message2print = "{(Step 9B/70) got" + "; commandqueueinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '9B-'  
        except:
            message2print = "{(Step 9Z/70) failed" + "; commandqueueinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '9Z-'
    
        task4copybuffertoimage2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
        
        
        
        # ---------------------------------------------------------------------------------------------
        # (11) EVENT - REQUIRED                            
        # ---------------------------------------------------------------------------------------------             
        eventinfo2kpu = {}
       
        message2print = "{(Step 11A/70) trying" + "; eventinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventinfo2kpu = get2eventinfo4kpu()                                                                    
            message2print = "{(Step 11B/70) got" + "; eventinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 'DATA2CALLBACK': ...
            
            code2ack  = code2ack + '11B-' 
        except:
            message2print = "{(Step 11Z/70) failed" + "; eventinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '11Z-'
    
        task4copybuffertoimage2kpu ['EVENT'] = eventinfo2kpu 
        
        
        # ---------------------------------------------------------------------------------------------
        # (12) EVENTWAITLIST - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlist2kpu = {}
       
        message2print = "{(Step 12A/70) trying" + "; eventwaitlist2kpu-trying-by-copy2buffertoimage4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            eventwaitlist2kpu = {}                                                                   
            message2print = "{(Step 12B/70) set" + "; eventwaitlist2kpu-set-by-copy2buffertoimage4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {}
            
            code2ack  = code2ack + '12B-' 
        except:
            message2print = "{(Step 12Z/70) failed" + "; eventwaitlist2kpu-failed-by-copy2buffertoimage4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '12Z-'
    
        task4copybuffertoimage2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     
         
        # ---------------------------------------------------------------------------------------------
        # (13) EVENTWAITLISTQUANTITY - REQUIRED                              
        # ---------------------------------------------------------------------------------------------             
        eventwaitlistquantity2kpu = 0
       
        message2print = "{(Step 13A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-copy2buffertoimage4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            keys2event = eventwaitlist2kpu.keys()
            for key2event in keys2event:
                eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                     
            message2print = "{(Step 13B/70) set" + "; eventwaitlistquantity2kpu-set-by-copy2buffertoimage4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 0 
            
            code2ack  = code2ack + '13B-' 
        except:
            message2print = "{(Step 13Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-copy2buffertoimage4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '13Z-'
    
        task4copybuffertoimage2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                             
        
        # ---------------------------------------------------------------------------------------------
        # (15) ORIGIN FOR DESTINATION - REQUIRED                             
        # ---------------------------------------------------------------------------------------------              
        origin4destination2kpu = {}
        
        message2print = "{(Step 15A/70) trying" + "; origin4destination2kpu-trying-by-copy2buffertoimage4kpu: " + str(origin4destination2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:                                                                                    
            origin4destination2kpu['X'] = 100 
            origin4destination2kpu['Y'] = 100  
            origin4destination2kpu['Z'] = 100                                                                                                                          
            message2print = "{(Step 15B/70) set" + "; origin4destination2kpu-set-by-copy2buffertoimage4kpu to: " + str(origin4destination2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 100, 'X': 100, 'Z': 100}
            code2ack  = code2ack + '15B-' 
        except:
            message2print = "{(Step 15Z/70) failed" + "; origin4destination2kpu-failed-by-copy2buffertoimage4kpu: " + str(origin4destination2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '15Z-'
    
        task4copybuffertoimage2kpu ['DESTINATIONORIGIN'] = origin4destination2kpu
                    
        
        
        # --------------------------------------------------------------------------------------------------------
        # (16) REGION - REGION OF A IMAGE TO BE COPIED                         
        # --------------------------------------------------------------------------------------------------------             
        region4copy2kpu = {}                                                                                                         
       
        message2print = "{(Step 16A/70) trying" + "; region4copy2kpu-trying-by-copy2buffertoimage4kpu: " + str(region4copy2kpu)  + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
                    
        try:
            region4copy2kpu['X'] = 10 
            region4copy2kpu['Y'] = 10  
            region4copy2kpu['Z'] = 10                                                                           
            message2print = "{(Step 16B/70) set" + "; region4copy2kpu-set-by-copy2buffertoimage4kpu to: " + str(region4copy2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'Y': 10, 'X': 10, 'Z': 10}
            code2ack  = code2ack + '16B-'                
             
        except:
            message2print = "{(Step 16Z/70) failed" + "; region4copy2kpu-failed-by-copy2buffertoimage4kpu: " + str(region4copy2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '16Z-'
        
        task4copybuffertoimage2kpu ['REGION'] = region4copy2kpu              
        
        # --------------------------------------------------------------------------------------------------------
        # (17) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER                   
        # --------------------------------------------------------------------------------------------------------             
        offset4copy2kpu = 0                                                                                                         
        message2print = "{(Step 17A/70) set" + "; offset4copy2kpu-set-by-copy2buffertoimage4kpu to: " + str(offset4copy2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # 0
        
        task4copybuffertoimage2kpu ['OFFSET'] = offset4copy2kpu            
                                
        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER PROVIDING                
        # ---------------------------------------------------------------------------------------------                                                                                                                               
        
        # ---------------------------------------------------------------------------------------------
        # ANALYZE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2kpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-copy2buffertoimage4kpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:41:15 10/04/11 Pacific Daylight Time}
 
        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                             # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4kpu-created-by-copy2buffertoimage4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "copy2buffertoimage4kpu"                                                        # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "copy2buffertoimage4kpu"                                                        # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; copy2buffertoimage4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-copy2buffertoimage4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4copybuffertoimage2kpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4copybuffertoimage2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4copybuffertoimage2kpu-created-by-copy2buffertoimage4kpu to: " + str(task4copybuffertoimage2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-copy2buffertoimage4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-copy2buffertoimage4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-copy2buffertoimage4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4copybuffertoimage2kpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2kpu-trying-by-copy2buffertoimage4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2kpu-got-by-copy2buffertoimage4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2kpu-failed-by-copy2buffertoimage4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4copybuffertoimage2kpu ['CONTEXT'] = context2kpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2kpu-trying-by-copy2buffertoimage4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2kpu-got-by-copy2buffertoimage4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2kpu-failed-by-copy2buffertoimage4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4copybuffertoimage2kpu ['DEVICE'] = device2kpu
        
        if 'IMAGEDESTINATION' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) DESTINATION IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            destinationimageinfo2kpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; destinationimageinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(destinationimageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                destinationimageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; destinationimageinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(destinationimageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; destinationimageinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(destinationimageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4copybuffertoimage2kpu ['IMAGEDESTINATION'] = destinationimageinfo2kpu         
        
        if 'BUFFERSOURCE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (26) SOURCE BUFFER - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            sourcebufferinfo2kpu = {}
             
            message2print = "{(Step 26A/70) trying" + "; sourcebufferinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(sourcebufferinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                sourcebufferinfo2kpu = get2bufferinfo4kpu()                                                                         
                
                message2print = "{(Step 26B/70) got" + "; sourcebufferinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(sourcebufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; sourcebufferinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(sourcebufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'          
                      
            task4copybuffertoimage2kpu ['BUFFERSOURCE'] = sourcebufferinfo2kpu    
                                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (27) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2copybuffertoimage4kpu = None
            
            message2print = "{(Step 27A/70) trying" + "; queue2copybuffertoimage4kpu-trying-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2copybuffertoimage4kpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 27B/70) got" + "; queue2copybuffertoimage4kpu-got-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; queue2copybuffertoimage4kpu-failed-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4copybuffertoimage2kpu ['QUEUE'] = queue2copybuffertoimage4kpu
            
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 28A/70) trying" + "; pipe2kpu-trying-by-copy2buffertoimage4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 28B/70) got" + "; pipe2kpu-got-by-copy2buffertoimage4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; pipe2kpu-failed-by-copy2buffertoimage4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
                
            task4copybuffertoimage2kpu ['PIPE'] = pipe2kpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; commandqueueinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 29B/70) got" + "; commandqueueinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-'  
            except:
                message2print = "{(Step 29Z/70) failed" + "; commandqueueinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4copybuffertoimage2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
        
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventinfo2kpu-trying-by-copy2buffertoimage4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 31B/70) got" + "; eventinfo2kpu-got-by-copy2buffertoimage4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventinfo2kpu-failed-by-copy2buffertoimage4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4copybuffertoimage2kpu ['EVENT'] = eventinfo2kpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlist2kpu-trying-by-copy2buffertoimage4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlist2kpu-set-by-copy2buffertoimage4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlist2kpu-failed-by-copy2buffertoimage4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4copybuffertoimage2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (33) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
           
            message2print = "{(Step 33A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-copy2buffertoimage4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                     
                message2print = "{(Step 33B/70) set" + "; eventwaitlistquantity2kpu-set-by-copy2buffertoimage4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-copy2buffertoimage4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4copybuffertoimage2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                             
  
             
        if 'DESTINATIONORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) DESTINATIONORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            destination4origin2kpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; destination4origin2kpu-trying-by-copy2buffertoimage4kpu: " + str(destination4origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                destination4origin2kpu['X'] = 0 
                destination4origin2kpu['Y'] = 0  
                destination4origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; destination4origin2kpu-set-by-copy2buffertoimage4kpu to: " + str(destination4origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; destination4origin2kpu-failed-by-copy2buffertoimage4kpu: " + str(destination4origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4copybuffertoimage2kpu ['DESTINATIONORIGIN'] = destination4origin2kpu
             
     
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (36) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 36A/70) trying" + "; region2kpu-trying-by-copy2buffertoimage4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 36B/70) set" + "; region2kpu-set-by-copy2buffertoimage4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; region2kpu-failed-by-copy2buffertoimage4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '36Z-'
        
            task4copybuffertoimage2kpu ['REGION'] = region2kpu
        
             
        if 'OFFSET' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (37) OFFSET - WHERE TO BEGIN COPYING DATA FROM SOURCE BUFFER - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            offset2kpu = 0
            message2print = "{(Step 37A/70) set" + "; offset2kpu-set-by-copy2buffertoimage4kpu to: " + str(offset2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 
            task4copybuffertoimage2kpu ['OFFSET'] = offset2kpu        
            code2ack  = code2ack + '37A-' 
                    
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4copybuffertoimage2kpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-copy2buffertoimage4kpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BUFFERSOURCE',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DESTINATIONORIGIN',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGEDESTINATION',
        #  'OFFSET',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION']; time: 04:52:11 10/04/11 Pacific Daylight Time}
                


    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2copybuffertoimage4kpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2kpu = False
    channels2kpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2kpu-trying-by-copy2buffertoimage4kpu: " + str(channels2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2copybuffertoimage4kpu, publisherchannel2copybuffertoimage4kpu = task4copybuffertoimage2kpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2copybuffertoimage4kpu-got-by-copy2buffertoimage4kpu: " + str(consumerchannel2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 704>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2copybuffertoimage4kpu-got-by-copy2buffertoimage4kpu: " + str(publisherchannel2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 708>                      
        ack4channels2kpu = True
        
        control2copybuffertoimage4kpu['CONSUMERCHANNEL']  = consumerchannel2copybuffertoimage4kpu
        
        
        control2copybuffertoimage4kpu['PUBLISHERCHANNEL'] = publisherchannel2copybuffertoimage4kpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2kpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2copybuffertoimage4kpu = False
    queue2copybuffertoimage4kpu = None
    message2print = "{(Step 41A/70) trying" + "queue2copybuffertoimage4kpu-trying-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2copybuffertoimage4kpu = task4copybuffertoimage2kpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2copybuffertoimage4kpu-got-by-copy2buffertoimage4kpu: " + str(queue2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003DF2198>
              
        ack4queue2copybuffertoimage4kpu = True
        control2copybuffertoimage4kpu['QUEUE']  = queue2copybuffertoimage4kpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2copybuffertoimage4kpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4kpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4kpu-trying-by-copy2buffertoimage4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4kpu = DummyProcess4Xi(target=manoeuvre2copybuffertoimage4kpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2copybuffertoimage4kpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2copybuffertoimage4kpu, publisherchannel2copybuffertoimage4kpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4kpu-created-by-copy2buffertoimage4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-copy2buffertoimage4kpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-11B-12B-13B-15B-16B-40D-41C- 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-copy2buffertoimage4kpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 
        
    message2print = "{(Step 70C/70) returning" + "; control2copybuffertoimage4kpu-returning-by-copy2buffertoimage4kpu: " + str(control2copybuffertoimage4kpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003DF2198>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 708>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 704>}; time: 04:52:11 10/04/11 Pacific Daylight Time}

        
    return control2copybuffertoimage4kpu
                    
#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "read2image4kpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4KPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4KPU ', time4xi())    
#   respon4read2image = {} 
#   task4read2image = None
#   try:                               
#       response4read2image = read2image4kpu(task4read2image)                                                                                                          
#       print('(Step 16B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4KPU : ', response4read2image)
#       queue2read4kpu      = response4read2image['QUEUE']
#       channel2readpublish4kpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readconsume4kpu = response4read2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4KPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4KPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:                               
#       message2readsend4kpu = "<copy4xi>copy2read4kpu</copy4xi>"
#       channel2readpublish4kpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4kpu.send(message2readsend4kpu)
#       print("(Step 17B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2readpublish4kpu, " MESSAGE2SEND4KPU: " , message2readsend4kpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4KPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2read4kpu, channel2readconsume4kpu, channel2readpublish4kpu)
#       print("(Step 18D/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2readimage4kpu)
#   except:
#       print("(Step 18Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:
#       channel2readconsume4kpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4kpu = channel2readconsume4kpu.recv()
#       print("(Step 19B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2readconsume4kpu, " CONTROL & COMMAND: ", message2readreceive4kpu)                                                                                                  
#       # <roger4xi>roger2read4kpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4KPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4KPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:                               
#       message2readsend4kpu = "<ready4xi>ready2read4kpu</ready4xi>"
#       channel2readpublish4kpu = response4read2image['PUBLISHERCHANNEL']
#       channel2readpublish4kpu.send(message2readsend4kpu)
#       print("(Step 20B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2readpublish4kpu, " MESSAGE2SEND4KPU: " , message2readsend4kpu)                                                                                                
#       # <ready4xi>ready2read4kpu</ready4xi> 
#   except:
#        print('(Step 20Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4KPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2read4kpu, channel2readconsume4kpu, channel2readpublish4kpu)
#       print("(Step 21B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2readimage4kpu)
#   except:
#       print("(Step 21Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:
#       channel2readconsume4kpu = response4read2image['CONSUMERCHANNEL']                              
#       message2readreceive4kpu = channel2readconsume4kpu.recv()
#       print("(Step 22B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2readconsume4kpu, " CONTROL & COMMAND: ", message2readreceive4kpu)                                                                                                  
#       # <ready4xi>ready2read4kpu</ready4xi> 
#   except:
#       print('(Step 22Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4KPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2read4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4KPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2read4kpu, channel2readconsume4kpu, channel2readpublish4kpu)
#       print("(Step 25B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2readimage4kpu)
#   except:
#       print("(Step 25Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2readreceive4kpu = channel2readconsume4kpu.recv()
#       print("(Step 26D/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2readconsume4kpu, " CONTROL & COMMAND: ", message2readreceive4kpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4kpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4KPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2read4kpu, channel2readconsume4kpu, channel2readpublish4kpu)
#       print("(Step 27B/700) OF KPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4KPU ", response2manoeuvre4kpu)
#   except:
#       print("(Step 27Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())   
#">

# -------------------------------------------------------------------------------------------------   
def read2image4kpu(task4imageread2kpu=None):

    '''
    reads 2D images for KPU
    '''
    global kpu2image4xi                                                     # 2D-IMAGES FOR KPU  
    global control2readimage4kpu                                              # COMMAND AND CONTROL FOR KPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "read2image4kpu"
    method2print = "read2image4kpu"

    message2print = "{(Step 1A/33) started" + "; read2image4kpu-started: " + str(read2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function read2image4kpu at 0x00000000039595C8> 
    
    message2print = "{(Step 1B/33) passed" + "; task4imageread2kpu-to-read2image4kpu: " + str(task4imageread2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                              # INITIALIZE IMAGE FOR KPU 
            message2print = "{(Step 1C/70) created" + "; kpu2image4xi-created-by-read2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4kpu-skipped-by-read2image4kpu: " + str(read2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # skipped                                 
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 1E/70) set" + "; kpu2image4xi-set-by-read2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; kpu2image4xi-yielded-by-read2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imageread2kpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imageread2kpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imageread2kpu-created-by-read2image4kpu to: " + str(task4imageread2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                              # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4kpu-created-by-read2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4kpu at 0x0000000004737548>;
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4kpu"                                                          # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4kpu"                                                          # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-read2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; read2image4kpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4kpu', 
            #  'METHOD': 'read2image4kpu'}; time: 05:06:38 10/04/11 Pacific Daylight Time} 

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2kpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imageread2kpu-created-by-read2image4kpu to: " + str(task4imageread2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4kpu at 0x0000000004737548>, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; read2image4kpu-failed ; time: 05:06:38 10/04/11 Pacific Daylight Time}', 
            #  'CLASS': 'read2image4kpu', 
            #  'METHOD': 'read2image4kpu'}}; time: 05:06:38 10/04/11 Pacific Daylight Time}
             
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-read2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-read2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
                 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-read2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imageread2kpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2kpu-trying-by-read2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2kpu-got-by-read2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
    
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2kpu-failed-by-read2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imageread2kpu ['CONTEXT'] = context2kpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2kpu-trying-by-read2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2kpu-got-by-read2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 
                #  'ID2PLATFORM': '0000000001509568', 
                #  'PROFILING2QUEUE': 'ACK', 
                #  'EXECUTE2KERNELS4OPENCL': 'ACK', 
                #  'NATIVE2WIDTHLONG4VECTOR': 2 ...
                
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2kpu-failed-by-read2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imageread2kpu ['DEVICE'] = device2kpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2kpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2kpu-trying-by-read2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2kpu-got-by-read2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x00000000041F9320>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
                
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2kpu-failed-by-read2image4kpu: " + str(imageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imageread2kpu ['IMAGE'] = imageinfo2kpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2kpu-trying-by-read2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2kpu-got-by-read2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000004607160> 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2kpu-failed-by-read2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imageread2kpu ['QUEUE'] = queue2kpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2kpu-trying-by-read2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2kpu-got-by-read2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 408>, <read-write PipeConnection, handle 412>) 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2kpu-failed-by-read2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imageread2kpu ['PIPE'] = pipe2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2kpu-trying-by-read2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2kpu-got-by-read2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2kpu-failed-by-read2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imageread2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2kpu-trying-by-read2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                                   # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2kpu-got-by-read2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2kpu-failed-by-read2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imageread2kpu ['BUFFER'] = bufferinfo2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2kpu-trying-by-read2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2kpu-got-by-read2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 'DATA2CALLBACK': {'MESSAGE': ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2kpu-failed-by-read2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imageread2kpu ['EVENT'] = eventinfo2kpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2kpu-trying-by-read2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2kpu-set-by-read2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {}
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2kpu-failed-by-read2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imageread2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-read2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2kpu-set-by-read2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-read2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imageread2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2kpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2kpu-trying-by-read2image4kpu: " + str(blocking2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2kpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2kpu-set-by-read2image4kpu to: " + str(blocking2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2kpu-failed-by-read2image4kpu: " + str(blocking2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imageread2kpu ['BLOCKING'] = blocking2kpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2kpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2kpu-trying-by-read2image4kpu: " + str(origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2kpu['X'] = 0 
                origin2kpu['Y'] = 0  
                origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2kpu-set-by-read2image4kpu to: " + str(origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2kpu-failed-by-read2image4kpu: " + str(origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imageread2kpu ['ORIGIN'] = origin2kpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2kpu-trying-by-read2image4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2kpu-set-by-read2image4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2kpu-failed-by-read2image4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imageread2kpu ['REGION'] = region2kpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2kpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2kpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2kpu-trying-by-read2image4kpu: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2kpu = int(width2kpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2kpu-set-by-read2image4kpu to: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2kpu-failed-by-read2image4kpu: " + str(rowpitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imageread2kpu ['ROWPITCH'] = rowpitch2kpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2kpu = None
            height2kpu = 800
            size2kpu = int(width2kpu * height2kpu)                                                                  # CALCULATED MEMORY
            if size2kpu == 0:
                size2kpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2kpu-trying-by-read2image4kpu: " + str(pointer2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2kpu = Arena4Xi(size2kpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2kpu-got-by-read2image4kpu: " + str(arena2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x0000000004449EF0>
                
                size2arena = arena2kpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-read2buffer4kpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000
                
                buffer2arena = arena2kpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-read2buffer4kpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2kpu:
                    size2kpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2kpu-set-by-read2buffer4kpu to: " + str(size2kpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2kpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2kpu-set-by-read2buffer4kpu to: " + str(pointer2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x00000000041A0C90>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2kpu-failed-by-read2image4kpu: " + str(pointer2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imageread2kpu ['POINTER'] = pointer2kpu                            
            task4imageread2kpu ['SIZE']    = size2kpu                                                               # AUTOMATIC BY KPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2kpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2kpu-trying-by-read2image4kpu: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2kpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2kpu-set-by-read2image4kpu to: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2kpu-failed-by-read2image4kpu: " + str(slicepitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imageread2kpu ['SLICEPITCH'] = slicepitch2kpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2kpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-read2image4kpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                              # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4kpu-created-by-read2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "read2image4kpu"                                                          # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "read2image4kpu"                                                          # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; read2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-read2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imageread2kpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imageread2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imageread2kpu-created-by-read2image4kpu to: " + str(task4imageread2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-read2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-read2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-read2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imageread2kpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2kpu-trying-by-read2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2kpu-got-by-read2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2kpu-failed-by-read2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imageread2kpu ['CONTEXT'] = context2kpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2kpu-trying-by-read2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2kpu-got-by-read2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2kpu-failed-by-read2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imageread2kpu ['DEVICE'] = device2kpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2kpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2kpu-trying-by-read2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2kpu-got-by-read2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2kpu-failed-by-read2image4kpu: " + str(imageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imageread2kpu ['IMAGE'] = imageinfo2kpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2kpu-trying-by-read2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2kpu-got-by-read2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2kpu-failed-by-read2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imageread2kpu ['QUEUE'] = queue2kpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2kpu-trying-by-read2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2kpu-got-by-read2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2kpu-failed-by-read2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imageread2kpu ['PIPE'] = pipe2kpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2kpu-trying-by-read2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2kpu-got-by-read2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2kpu-failed-by-read2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imageread2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2kpu-trying-by-read2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                                   # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2kpu-got-by-read2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2kpu-failed-by-read2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imageread2kpu ['BUFFER'] = bufferinfo2kpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2kpu-trying-by-read2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2kpu-got-by-read2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2kpu-failed-by-read2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imageread2kpu ['EVENT'] = eventinfo2kpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2kpu-trying-by-read2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2kpu-set-by-read2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2kpu-failed-by-read2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imageread2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-read2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                      
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2kpu-set-by-read2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-read2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imageread2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2kpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2kpu-trying-by-read2image4kpu: " + str(blocking2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2kpu = 'CL_FALSE'                                                          # the read command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2kpu-set-by-read2image4kpu to: " + str(blocking2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2kpu-failed-by-read2image4kpu: " + str(blocking2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imageread2kpu ['BLOCKING'] = blocking2kpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2kpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2kpu-trying-by-read2image4kpu: " + str(origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2kpu['X'] = 0 
                origin2kpu['Y'] = 0  
                origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2kpu-set-by-read2image4kpu to: " + str(origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2kpu-failed-by-read2image4kpu: " + str(origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imageread2kpu ['ORIGIN'] = origin2kpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2kpu-trying-by-read2image4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2kpu-set-by-read2image4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2kpu-failed-by-read2image4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imageread2kpu ['REGION'] = region2kpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2kpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2kpu-trying-by-read2image4kpu: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2kpu = int(width2kpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2kpu-set-by-read2image4kpu to: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2kpu-failed-by-read2image4kpu: " + str(rowpitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imageread2kpu ['ROWPITCH'] = rowpitch2kpu     
             
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2kpu = None
            size2kpu = int(width2kpu * height2kpu)                                                                  # CALCULATED MEMORY
            if size2kpu == 0:
                size2kpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2kpu-trying-by-read2image4kpu: " + str(pointer2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2kpu = Arena4Xi(size2kpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2kpu-got-by-read2image4kpu: " + str(arena2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2kpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-read2buffer4kpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2kpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-read2buffer4kpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2kpu:
                    size2kpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2kpu-set-by-read2buffer4kpu to: " + str(size2kpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2kpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2kpu-set-by-read2buffer4kpu to: " + str(pointer2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2kpu-failed-by-read2image4kpu: " + str(pointer2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imageread2kpu ['POINTER'] = pointer2kpu                            
            task4imageread2kpu ['SIZE']    = size2kpu                                                               # AUTOMATIC BY KPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2kpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2kpu-trying-by-read2image4kpu: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2kpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2kpu-set-by-read2image4kpu to: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2kpu-failed-by-read2image4kpu: " + str(slicepitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imageread2kpu ['SLICEPITCH'] = slicepitch2kpu
                              
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imageread2kpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-read2image4kpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 05:25:42 10/04/11 Pacific Daylight Time}                
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imageread2kpu-yielded-by-read2image4kpu: " + str(task4imageread2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2readimage4kpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2kpu = False
    channels2kpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2kpu-trying-by-read2image4kpu: " + str(channels2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>) 
        consumerchannel2kpu, publisherchannel2kpu = task4imageread2kpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2kpu-got-by-read2image4kpu: " + str(consumerchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 

        message2print = "{(Step 40C/70) got" + "; publisherchannel2kpu-got-by-read2image4kpu: " + str(publisherchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        #                      
        ack4channels2kpu = True
        
        control2readimage4kpu['CONSUMERCHANNEL']  = consumerchannel2kpu
        
        
        control2readimage4kpu['PUBLISHERCHANNEL'] = publisherchannel2kpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2kpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2kpu = False
    queue2kpu = None
    message2print = "{(Step 41A/70) trying" + "; queue2kpu-trying-by-read2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2kpu = task4imageread2kpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2kpu-got-by-read2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # 
              
        ack4queue2kpu = True
        control2readimage4kpu['QUEUE']  = queue2kpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2kpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4kpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4kpu-trying-by-read2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4kpu = DummyProcess4Xi(target=manoeuvre2readimage4kpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2kpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2kpu, publisherchannel2kpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4kpu-created-by-read2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'

    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-read2image4kpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-read2image4kpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #   
        
    message2print = "{(Step 70C/70) returning" + "; control2readimage4kpu-returning-by-read2image4kpu: " + str(control2readimage4kpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003FE3B70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 416>, 
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 412>}
        
    return control2readimage4kpu
                    
#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "map2image4kpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # ESTABLISH CONTROL & COMMAND                 
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 92A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4KPU ', time4xi())           
#                    
#   response4map2image = {} 
#   task4map2image = None                                                                                  # WE ONLY WANT TO ESTABLISH CONTROL AND CONTROL FIRST
#
#   try:                               
#       response4map2image = map2image4kpu(task4map2image)                                                                                                          
#       print('(Step 92B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH MAP2IMAGE4KPU : ', response4map2image)
#    
#       queue4map2image4kpu      = response4map2image['QUEUE']
#       channel4map2imagepublish4kpu = response4map2image['PUBLISHERCHANNEL']
#       channel4map2imageconsume4kpu = response4map2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 92Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH MAP2IMAGE4KPU ', time4xi()) 
#        
#   # -----------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2COPYIMAGE4KPU TO RECEIVE                 
#   # -----------------------------------------------------------------------------------------------------------      
#   print('(Step 93A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4KPU ', time4xi())      
#   try:                               
#       message2send4kpu = "<copy4xi>copy2mapimage4kpu</copy4xi>"
#       channel4map2imagepublish4kpu.send(message2send4kpu)
#       print("(Step 93B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel4map2imagepublish4kpu, " MESSAGE2SEND4KPU: " , message2send4kpu)                                                                                                
#   except:
#       print('(Step 93Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4KPU ', time4xi())   
# 
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4KPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4kpu is True:
#       print("(Step 94A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4kpu(queue4map2image4kpu, channel4map2imageconsume4kpu, channel4map2imagepublish4kpu)
#           print("(Step 94B/700) OF KPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4KPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#        except:
#           print("(Step 94Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#
#   # ---------------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND ROGER VIA PIPE                  
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 95A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4KPU ', time4xi())      
#   try:
#       message2receive4kpu = channel4map2imageconsume4kpu.recv()
#       print("(Step 95B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel4map2imageconsume4kpu, " CONTROL & COMMAND: ", message2receive4kpu)                                                                                                  
#   except:
#       print('(Step 95Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4KPU ', time4xi())          
#   
#   # ------------------------------------------------------------------------------------------------------------
#   # SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2COPYIMAGE4KPU TO RECEIVE                
#   # ------------------------------------------------------------------------------------------------------------      
#   print('(Step 96A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4KPU ', time4xi())     
#   try:                               
#       message2send4kpu = "<ready4xi>ready2mapimage4kpu</ready4xi>"
#       channel4map2imagepublish4kpu.send(message2send4kpu)
#       print("(Step 96B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel4map2imagepublish4kpu, " MESSAGE2SEND4KPU: " , message2send4kpu)                                                                                                
#   except:
#       print('(Step 84Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4KPU ', time4xi())                                                
# 
#   # -------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4KPU  - LET IT RECEIVE READY AND SEND READY                
#   # -------------------------------------------------------------------------------------------------------       
#   if ack2simulate4kpu is True:
#       print("(Step 97A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4kpu(queue4map2image4kpu, channel4map2imageconsume4kpu, channel4map2imagepublish4kpu)
#           print("(Step 97B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", manoeuvre2copyimage4kpu)
#       except:
#           print("(Step 97Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#            
#    
#   # ---------------------------------------------------------------------------------------
#   # RECEIVE CONTROL & COMMAND READY VIA PIPE                
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 98A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH MAP2IMAGE4KPU ', time4xi())     
#   try:
#       message2receive4kpu = channel4map2imageconsume4kpu.recv()
#       print("(Step 98B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel4map2imageconsume4kpu, " CONTROL & COMMAND: ", message2receive4kpu)                                                                                                  
#   except:
#       print('(Step 86Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH MAP2IMAGE4KPU ', time4xi())          
#                         
#   # -----------------------------------------------------------------------------------------
#   # IMAGE KPUXI COPY OPERATIONS 
#   # -----------------------------------------------------------------------------------------         
#   print('(Step 99A/700) OF KPU2IMAGE4XI5TEST - TRYING FILE OPERATIONS WITH IMAGE KPUXI ', time4xi())       
#   try:        
#       path2dragon = '..//..//..//..//..//web//images//png//dragonxi.png'
#       file2dragon = open(path2dragon)
#       print("(Step 99B/700) OF KPU2IMAGE4XI5TEST - FILENO: ", file2kpu.fileno( ))  
#       print("(Step 99C/700) OF KPU2IMAGE4XI5TEST - MODE: ", file2kpu.mode)
#       print("(Step 99D/700) OF KPU2IMAGE4XI5TEST - NAME: ", file2kpu.name)
#       offset2kpu = 10
#       file2kpu.seek(offset2kpu)
#       print("(Step 99E/700) OF KPU2IMAGE4XI5TEST - PERFORMED SEEK WITH OFFSET: ", offset2kpu)
#       file2kpu.close()
#       print("(Step 99F/700) OF DRAGON2IMAGE4XI5TEST - CLOSED IMAGE FILE: ", path2dragon)  
#   except:
#       print("(Step 87Z/700) OF DRAGON2IMAGE4XI5TEST *** FAILED TO PERFORM FILE OPERATIONS WITH IMAGE: ", path2dragon)        
# 
#   # -----------------------------------------------------------------------------------------
#   # PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 100A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue4map2image4dragon.put(path2dragon)             
#       print("(Step 100B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#   except:
#       print("(Step 88Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   
#   # -------------------------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4KPU  - STATE III - GETS FROM QUEUE AND SENDS EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4kpu is True:
#       print("(Step 101A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi()) 
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4kpu(queue4map2image4kpu, channel4map2imageconsume4kpu, channel4map2imagepublish4kpu)
#           print("(Step 101B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", manoeuvre2copyimage4kpu)
#       except:
#           print("(Step 101Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread                     
#                 
#                                  
#   # ----------------------------------------------------------------------------------------------------
#   # RECEIVE ACK OR NACK ABOUT IMAGE OPERATION CONTROL & COMMAND VIA PIPE                
#   # ----------------------------------------------------------------------------------------------------          
#   print('(Step 102A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2receive4kpu = channel4map2imageconsume4kpu.recv()
#       print("(Step 102B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel4map2imageconsume4kpu, " CONTROL & COMMAND: ", message2receive4kpu)                                                                                                   
#   except:
#       print('(Step 90Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#
#   
#   # ----------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4KPU  - LET MANOEUVRE2COPYIMAGE4KPU TO GO TO STATE 0               
#   # ----------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4kpu is True:
#       print("(Step 103A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())  
#       try:        
#           response2manoeuvre4copy = manoeuvre2mapimage4kpu(queue4map2image4kpu, channel4map2imageconsume4kpu, channel4map2imagepublish4kpu)
#           print("(Step 103B/700) OF KPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4KPU ", response2manoeuvre4copy)
#       except:
#           print("(Step 103Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread  
#                                           
#">

# -------------------------------------------------------------------------------------------------   
def map2image4kpu(task4imagemap2kpu=None):

    '''
    maps 2D images for KPU
    '''
    global kpu2image4xi                                                                                             # 2D-IMAGES FOR KPU  
    global control2mapimage4kpu                                                                                     # COMMAND AND CONTROL FOR KPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "map2image4kpu"
    method2print = "map2image4kpu"

    message2print = "{(Step 1A/33) started" + "; map2image4kpu-started: " + str(map2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 1B/33) passed" + "; task4imagemap2kpu-to-map2image4kpu: " + str(task4imagemap2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                              # INITIALIZE IMAGE FOR KPU 
            message2print = "{(Step 1C/70) created" + "; kpu2image4xi-created-by-map2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4kpu-skipped-by-map2image4kpu: " + str(map2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                                
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 1E/70) set" + "; kpu2image4xi-set-by-map2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; kpu2image4xi-yielded-by-map2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagemap2kpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR MAP - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagemap2kpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagemap2kpu-created-by-map2image4kpu to: " + str(task4imagemap2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                              # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4kpu-created-by-map2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4kpu"                                                          # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4kpu"                                                          # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-map2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2kpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagemap2kpu-created-by-map2image4kpu to: " + str(task4imagemap2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-map2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-map2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-map2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagemap2kpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2kpu-trying-by-map2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2kpu-got-by-map2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2kpu-failed-by-map2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagemap2kpu ['CONTEXT'] = context2kpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2kpu-trying-by-map2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2kpu-got-by-map2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2kpu-failed-by-map2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagemap2kpu ['DEVICE'] = device2kpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2kpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2kpu-trying-by-map2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2kpu-got-by-map2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2kpu-failed-by-map2image4kpu: " + str(imageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagemap2kpu ['IMAGE'] = imageinfo2kpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2kpu-trying-by-map2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2kpu-got-by-map2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2kpu-failed-by-map2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagemap2kpu ['QUEUE'] = queue2kpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2kpu-trying-by-map2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2kpu-got-by-map2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2kpu-failed-by-map2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagemap2kpu ['PIPE'] = pipe2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2kpu-trying-by-map2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2kpu-got-by-map2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2kpu-failed-by-map2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagemap2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2kpu-trying-by-map2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                                   # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2kpu-got-by-map2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2kpu-failed-by-map2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagemap2kpu ['BUFFER'] = bufferinfo2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2kpu-trying-by-map2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2kpu-got-by-map2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2kpu-failed-by-map2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagemap2kpu ['EVENT'] = eventinfo2kpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2kpu-trying-by-map2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2kpu-set-by-map2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2kpu-failed-by-map2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagemap2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-map2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                      
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2kpu-set-by-map2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-map2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagemap2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2kpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2kpu-trying-by-map2image4kpu: " + str(blocking2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2kpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2kpu-set-by-map2image4kpu to: " + str(blocking2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2kpu-failed-by-map2image4kpu: " + str(blocking2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagemap2kpu ['BLOCKING'] = blocking2kpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2kpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2kpu-trying-by-map2image4kpu: " + str(origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2kpu['X'] = 0 
                origin2kpu['Y'] = 0  
                origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2kpu-set-by-map2image4kpu to: " + str(origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2kpu-failed-by-map2image4kpu: " + str(origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagemap2kpu ['ORIGIN'] = origin2kpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2kpu-trying-by-map2image4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2kpu-set-by-map2image4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2kpu-failed-by-map2image4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagemap2kpu ['REGION'] = region2kpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.


            rowpitch2kpu = 0
            size2element = 8                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 10                                                                                # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 8                                                                                  # ONE DIRECTIONS IN 2D
            width2kpu = 800
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2kpu-trying-by-map2image4kpu: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"  
            print4xi(class2print, method2print, message2print) 
            # 
                     
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2kpu = int(width2kpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2kpu-set-by-map2image4kpu to: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  8000                                                                                          
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2kpu-failed-by-map2image4kpu: " + str(rowpitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagemap2kpu ['ROWPITCH'] = rowpitch2kpu     
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2kpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2kpu-trying-by-map2image4kpu: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2kpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2kpu-set-by-map2image4kpu to: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2kpu-failed-by-map2image4kpu: " + str(slicepitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagemap2kpu ['SLICEPITCH'] = slicepitch2kpu                  
            
            # ---------------------------------------------------------------------------------------------
            # (19) FLAGS - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            flags2kpu = 'CL_MAP_READ'                                                                
     
            message2print = "{(Step 19A/70) trying" + "; flags2kpu-trying-by-map2image4kpu: " + str(flags2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2kpu = 'CL_MAP_WRITE'  # THE REGION SPECIFIED BY (ORIGIN, REGION) IN THE IMAGE OBJECT IS BEING MAPPED FOR WRITING                                                                                                                     
                message2print = "{(Step 19B/70) set" + "; flags2kpu-set-by-map2image4kpu to: " + str(flags2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '19B-' 
            except:
                message2print = "{(Step 19Z/70) failed" + "; flags2kpu-failed-by-map2image4kpu: " + str(flags2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '19Z-'
        
            task4imagemap2kpu ['FLAGS'] = flags2kpu 
                                  
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # ANALYSE TASKS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2kpu.keys()
        message2print = "{(Step 20/70) got" + "; keys2task-got-by-map2image4kpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}
 

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                              # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4kpu-created-by-map2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                   # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "map2image4kpu"                                                          # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "map2image4kpu"                                                          # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; map2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-map2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 

            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagemap2kpu['DATA2CALLBACK'] = args2callback                                                   # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagemap2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                       # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagemap2kpu-created-by-map2image4kpu to: " + str(task4imagemap2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-map2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-map2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                 
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-map2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                 
            task4imagemap2kpu['PLATFORM'] = platform2xi                                                          # PLATFORM OF SYSTEM 
          
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2kpu-trying-by-map2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2kpu-got-by-map2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2kpu-failed-by-map2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagemap2kpu ['CONTEXT'] = context2kpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2kpu-trying-by-map2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2kpu-got-by-map2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2kpu-failed-by-map2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagemap2kpu ['DEVICE'] = device2kpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2kpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2kpu-trying-by-map2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2kpu-got-by-map2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2kpu-failed-by-map2image4kpu: " + str(imageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagemap2kpu ['IMAGE'] = imageinfo2kpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2kpu-trying-by-map2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                                  # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2kpu-got-by-map2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2kpu-failed-by-map2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagemap2kpu ['QUEUE'] = queue2kpu
        
        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2kpu-trying-by-map2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                                    # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2kpu-got-by-map2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2kpu-failed-by-map2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagemap2kpu ['PIPE'] = pipe2kpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2kpu-trying-by-map2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                    # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2kpu-got-by-map2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2kpu-failed-by-map2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagemap2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
        
        
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2kpu-trying-by-map2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                                   # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2kpu-got-by-map2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2kpu-failed-by-map2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagemap2kpu ['BUFFER'] = bufferinfo2kpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2kpu-trying-by-map2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2kpu-got-by-map2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2kpu-failed-by-map2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagemap2kpu ['EVENT'] = eventinfo2kpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2kpu-trying-by-map2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2kpu-set-by-map2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2kpu-failed-by-map2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagemap2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     
              
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-map2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                   
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2kpu-set-by-map2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-map2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagemap2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2kpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2kpu-trying-by-map2image4kpu: " + str(blocking2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2kpu = 'CL_FALSE'                                                          # the map command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2kpu-set-by-map2image4kpu to: " + str(blocking2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2kpu-failed-by-map2image4kpu: " + str(blocking2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagemap2kpu ['BLOCKING'] = blocking2kpu             
             
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2kpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2kpu-trying-by-map2image4kpu: " + str(origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2kpu['X'] = 0 
                origin2kpu['Y'] = 0  
                origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2kpu-set-by-map2image4kpu to: " + str(origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2kpu-failed-by-map2image4kpu: " + str(origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagemap2kpu ['ORIGIN'] = origin2kpu
            
             
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2kpu-trying-by-map2image4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                            
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO MAP 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO MAP 
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO MAP                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2kpu-set-by-map2image4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2kpu-failed-by-map2image4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagemap2kpu ['REGION'] = region2kpu
        
            
             
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2kpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2kpu-trying-by-map2image4kpu: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2kpu = int(width2kpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2kpu-set-by-map2image4kpu to: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2kpu-failed-by-map2image4kpu: " + str(rowpitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagemap2kpu ['ROWPITCH'] = rowpitch2kpu     
   
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING MAP                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2kpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2kpu-trying-by-map2image4kpu: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2kpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2kpu-set-by-map2image4kpu to: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2kpu-failed-by-map2image4kpu: " + str(slicepitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagemap2kpu ['SLICEPITCH'] = slicepitch2kpu
   
        if 'FLAGS' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (39) FLAGS - REQUIRED                   
            # --------------------------------------------------------------------------------------------------------             
            flags2kpu = 'CL_MAP_READ'                                                                                                             
           
            message2print = "{(Step 39A/70) trying" + "; flags2kpu-trying-by-map2image4kpu: " + str(flags2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                flags2kpu = 'CL_MAP_WRITE'                                                          
                message2print = "{(Step 39B/70) set" + "; flags2kpu-set-by-map2image4kpu to: " + str(flags2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '39B-'                
                 
            except:
                message2print = "{(Step 39Z/70) failed" + "; flags2kpu-failed-by-map2image4kpu: " + str(flags2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '39Z-'
            
            task4imagemap2kpu ['FLAGS'] = flags2kpu
                                          
        # ---------------------------------------------------------------------------------------------
        # EOF TASK ANALYSIS
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagemap2kpu.keys()
        message2print = "{(Step 40A/70) set" + "; keys2task-set-by-map2image4kpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FLAGS',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SLICEPITCH']; time: 01:28:44 10/05/11 Pacific Daylight Time}

                       
    except: 
        message2print = "{(Step 40Z/70) yielded" + "; task4imagemap2kpu-yielded-by-map2image4kpu: " + str(task4imagemap2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '40Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2mapimage4kpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2kpu = False
    channels2kpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2kpu-trying-by-map2image4kpu: " + str(channels2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2kpu, publisherchannel2kpu = task4imagemap2kpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2kpu-got-by-map2image4kpu: " + str(consumerchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 748>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2kpu-got-by-map2image4kpu: " + str(publisherchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 752>                     
        ack4channels2kpu = True
        
        control2mapimage4kpu['CONSUMERCHANNEL']  = consumerchannel2kpu
        
        
        control2mapimage4kpu['PUBLISHERCHANNEL'] = publisherchannel2kpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2kpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2kpu = False
    queue2kpu = None
    message2print = "{(Step 41A/70) trying" + "queue2kpu-trying-by-map2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2kpu = task4imagemap2kpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2kpu-got-by-map2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000004B3FB70> 
              
        ack4queue2kpu = True
        control2mapimage4kpu['QUEUE']  = queue2kpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2kpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4kpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4kpu-trying-by-map2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4kpu = DummyProcess4Xi(target=manoeuvre2mapimage4kpu,                                            # in this module to handle queue and pipe channels
        #                                   args =(queue2kpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2kpu, publisherchannel2kpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4kpu-created-by-map2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-map2image4kpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-map2image4kpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-18B-19B-40D-41C- 
        
    message2print = "{(Step 70C/70) returning" + "; control2mapimage4kpu-returning-by-map2image4kpu: " + str(control2mapimage4kpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004B3FB70>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 752>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 748>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
    return control2mapimage4kpu
                    
#</function4xi> 

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimage4kpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4KPU - LET IT TO RECEIVE COPY MESSAGE AND TO SEND ROGER MESSAGE VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4kpu is True:
#       print("(Step 42A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimage4kpu(queue2copy4kpu, channel2copyconsume4kpu, channel2copypublish4kpu)
#           print("(Step 42B/700) OF KPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4KPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4KPU': '<roger4xi>roger2copy4kpu</roger4xi>', 'ACK2INFO4KPU': False, 'MESSAGE2GET4KPU': '<default4xi>default2copy4kpu</default4xi>', 
#           #  'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2copy4kpu</copy4xi>', 
#           'CODE2ACK4KPU': ' 0D-1C-', 'MESSAGE2PUT4KPU': '<default4xi>default2copy4kpu</default4xi>', 'CODE2NACK4KPU': ' ', 'STATE2MANOEUVRE4KPU': 1}
#        except:
#           print("(Step 42Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread
#">            

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimage4kpu(queue2copymanoeuvre4kpu, consumerchannel2copymanoeuvre4kpu, publisherchannel2copymanoeuvre4kpu):
    '''
    manoeuvres copy control and command
    '''
    global state2copyimage4kpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagestate4kpu                                                                                        # MAX FOR STATE
    global ack2copyimageinfo4kpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimageget4kpu                                 
    global message2copyimageput4kpu
    global message2copyimagereceive4kpu
    global message2copyimagesend4kpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimage4kpu" 
    method2print = "manoeuvre2copyimage4kpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimage4kpu-started: " + str(manoeuvre2copyimage4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2copyimage4kpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2copymanoeuvre4kpu-to-manoeuvre2copyimage4kpu: " + str(queue2copymanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copymanoeuvre4kpu-to-manoeuvre2copyimage4kpu: " + str(consumerchannel2copymanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copymanoeuvre4kpu-to-manoeuvre2copyimage4kpu: " + str(publisherchannel2copymanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <copy-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copy = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimage4kpu-try-by-manoeuvre2copyimage4kpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimage4kpu is None:
            state2copyimage4kpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimage4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str(state2copyimage4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimage4kpu-used-by-manoeuvre2copyimage4kpu: " + str(state2copyimage4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimage4kpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimage4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str(state2copyimage4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4kpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimageinfo4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str(ack2copyimageinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4kpu     = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimageget4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str(message2copyimageget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4kpu</default4xi>
        message2copyimageput4kpu     = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimageput4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str(message2copyimageput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4kpu</default4xi>
        message2copyimagereceive4kpu = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagereceive4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str(message2copyimagereceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4kpu</default4xi>
        message2copyimagesend4kpu    = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagesend4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str(message2copyimagesend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4kpu</default4xi>
        max2copyimagestate4kpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagestate4kpu-initialized-by-manoeuvre2copyimage4kpu to: " + str( max2copyimagestate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimage4kpu = state2copyimage4kpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimage4kpu-added-by-manoeuvre2copyimage4kpu: " + str(state2copyimage4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2copyimage4kpu > max2copyimagestate4kpu:
        state2copyimage4kpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimage4kpu-set-by-manoeuvre2copyimage4kpu to: " + str(state2copyimage4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimageinfo4kpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimageinfo4kpu-set-by-manoeuvre2copyimage4kpu to: " + str(ack2copyimageinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimageget4kpu     = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimageget4kpu-set-by-manoeuvre2copyimage4kpu to: " + str(message2copyimageget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2copy4kpu</default4xi>
        message2copyimageput4kpu     = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimageput4kpu-set-by-manoeuvre2copyimage4kpu to: " + str(message2copyimageput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4kpu</default4xi>
        message2copyimagereceive4kpu = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagereceive4kpu-set-by-manoeuvre2copyimage4kpu to: " + str(message2copyimagereceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4kpu</default4xi>
        message2copyimagesend4kpu    = '<default4xi>default2copy4kpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagesend4kpu-set-by-manoeuvre2copyimage4kpu to: " + str(message2copyimagesend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2copy4kpu</default4xi>
        max2copyimagestate4kpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagestate4kpu-set-by-manoeuvre2copyimage4kpu to: " + str( max2copyimagestate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4kpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4kpu = consumerchannel2copymanoeuvre4kpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagereceive4kpu-received-by-manoeuvre2copyimage4kpu: " + str(message2copyimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2copy4kpu</copy4xi>
                         
            if message2copyimagereceive4kpu == '<copy4xi>copy2copy4kpu</copy4xi>':
                message2copyimagesend4kpu = '<roger4xi>roger2copy4kpu</roger4xi>'
            else:
                message2copyimagesend4kpu = '<negative4xi><negative2copy4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4kpu.send(message2copyimagesend4kpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagesend4kpu-sent-by-manoeuvre2copyimage4kpu: " + str(message2copyimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2copy4kpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagereceive4kpu-failed-by-manoeuvre2copyimage4kpu: " + str(message2copyimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4kpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagereceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagereceive4kpu = consumerchannel2copymanoeuvre4kpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagereceive4kpu-received-by-manoeuvre2copyimage4kpu: " + str(message2copyimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2copy4kpu</ready4xi>
                         
            if message2copyimagereceive4kpu == '<ready4xi>ready2copy4kpu</ready4xi>':
                message2copyimagesend4kpu = '<ready4xi>ready2copy4kpu</ready4xi>'
            else:
                message2copyimagesend4kpu = '<negative4xi><negative2copy4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copymanoeuvre4kpu.send(message2copyimagesend4kpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagesend4kpu-sent-by-manoeuvre2copyimage4kpu: " + str(message2copyimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagereceive4kpu-failed-by-manoeuvre2copyimage4kpu: " + str(message2copyimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimage4kpu == 3:    
        info2image4kpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4kpu-try-by-manoeuvre2copyimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4kpu = queue2copymanoeuvre4kpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4kpu-got-by-manoeuvre2copyimage4kpu: " + str(info2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4kpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimageinfo4kpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimageinfo4kpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copymanoeuvre4kpu-failed-by-manoeuvre2copyimage4kpu: " + str(queue2copymanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagesend4kpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimageinfo4kpu is True:
                message2copyimagesend4kpu = '<ack4xi>ack2imageinfo4kpu</ack4xi>'
            else:
                message2copyimagesend4kpu = '<nack4xi>nack2imageinfo4kpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagesend4kpu-set-by-manoeuvre2copyimage4kpu to: " + str(message2copyimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4kpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4kpu-failed-by-manoeuvre2copyimage4kpu: " + str(message2copyimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copymanoeuvre4kpu-calling-by-manoeuvre2copyimage4kpu: " + str(publisherchannel2copymanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagesend4kpu-passing-to-send-by-manoeuvre2copyimage4kpu: " + str(message2copyimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagesend4kpu-failed-by-manoeuvre2copyimage4kpu: " + str(message2copyimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copymanoeuvre4kpu.send(message2copyimagesend4kpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagesend4kpu-sent-by-manoeuvre2copyimage4kpu: " + str(message2copyimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4kpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagesend4kpu-failed-by-manoeuvre2copyimage4kpu: " + str(message2copyimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copy['ACK2INFO4KPU']        = ack2copyimageinfo4kpu
    response2manouvre4copy['CODE2ACK4KPU']        = code2ack
    response2manouvre4copy['CODE2NACK4KPU']       = code2nack
    response2manouvre4copy['MESSAGE2GET4KPU']     = message2copyimageget4kpu
    response2manouvre4copy['MESSAGE2PUT4KPU']     = message2copyimageput4kpu
    response2manouvre4copy['MESSAGE2RECEIVE4KPU'] = message2copyimagereceive4kpu
    response2manouvre4copy['MESSAGE2SEND4KPU']    = message2copyimagesend4kpu
    response2manouvre4copy['STATE2MANOEUVRE4KPU'] = state2copyimage4kpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copy-returning-by-manoeuvre2copyimage4kpu: " + str(response2manouvre4copy) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4KPU': '<roger4xi>roger2copy4kpu</roger4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2copy4kpu</copy4xi>',
    #   'CODE2ACK4KPU': ' 0D-1C-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 1}; time: 05:25:15 10/03/11 Pacific Daylight Time}

 
    #2 {'MESSAGE2SEND4KPU': '<ready4xi>ready2copy4kpu</ready4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2copy4kpu</ready4xi>',
    #   'CODE2ACK4KPU': ' 0D-2C-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 2}; time: 05:25:15 10/03/11 Pacific Daylight Time}

   
    #3 {'MESSAGE2SEND4KPU': '<ack4xi>ack2imageinfo4kpu</ack4xi>',
    #   'ACK2INFO4KPU': True,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2copy4kpu</ready4xi>',
    #   'CODE2ACK4KPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ', 'STATE2MANOEUVRE4KPU': 3}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    
    #4 {'MESSAGE2SEND4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'CODE2ACK4KPU': ' 0D-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copy4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ', 'STATE2MANOEUVRE4KPU': 0}; time: 05:25:15 10/03/11 Pacific Daylight Time}

    return response2manouvre4copy           
          
#</function4xi>


# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copyimagetobuffer4kpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # CALL MANOEUVRE2COPYIMAGE4KPU  - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4kpu is True:
#       print("(Step 62A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copyimagetobuffer4kpu(queue4copy2imagetobuffer4kpu, channel4copy2imagetobufferconsume4kpu, channel4copy2imagetobufferpublish4kpu)
#           print("(Step 62B/700) OF KPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4KPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # { 'MESSAGE2SEND4KPU': '<roger4xi>roger2copyimagetobuffer4kpu</roger4xi>',
#           #   'ACK2INFO4KPU': False,
#           #   'MESSAGE2GET4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
#           #   'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2copyimagetobuffer4kpu</copy4xi>',
#           #   'CODE2ACK4KPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
#           #   'CODE2NACK4KPU': ' ',
#           #   'STATE2MANOEUVRE4KPU': 1}
#        except:
#           print("(Step 62Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copyimagetobuffer4kpu(queue2copyimagetobuffermanoeuvre4kpu, consumerchannel2copyimagetobuffermanoeuvre4kpu, publisherchannel2copyimagetobuffermanoeuvre4kpu):
    '''
    manoeuvres copy control and command for copy2imagetobuffer4kpu
    '''
    global state2copyimagetobuffer4kpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copyimagetobufferstate4kpu                                                                                        # MAX FOR STATE
    global ack2copyimagetobufferinfo4kpu                                                                                         # ACK FOR IMAGE INFO
    global message2copyimagetobufferget4kpu                                 
    global message2copyimagetobufferput4kpu
    global message2copyimagetobufferreceive4kpu
    global message2copyimagetobuffersend4kpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copyimagetobuffer4kpu" 
    method2print = "manoeuvre2copyimagetobuffer4kpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copyimagetobuffer4kpu-started: " + str(manoeuvre2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copyimagetobuffermanoeuvre4kpu-to-manoeuvre2copyimagetobuffer4kpu: " + str(queue2copyimagetobuffermanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copyimagetobuffermanoeuvre4kpu-to-manoeuvre2copyimagetobuffer4kpu: " + str(consumerchannel2copyimagetobuffermanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copyimagetobuffermanoeuvre4kpu-to-manoeuvre2copyimagetobuffer4kpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copyimagetobuffer = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copyimagetobuffer4kpu-try-by-manoeuvre2copyimagetobuffer4kpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copyimagetobuffer4kpu is None:
            state2copyimagetobuffer4kpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copyimagetobuffer4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str(state2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copyimagetobuffer4kpu-used-by-manoeuvre2copyimagetobuffer4kpu: " + str(state2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copyimagetobuffer4kpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copyimagetobuffer4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str(state2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copyimagetobufferinfo4kpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copyimagetobufferinfo4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str(ack2copyimagetobufferinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copyimagetobufferget4kpu     = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copyimagetobufferget4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobufferget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4kpu     = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copyimagetobufferput4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobufferput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4kpu = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copyimagetobufferreceive4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobufferreceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4kpu    = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copyimagetobuffersend4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobuffersend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4kpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copyimagetobufferstate4kpu-initialized-by-manoeuvre2copyimagetobuffer4kpu to: " + str( max2copyimagetobufferstate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copyimagetobuffer4kpu = state2copyimagetobuffer4kpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copyimagetobuffer4kpu-added-by-manoeuvre2copyimagetobuffer4kpu: " + str(state2copyimagetobuffer4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copyimagetobuffer4kpu > max2copyimagetobufferstate4kpu:
        state2copyimagetobuffer4kpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copyimagetobuffer4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str(state2copyimagetobuffer4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copyimagetobufferinfo4kpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copyimagetobufferinfo4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str(ack2copyimagetobufferinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copyimagetobufferget4kpu     = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copyimagetobufferget4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobufferget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copyimagetobufferput4kpu     = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copyimagetobufferput4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobufferput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobufferreceive4kpu = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copyimagetobufferreceive4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobufferreceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copyimagetobuffersend4kpu    = '<default4xi>default2copyimagetobuffer4kpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copyimagetobuffersend4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobuffersend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copyimagetobufferstate4kpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copyimagetobufferstate4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str( max2copyimagetobufferstate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4kpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4kpu = consumerchannel2copyimagetobuffermanoeuvre4kpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copyimagetobufferreceive4kpu-received-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobufferreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4kpu == '<copy4xi>copy2copyimagetobuffer4kpu</copy4xi>': 
                message2copyimagetobuffersend4kpu = '<roger4xi>roger2copyimagetobuffer4kpu</roger4xi>'
            else:
                message2copyimagetobuffersend4kpu = '<negative4xi><negative2copyimagetobuffer4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4kpu.send(message2copyimagetobuffersend4kpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copyimagetobuffersend4kpu-sent-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobuffersend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copyimagetobufferreceive4kpu-failed-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobufferreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4kpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copyimagetobuffer4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copyimagetobufferreceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copyimagetobufferreceive4kpu = consumerchannel2copyimagetobuffermanoeuvre4kpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copyimagetobufferreceive4kpu-received-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobufferreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copyimagetobufferreceive4kpu == '<ready4xi>ready2copyimagetobuffer4kpu</ready4xi>':
                message2copyimagetobuffersend4kpu = '<ready4xi>ready2copyimagetobuffer4kpu</ready4xi>'
            else:
                message2copyimagetobuffersend4kpu = '<negative4xi><negative2copyimagetobuffer4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copyimagetobuffermanoeuvre4kpu.send(message2copyimagetobuffersend4kpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copyimagetobuffersend4kpu-sent-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobuffersend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copyimagetobufferreceive4kpu-failed-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobufferreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copyimagetobuffer4kpu == 3:    
        info2image4kpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4kpu-try-by-manoeuvre2copyimagetobuffer4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4kpu = queue2copyimagetobuffermanoeuvre4kpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4kpu-got-by-manoeuvre2copyimagetobuffer4kpu: " + str(info2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4kpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copyimagetobufferinfo4kpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copyimagetobufferinfo4kpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copyimagetobuffermanoeuvre4kpu-failed-by-manoeuvre2copyimagetobuffer4kpu: " + str(queue2copyimagetobuffermanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copyimagetobuffersend4kpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copyimagetobufferinfo4kpu is True:
                message2copyimagetobuffersend4kpu = '<ack4xi>ack2imageinfo4kpu</ack4xi>'
            else:
                message2copyimagetobuffersend4kpu = '<nack4xi>nack2imageinfo4kpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copyimagetobuffersend4kpu-set-by-manoeuvre2copyimagetobuffer4kpu to: " + str(message2copyimagetobuffersend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4kpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4kpu-failed-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobuffersend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copyimagetobuffermanoeuvre4kpu-calling-by-manoeuvre2copyimagetobuffer4kpu: " + str(publisherchannel2copyimagetobuffermanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copyimagetobuffersend4kpu-passing-to-send-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobuffersend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copyimagetobuffersend4kpu-failed-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobuffersend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copyimagetobuffermanoeuvre4kpu.send(message2copyimagetobuffersend4kpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copyimagetobuffersend4kpu-sent-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobuffersend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4kpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copyimagetobuffersend4kpu-failed-by-manoeuvre2copyimagetobuffer4kpu: " + str(message2copyimagetobuffersend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copyimagetobuffer['ACK2INFO4KPU']        = ack2copyimagetobufferinfo4kpu
    response2manouvre4copyimagetobuffer['CODE2ACK4KPU']        = code2ack
    response2manouvre4copyimagetobuffer['CODE2NACK4KPU']       = code2nack
    response2manouvre4copyimagetobuffer['MESSAGE2GET4KPU']     = message2copyimagetobufferget4kpu
    response2manouvre4copyimagetobuffer['MESSAGE2PUT4KPU']     = message2copyimagetobufferput4kpu
    response2manouvre4copyimagetobuffer['MESSAGE2RECEIVE4KPU'] = message2copyimagetobufferreceive4kpu
    response2manouvre4copyimagetobuffer['MESSAGE2SEND4KPU']    = message2copyimagetobuffersend4kpu
    response2manouvre4copyimagetobuffer['STATE2MANOEUVRE4KPU'] = state2copyimagetobuffer4kpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copyimagetobuffer-returning-by-manoeuvre2copyimagetobuffer4kpu: " + str(response2manouvre4copyimagetobuffer) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4KPU': '<roger4xi>roger2copyimagetobuffer4kpu</roger4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2copyimagetobuffer4kpu</copy4xi>',
    #   'CODE2ACK4KPU': ' 0D-1C-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 1}; time: 02:00:20 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4KPU': '<ready4xi>ready2copyimagetobuffer4kpu</ready4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2copyimagetobuffer4kpu</ready4xi>',
    #   'CODE2ACK4KPU': ' 0D-2C-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 2}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #3 {'MESSAGE2SEND4KPU': '<ack4xi>ack2imageinfo4kpu</ack4xi>',
    #   'ACK2INFO4KPU': True,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2copyimagetobuffer4kpu</ready4xi>',
    #   'CODE2ACK4KPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 3}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    #4 {'MESSAGE2SEND4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'CODE2ACK4KPU': ' 0D-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copyimagetobuffer4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 0}; time: 02:10:33 10/04/11 Pacific Daylight Time}


    return response2manouvre4copyimagetobuffer           
          
#</function4xi>

# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2copybuffertoimage4kpu" usage = "
#   # ---------------------------------------------------------------------------------------------------------------------
#   # SIMULATE MANOEUVRE2COPYIMAGE4KPU - LET IT TO RECEIVE COPY AND TO SEND ROGER VIA PIPE                
#   # ---------------------------------------------------------------------------------------------------------------------       
#   if ack2simulate4kpu is True:
#       print("(Step 82A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())   
#       try:        
#           response2manoeuvre4copy = manoeuvre2copybuffertoimage4kpu(queue4copybuffertoimage4kpu, channel4copybuffertoimageconsume4kpu, channel4copybuffertoimagepublish4kpu)
#           print("(Step 82B/700) OF KPU2IMAGE4XI5TEST - PERFORMED MANOEUVRE2COPYIMAGE4KPU, RESPONSE2MANOEUVRE4COPY: ", response2manoeuvre4copy)
#           # {'MESSAGE2SEND4KPU': '<roger4xi>roger2copybuffertoimage4kpu</roger4xi>',
#           #   'ACK2INFO4KPU': False,
#           #   'MESSAGE2GET4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
#           #   'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2copybuffertoimage4kpu</copy4xi>',
#           #   'CODE2ACK4KPU': ' 0D-1C-',
#           #   'MESSAGE2PUT4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
#           #   'CODE2NACK4KPU': ' ',
#           #   'STATE2MANOEUVRE4KPU': 1}
#       except:
#           print("(Step 82Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2COPYIMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2copybuffertoimage4kpu(queue2copybuffertoimagemanoeuvre4kpu, consumerchannel2copybuffertoimagemanoeuvre4kpu, publisherchannel2copybuffertoimagemanoeuvre4kpu):
    '''
    manoeuvres copy control and command for copy2buffertoimage4kpu
    '''
    global state2copybuffertoimage4kpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2copybuffertoimagestate4kpu                                                                                        # MAX FOR STATE
    global ack2copybuffertoimageinfo4kpu                                                                                         # ACK FOR IMAGE INFO
    global message2copybuffertoimageget4kpu                                 
    global message2copybuffertoimageput4kpu
    global message2copybuffertoimagereceive4kpu
    global message2copybuffertoimagesend4kpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2copybuffertoimage4kpu" 
    method2print = "manoeuvre2copybuffertoimage4kpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2copybuffertoimage4kpu-started: " + str(manoeuvre2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # 
    
    message2print = "{(Step 0B/33) passed" + "; queue2copybuffertoimagemanoeuvre4kpu-to-manoeuvre2copybuffertoimage4kpu: " + str(queue2copybuffertoimagemanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2copybuffertoimagemanoeuvre4kpu-to-manoeuvre2copybuffertoimage4kpu: " + str(consumerchannel2copybuffertoimagemanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2copybuffertoimagemanoeuvre4kpu-to-manoeuvre2copybuffertoimage4kpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4copybuffertoimage = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2copybuffertoimage4kpu-try-by-manoeuvre2copybuffertoimage4kpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2copybuffertoimage4kpu is None:
            state2copybuffertoimage4kpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2copybuffertoimage4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str(state2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2copybuffertoimage4kpu-used-by-manoeuvre2copybuffertoimage4kpu: " + str(state2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2copybuffertoimage4kpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2copybuffertoimage4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str(state2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2copybuffertoimageinfo4kpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2copybuffertoimageinfo4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str(ack2copybuffertoimageinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2copybuffertoimageget4kpu     = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2copybuffertoimageget4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimageget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4kpu     = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2copybuffertoimageput4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimageput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4kpu = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2copybuffertoimagereceive4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimagereceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4kpu    = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2copybuffertoimagesend4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimagesend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4kpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2copybuffertoimagestate4kpu-initialized-by-manoeuvre2copybuffertoimage4kpu to: " + str( max2copybuffertoimagestate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2copybuffertoimage4kpu = state2copybuffertoimage4kpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2copybuffertoimage4kpu-added-by-manoeuvre2copybuffertoimage4kpu: " + str(state2copybuffertoimage4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4 
     
    if state2copybuffertoimage4kpu > max2copybuffertoimagestate4kpu:
        state2copybuffertoimage4kpu = 0
        message2print = "{(Step 0I/33) set" + "; state2copybuffertoimage4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str(state2copybuffertoimage4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
        ack2copybuffertoimageinfo4kpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2copybuffertoimageinfo4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str(ack2copybuffertoimageinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # 
        message2copybuffertoimageget4kpu     = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2copybuffertoimageget4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimageget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # 
        message2copybuffertoimageput4kpu     = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2copybuffertoimageput4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimageput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagereceive4kpu = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2copybuffertoimagereceive4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimagereceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        message2copybuffertoimagesend4kpu    = '<default4xi>default2copybuffertoimage4kpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2copybuffertoimagesend4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimagesend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # 
        max2copybuffertoimagestate4kpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2copybuffertoimagestate4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str( max2copybuffertoimagestate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4kpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4kpu = consumerchannel2copybuffertoimagemanoeuvre4kpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2copybuffertoimagereceive4kpu-received-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4kpu == '<copy4xi>copy2copybuffertoimage4kpu</copy4xi>':
                message2copybuffertoimagesend4kpu = '<roger4xi>roger2copybuffertoimage4kpu</roger4xi>'
            else:
                message2copybuffertoimagesend4kpu = '<negative4xi><negative2copybuffertoimage4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4kpu.send(message2copybuffertoimagesend4kpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2copybuffertoimagesend4kpu-sent-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2copybuffertoimagereceive4kpu-failed-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4kpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2copybuffertoimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2copybuffertoimagereceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2copybuffertoimagereceive4kpu = consumerchannel2copybuffertoimagemanoeuvre4kpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2copybuffertoimagereceive4kpu-received-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2copybuffertoimagereceive4kpu == '<ready4xi>ready2copybuffertoimage4kpu</ready4xi>':
                message2copybuffertoimagesend4kpu = '<ready4xi>ready2copybuffertoimage4kpu</ready4xi>'
            else:
                message2copybuffertoimagesend4kpu = '<negative4xi><negative2copybuffertoimage4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND READY OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2copybuffertoimagemanoeuvre4kpu.send(message2copybuffertoimagesend4kpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2copybuffertoimagesend4kpu-sent-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2copybuffertoimagereceive4kpu-failed-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2copybuffertoimage4kpu == 3:    
        info2image4kpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4kpu-try-by-manoeuvre2copybuffertoimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4kpu = queue2copybuffertoimagemanoeuvre4kpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4kpu-got-by-manoeuvre2copybuffertoimage4kpu: " + str(info2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4kpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2copybuffertoimageinfo4kpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2copybuffertoimageinfo4kpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2copybuffertoimagemanoeuvre4kpu-failed-by-manoeuvre2copybuffertoimage4kpu: " + str(queue2copybuffertoimagemanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2copybuffertoimagesend4kpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2copybuffertoimageinfo4kpu is True:
                message2copybuffertoimagesend4kpu = '<ack4xi>ack2imageinfo4kpu</ack4xi>'
            else:
                message2copybuffertoimagesend4kpu = '<nack4xi>nack2imageinfo4kpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2copybuffertoimagesend4kpu-set-by-manoeuvre2copybuffertoimage4kpu to: " + str(message2copybuffertoimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4kpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4kpu-failed-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2copybuffertoimagemanoeuvre4kpu-calling-by-manoeuvre2copybuffertoimage4kpu: " + str(publisherchannel2copybuffertoimagemanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <copy-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2copybuffertoimagesend4kpu-passing-to-send-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2copybuffertoimagesend4kpu-failed-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2copybuffertoimagemanoeuvre4kpu.send(message2copybuffertoimagesend4kpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2copybuffertoimagesend4kpu-sent-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4kpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2copybuffertoimagesend4kpu-failed-by-manoeuvre2copybuffertoimage4kpu: " + str(message2copybuffertoimagesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4copybuffertoimage['ACK2INFO4KPU']        = ack2copybuffertoimageinfo4kpu
    response2manouvre4copybuffertoimage['CODE2ACK4KPU']        = code2ack
    response2manouvre4copybuffertoimage['CODE2NACK4KPU']       = code2nack
    response2manouvre4copybuffertoimage['MESSAGE2GET4KPU']     = message2copybuffertoimageget4kpu
    response2manouvre4copybuffertoimage['MESSAGE2PUT4KPU']     = message2copybuffertoimageput4kpu
    response2manouvre4copybuffertoimage['MESSAGE2RECEIVE4KPU'] = message2copybuffertoimagereceive4kpu
    response2manouvre4copybuffertoimage['MESSAGE2SEND4KPU']    = message2copybuffertoimagesend4kpu
    response2manouvre4copybuffertoimage['STATE2MANOEUVRE4KPU'] = state2copybuffertoimage4kpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4copybuffertoimage-returning-by-manoeuvre2copybuffertoimage4kpu: " + str(response2manouvre4copybuffertoimage) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4KPU': '<roger4xi>roger2copybuffertoimage4kpu</roger4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2copybuffertoimage4kpu</copy4xi>',
    #   'CODE2ACK4KPU': ' 0D-1C-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 1}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    
    #2 {'MESSAGE2SEND4KPU': '<ready4xi>ready2copybuffertoimage4kpu</ready4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2copybuffertoimage4kpu</ready4xi>',
    #   'CODE2ACK4KPU': ' 0D-2C-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 2}; time: 02:18:55 10/04/11 Pacific Daylight Time}

    #3 {'MESSAGE2SEND4KPU': '<ack4xi>ack2imageinfo4kpu</ack4xi>',
    #   'ACK2INFO4KPU': True,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2copybuffertoimage4kpu</ready4xi>',
    #   'CODE2ACK4KPU': ' 0D-3C-4E-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 3}; time: 02:18:55 10/04/11 Pacific Daylight Time}
 

    #4 {'MESSAGE2SEND4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'ACK2INFO4KPU': False,
    #   'MESSAGE2GET4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'MESSAGE2RECEIVE4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'CODE2ACK4KPU': ' 0D-',
    #   'MESSAGE2PUT4KPU': '<default4xi>default2copybuffertoimage4kpu</default4xi>',
    #   'CODE2NACK4KPU': ' ',
    #   'STATE2MANOEUVRE4KPU': 0}

    return response2manouvre4copybuffertoimage           
          
#</function4xi>



# -----------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2readimage4kpu" usage = "control function for read2image4kpu function in this module">

# ----------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2readimage4kpu(queue2readmanoeuvre4kpu, consumerchannel2readmanoeuvre4kpu, publisherchannel2readmanoeuvre4kpu):
    '''
    manoeuvres read control and command
    '''
    global state2read4kpu                                                                                           # STATE FOR COMMAND AND CONTROL
    global max2readstate4kpu                                                                                        # MAX FOR STATE
    global ack2readinfo4kpu                                                                                         # ACK FOR IMAGE INFO
    global message2readget4kpu                                 
    global message2readput4kpu
    global message2readreceive4kpu
    global message2readsend4kpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2readimage4kpu" 
    method2print = "manoeuvre2readimage4kpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2readimage4kpu-started: " + str(manoeuvre2readimage4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function manoeuvre2readimage4kpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2readmanoeuvre4kpu-to-manoeuvre2readimage4kpu: " + str(queue2readmanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x0000000003AD9400>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2readmanoeuvre4kpu-to-manoeuvre2readimage4kpu: " + str(consumerchannel2readmanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 408>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2readmanoeuvre4kpu-to-manoeuvre2readimage4kpu: " + str(publisherchannel2readmanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 412>     
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4read = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2read4kpu-try-by-manoeuvre2readimage4kpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2read4kpu is None:
            state2read4kpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2read4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str(state2read4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2read4kpu-used-by-manoeuvre2readimage4kpu: " + str(state2read4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2read4kpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2read4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str(state2read4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4kpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2readinfo4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str(ack2readinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4kpu     = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2readget4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str(message2readget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4kpu</default4xi>
        message2readput4kpu     = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2readput4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str(message2readput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4kpu</default4xi>
        message2readreceive4kpu = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2readreceive4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str(message2readreceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4kpu</default4xi>
        message2readsend4kpu    = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2readsend4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str(message2readsend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4kpu</default4xi>
        max2readstate4kpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2readstate4kpu-initialized-by-manoeuvre2readimage4kpu to: " + str( max2readstate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2read4kpu = state2read4kpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2read4kpu-added-by-manoeuvre2readimage4kpu: " + str(state2read4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2read4kpu > max2readstate4kpu:
        state2read4kpu = 0
        message2print = "{(Step 0I/33) set" + "; state2read4kpu-set-by-manoeuvre2readimage4kpu to: " + str(state2read4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2readinfo4kpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2readinfo4kpu-set-by-manoeuvre2readimage4kpu to: " + str(ack2readinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2readget4kpu     = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2readget4kpu-set-by-manoeuvre2readimage4kpu to: " + str(message2readget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2read4kpu</default4xi>
        message2readput4kpu     = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2readput4kpu-set-by-manoeuvre2readimage4kpu to: " + str(message2readput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4kpu</default4xi>
        message2readreceive4kpu = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2readreceive4kpu-set-by-manoeuvre2readimage4kpu to: " + str(message2readreceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4kpu</default4xi>
        message2readsend4kpu    = '<default4xi>default2read4kpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2readsend4kpu-set-by-manoeuvre2readimage4kpu to: " + str(message2readsend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2read4kpu</default4xi>
        max2readstate4kpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2readstate4kpu-set-by-manoeuvre2readimage4kpu to: " + str( max2readstate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4kpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2readimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4kpu = consumerchannel2readmanoeuvre4kpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2readreceive4kpu-received-by-manoeuvre2readimage4kpu: " + str(message2readreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2read4kpu</copy4xi>
                         
            if message2readreceive4kpu == '<copy4xi>copy2read4kpu</copy4xi>':
                message2readsend4kpu = '<roger4xi>roger2read4kpu</roger4xi>'
            else:
                message2readsend4kpu = '<negative4xi><negative2read4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4kpu.send(message2readsend4kpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2readsend4kpu-sent-by-manoeuvre2readimage4kpu: " + str(message2readsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2read4kpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2readreceive4kpu-failed-by-manoeuvre2readimage4kpu: " + str(message2readreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4kpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2readimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2readreceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2readreceive4kpu = consumerchannel2readmanoeuvre4kpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2readreceive4kpu-received-by-manoeuvre2readimage4kpu: " + str(message2readreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2read4kpu</ready4xi> 
                         
            if message2readreceive4kpu == '<ready4xi>ready2read4kpu</ready4xi>':
                message2readsend4kpu = '<ready4xi>ready2read4kpu</ready4xi>'
            else:
                message2readsend4kpu = '<negative4xi><negative2ready4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2readmanoeuvre4kpu.send(message2readsend4kpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2readsend4kpu-sent-by-manoeuvre2readimage4kpu: " + str(message2readsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2read4kpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2readreceive4kpu-failed-by-manoeuvre2readimage4kpu: " + str(message2readreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2read4kpu == 3:    
        info2image4kpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4kpu-try-by-manoeuvre2readimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4kpu = queue2readmanoeuvre4kpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4kpu-got-by-manoeuvre2readimage4kpu: " + str(info2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4kpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2readinfo4kpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2readinfo4kpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2readmanoeuvre4kpu-failed-by-manoeuvre2readimage4kpu: " + str(queue2readmanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2readsend4kpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2readinfo4kpu is True:
                message2readsend4kpu = '<ack4xi>ack2imageinfo4kpu</ack4xi>'
            else:
                message2readsend4kpu = '<nack4xi>nack2imageinfo4kpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2readsend4kpu-set-by-manoeuvre2readimage4kpu to: " + str(message2readsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4kpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4kpu-failed-by-manoeuvre2readimage4kpu: " + str(message2readsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2readmanoeuvre4kpu-calling-by-manoeuvre2readimage4kpu: " + str(publisherchannel2readmanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2readsend4kpu-passing-to-send-by-manoeuvre2readimage4kpu: " + str(message2readsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2readsend4kpu-failed-by-manoeuvre2readimage4kpu: " + str(message2readsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2readmanoeuvre4kpu.send(message2readsend4kpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2readsend4kpu-sent-by-manoeuvre2readimage4kpu: " + str(message2readsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4kpu</ack4xi>            
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2readsend4kpu-failed-by-manoeuvre2readimage4kpu: " + str(message2readsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4read['ACK2INFO4KPU']        = ack2readinfo4kpu
    response2manouvre4read['CODE2ACK4KPU']        = code2ack
    response2manouvre4read['CODE2NACK4KPU']       = code2nack
    response2manouvre4read['MESSAGE2GET4KPU']     = message2readget4kpu
    response2manouvre4read['MESSAGE2PUT4KPU']     = message2readput4kpu
    response2manouvre4read['MESSAGE2RECEIVE4KPU'] = message2readreceive4kpu
    response2manouvre4read['MESSAGE2SEND4KPU']    = message2readsend4kpu
    response2manouvre4read['STATE2MANOEUVRE4KPU'] = state2read4kpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4read-returning-by-manoeuvre2readimage4kpu: " + str(response2manouvre4read) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4KPU': '<roger4xi>roger2read4kpu</roger4xi>',
    #  'ACK2INFO4KPU': False,
    #  'MESSAGE2GET4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2read4kpu</copy4xi>',
    #  'CODE2ACK4KPU': ' 0D-1C-',
    #  'MESSAGE2PUT4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'CODE2NACK4KPU': ' ', 'STATE2MANOEUVRE4KPU': 1}; time: 01:44:40 09/29/11 Pacific Daylight Time}
 
    #2 {'MESSAGE2SEND4KPU': '<ready4xi>ready2read4kpu</ready4xi>',
    #  'ACK2INFO4KPU': False,
    #  'MESSAGE2GET4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2read4kpu</ready4xi>',
    #  'CODE2ACK4KPU': ' 0D-2C-',
    #  'MESSAGE2PUT4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'CODE2NACK4KPU': ' ', 'STATE2MANOEUVRE4KPU': 2}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #3 {'MESSAGE2SEND4KPU': '<ack4xi>ack2imageinfo4kpu</ack4xi>',
    #  'ACK2INFO4KPU': True,
    #  'MESSAGE2GET4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2read4kpu</ready4xi>',
    #  'CODE2ACK4KPU': ' 0D-3C-4E-',
    #  'MESSAGE2PUT4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'CODE2NACK4KPU': ' ',
    #  'STATE2MANOEUVRE4KPU': 3}; time: 01:44:40 09/29/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'ACK2INFO4KPU': False,
    #  'MESSAGE2GET4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'MESSAGE2RECEIVE4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'CODE2ACK4KPU': ' 0D-',
    #  'MESSAGE2PUT4KPU': '<default4xi>default2read4kpu</default4xi>',
    #  'CODE2NACK4KPU': ' ',
    #  'STATE2MANOEUVRE4KPU': 0}; time: 02:11:49 09/29/11 Pacific Daylight Time}


 
    
    
    return response2manouvre4read           
          
#</function4xi>


# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2writeimage4kpu" usage = "control function for write2image4kpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2writeimage4kpu(queue2writemanoeuvre4kpu, consumerchannel2writemanoeuvre4kpu, publisherchannel2writemanoeuvre4kpu):
    '''
    manoeuvres control and command
    '''
    global state2write4kpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2writestate4kpu                                                    # MAX FOR STATE
    global ack2writeinfo4kpu                                                     # ACK FOR IMAGE INFO
    global message2writeget4kpu                                 
    global message2writeput4kpu
    global message2writereceive4kpu
    global message2writesend4kpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2writeimage4kpu" 
    method2print = "manoeuvre2writeimage4kpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2writeimage4kpu-started: " + str(manoeuvre2writeimage4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    # <function manoeuvre2writeimage4kpu at 0x0000000003DB8C48>  
    
    message2print = "{(Step 0B/33) passed" + "; queue2writemanoeuvre4kpu-to-manoeuvre2writeimage4kpu: " + str(queue2writemanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <queue4xi.Queue4Xi object at 0x000000000422D9E8>
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2writemanoeuvre4kpu-to-manoeuvre2writeimage4kpu: " + str(consumerchannel2writemanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # read-write PipeConnection, handle 568>
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2writemanoeuvre4kpu-to-manoeuvre2writeimage4kpu: " + str(publisherchannel2writemanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # <read-write PipeConnection, handle 572> 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4write = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2write4kpu-try-by-manoeuvre2writeimage4kpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2write4kpu is None:
            state2write4kpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2write4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str(state2write4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2write4kpu-used-by-manoeuvre2writeimage4kpu: " + str(state2write4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2write4kpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2write4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str(state2write4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4kpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2writeinfo4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str(ack2writeinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4kpu     = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2writeget4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str(message2writeget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4kpu</default4xi>
        message2writeput4kpu     = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2writeput4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str(message2writeput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4kpu</default4xi>
        message2writereceive4kpu = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2writereceive4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str(message2writereceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4kpu</default4xi>
        message2writesend4kpu    = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2writesend4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str(message2writesend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4kpu</default4xi>
        max2writestate4kpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2writestate4kpu-initialized-by-manoeuvre2writeimage4kpu to: " + str( max2writestate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2write4kpu = state2write4kpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2write4kpu-added-by-manoeuvre2writeimage4kpu: " + str(state2write4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2write4kpu > max2writestate4kpu:
        state2write4kpu = 0
        message2print = "{(Step 0I/33) set" + "; state2write4kpu-set-by-manoeuvre2writeimage4kpu to: " + str(state2write4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2writeinfo4kpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2writeinfo4kpu-set-by-manoeuvre2writeimage4kpu to: " + str(ack2writeinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2writeget4kpu     = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2writeget4kpu-set-by-manoeuvre2writeimage4kpu to: " + str(message2writeget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2write4kpu</default4xi>
        message2writeput4kpu     = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2writeput4kpu-set-by-manoeuvre2writeimage4kpu to: " + str(message2writeput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4kpu</default4xi>
        message2writereceive4kpu = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2writereceive4kpu-set-by-manoeuvre2writeimage4kpu to: " + str(message2writereceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4kpu</default4xi>
        message2writesend4kpu    = '<default4xi>default2write4kpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2writesend4kpu-set-by-manoeuvre2writeimage4kpu to: " + str(message2writesend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2write4kpu</default4xi>
        max2writestate4kpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2writestate4kpu-set-by-manoeuvre2writeimage4kpu to: " + str( max2writestate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4kpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2writeimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4kpu = consumerchannel2writemanoeuvre4kpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2writereceive4kpu-received-by-manoeuvre2writeimage4kpu: " + str(message2writereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <copy4xi>copy2write4kpu</copy4xi>
                         
            if message2writereceive4kpu == '<copy4xi>copy2write4kpu</copy4xi>':
                message2writesend4kpu = '<roger4xi>roger2write4kpu</roger4xi>'
            else:
                message2writesend4kpu = '<negative4xi><negative2write4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4kpu.send(message2writesend4kpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2writesend4kpu-sent-by-manoeuvre2writeimage4kpu: " + str(message2writesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <roger4xi>roger2write4kpu</roger4xi>
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2writereceive4kpu-failed-by-manoeuvre2writeimage4kpu: " + str(message2writereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4kpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2writeimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2writereceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2writereceive4kpu = consumerchannel2writemanoeuvre4kpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2writereceive4kpu-received-by-manoeuvre2writeimage4kpu: " + str(message2writereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # <ready4xi>ready2write4kpu</ready4xi>
                         
            if message2writereceive4kpu == '<ready4xi>ready2write4kpu</ready4xi>':
                message2writesend4kpu = '<ready4xi>ready2write4kpu</ready4xi>'
            else:
                message2writesend4kpu = '<negative4xi><negative2ready4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2writemanoeuvre4kpu.send(message2writesend4kpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2writesend4kpu-sent-by-manoeuvre2writeimage4kpu: " + str(message2writesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ready4xi>ready2write4kpu</ready4xi>
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2writereceive4kpu-failed-by-manoeuvre2writeimage4kpu: " + str(message2writereceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2write4kpu == 3:    
        info2image4kpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4kpu-try-by-manoeuvre2writeimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4kpu = queue2writemanoeuvre4kpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4kpu-got-by-manoeuvre2writeimage4kpu: " + str(info2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # ..//..//..//..//..//web//images//png//dragonxi.png
            if info2image4kpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2writeinfo4kpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2writeinfo4kpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2writemanoeuvre4kpu-failed-by-manoeuvre2writeimage4kpu: " + str(queue2writemanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2writesend4kpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2writeinfo4kpu is True:
                message2writesend4kpu = '<ack4xi>ack2imageinfo4kpu</ack4xi>'
            else:
                message2writesend4kpu = '<nack4xi>nack2imageinfo4kpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2writesend4kpu-set-by-manoeuvre2writeimage4kpu to: " + str(message2writesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # <ack4xi>ack2imageinfo4kpu</ack4xi>       
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4kpu-failed-by-manoeuvre2writeimage4kpu: " + str(message2writesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2writemanoeuvre4kpu-calling-by-manoeuvre2writeimage4kpu: " + str(publisherchannel2writemanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # <read-write PipeConnection, handle 412>
            message2print = "{{(Step 4D/70) passing" + "; message2writesend4kpu-passing-to-send-by-manoeuvre2writeimage4kpu: " + str(message2writesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2writesend4kpu-failed-by-manoeuvre2writeimage4kpu: " + str(message2writesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2writemanoeuvre4kpu.send(message2writesend4kpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2writesend4kpu-sent-by-manoeuvre2writeimage4kpu: " + str(message2writesend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # <ack4xi>ack2imageinfo4kpu</ack4xi>          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2writesend4kpu-failed-by-manoeuvre2writeimage4kpu: " + str(message2writesend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4write['ACK2INFO4KPU']        = ack2writeinfo4kpu
    response2manouvre4write['CODE2ACK4KPU']        = code2ack
    response2manouvre4write['CODE2NACK4KPU']       = code2nack
    response2manouvre4write['MESSAGE2GET4KPU']     = message2writeget4kpu
    response2manouvre4write['MESSAGE2PUT4KPU']     = message2writeput4kpu
    response2manouvre4write['MESSAGE2RECEIVE4KPU'] = message2writereceive4kpu
    response2manouvre4write['MESSAGE2SEND4KPU']    = message2writesend4kpu
    response2manouvre4write['STATE2MANOEUVRE4KPU'] = state2write4kpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4write-returning-by-manoeuvre2writeimage4kpu: " + str(response2manouvre4write) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1
    # {'MESSAGE2SEND4KPU': '<roger4xi>roger2write4kpu</roger4xi>', 
    #  'ACK2INFO4KPU': False, 'MESSAGE2GET4KPU': '<default4xi>default2write4kpu</default4xi>', 
    #  'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2write4kpu</copy4xi>', 
    #  'CODE2ACK4KPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4KPU': '<default4xi>default2write4kpu</default4xi>', 
    #  'CODE2NACK4KPU': ' ', 
    #  'STATE2MANOEUVRE4KPU': 1}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #2
    # {'MESSAGE2SEND4KPU': '<ready4xi>ready2write4kpu</ready4xi>', 
    #  'ACK2INFO4KPU': False, 'MESSAGE2GET4KPU': '<default4xi>default2write4kpu</default4xi>', 
    #  'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2write4kpu</ready4xi>', 
    #  'CODE2ACK4KPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4KPU': '<default4xi>default2write4kpu</default4xi>', 
    #  'CODE2NACK4KPU': ' ', 
    #  'STATE2MANOEUVRE4KPU': 2}; time: 00:11:53 10/03/11 Pacific Daylight Time}
    #3
    # {'MESSAGE2SEND4KPU': '<ack4xi>ack2imageinfo4kpu</ack4xi>', 
    #  'ACK2INFO4KPU': True, 'MESSAGE2GET4KPU': '<default4xi>default2write4kpu</default4xi>', 
    #  'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2write4kpu</ready4xi>', 
    #  'CODE2ACK4KPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4KPU': '<default4xi>default2write4kpu</default4xi>', 
    #  'CODE2NACK4KPU': ' ', 
    #  'STATE2MANOEUVRE4KPU': 3}; time: 00:11:53 10/03/11 Pacific Daylight Time}

    return response2manouvre4write           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------------------------------
#<function4xi function = "manoeuvre2mapimage4kpu" usage = "control function for map2image4kpu function in this module">

# ------------------------------------------------------------------------------------------------------------------------- 
def manoeuvre2mapimage4kpu(queue2mapmanoeuvre4kpu, consumerchannel2mapmanoeuvre4kpu, publisherchannel2mapmanoeuvre4kpu):
    '''
    manoeuvres control and command
    '''
    global state2map4kpu                                                       # STATE FOR COMMAND AND CONTROL
    global max2mapstate4kpu                                                    # MAX FOR STATE
    global ack2mapinfo4kpu                                                     # ACK FOR IMAGE INFO
    global message2mapget4kpu                                 
    global message2mapput4kpu
    global message2mapreceive4kpu
    global message2mapsend4kpu
    
    # -----------------------------------------------------------------------------------------
    # (Step 0/33) PARAMETERS            
    # -----------------------------------------------------------------------------------------          
    class2print  = "manoeuvre2mapimage4kpu" 
    method2print = "manoeuvre2mapimage4kpu" 

    code2ack  = ' '
    code2nack = ' '
    
    message2print = "{(Step 0A/33) started" + "; manoeuvre2mapimage4kpu-started: " + str(manoeuvre2mapimage4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                      
    #  
    
    message2print = "{(Step 0B/33) passed" + "; queue2mapmanoeuvre4kpu-to-manoeuvre2mapimage4kpu: " + str(queue2mapmanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0C/33) passed" + "; consumerchannel2mapmanoeuvre4kpu-to-manoeuvre2mapimage4kpu: " + str(consumerchannel2mapmanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    message2print = "{(Step 0D/33) passed" + "; publisherchannel2mapmanoeuvre4kpu-to-manoeuvre2mapimage4kpu: " + str(publisherchannel2mapmanoeuvre4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # 
    
    code2ack  = code2ack + '0D-' 
    response2manouvre4map = {}
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # SETTINGS
    # ------------------------------------------------------------------------------------------------------------------------    
    message2print = "{(Step 0E/70) try" + "; state2map4kpu-try-by-manoeuvre2mapimage4kpu"  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        if state2map4kpu is None:
            state2map4kpu = 0
            message2print = "{(Step 0F/33) initialized" + "; state2map4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str(state2map4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
        else:
            message2print = "{(Step 0G/33) used" + "; state2map4kpu-used-by-manoeuvre2mapimage4kpu: " + str(state2map4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # 1; 2; 3;  
    except:
        state2map4kpu = 0
        message2print = "{(Step 0Z1/33) initialized" + "; state2map4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str(state2map4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4kpu        = False
        message2print = "{(Step 0Z2/33) initialized" + "; ack2mapinfo4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str(ack2mapinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4kpu     = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0Z3/33) initialized" + "; message2mapget4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str(message2mapget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4kpu</default4xi>
        message2mapput4kpu     = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0Z4/33) initialized" + "; message2mapput4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str(message2mapput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4kpu</default4xi>
        message2mapreceive4kpu = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0Z5/33) initialized" + "; message2mapreceive4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str(message2mapreceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4kpu</default4xi>
        message2mapsend4kpu    = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0Z6/33) initialized" + "; message2mapsend4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str(message2mapsend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4kpu</default4xi>
        max2mapstate4kpu = 3
        message2print = "{(Step 0Z7/33) initialized" + ";  max2mapstate4kpu-initialized-by-manoeuvre2mapimage4kpu to: " + str( max2mapstate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE UPDATE
    # ------------------------------------------------------------------------------------------------------------------------                      
    state2map4kpu = state2map4kpu + 1                                                                   
    message2print = "{(Step 0H/33) added" + "; state2map4kpu-added-by-manoeuvre2mapimage4kpu: " + str(state2map4kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 1; 2; 3; 4  
    if state2map4kpu > max2mapstate4kpu:
        state2map4kpu = 0
        message2print = "{(Step 0I/33) set" + "; state2map4kpu-set-by-manoeuvre2mapimage4kpu to: " + str(state2map4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 0
        ack2mapinfo4kpu        = False
        message2print = "{(Step 0J/33) set" + "; ack2mapinfo4kpu-set-by-manoeuvre2mapimage4kpu to: " + str(ack2mapinfo4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                     
        # False
        message2mapget4kpu     = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0K/33) set" + "; message2mapget4kpu-set-by-manoeuvre2mapimage4kpu to: " + str(message2mapget4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                               
        # <default4xi>default2map4kpu</default4xi>
        message2mapput4kpu     = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0L/33) set" + "; message2mapput4kpu-set-by-manoeuvre2mapimage4kpu to: " + str(message2mapput4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4kpu</default4xi>
        message2mapreceive4kpu = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0M/33) set" + "; message2mapreceive4kpu-set-by-manoeuvre2mapimage4kpu to: " + str(message2mapreceive4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4kpu</default4xi>
        message2mapsend4kpu    = '<default4xi>default2map4kpu</default4xi>'
        message2print = "{(Step 0N/33) set" + "; message2mapsend4kpu-set-by-manoeuvre2mapimage4kpu to: " + str(message2mapsend4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)              
        # <default4xi>default2map4kpu</default4xi>
        max2mapstate4kpu = 3
        message2print = "{(Step 0O/33) initialized" + ";  max2mapstate4kpu-set-by-manoeuvre2mapimage4kpu to: " + str( max2mapstate4kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        #     
                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE I - RECEIVE CONTROL AND COMMAND COPY MESSAGE VIA PIPE AND SEND ROGER OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4kpu == 1:
        message2print = "{(Step 1A/70) try" + "; receive-try-by-manoeuvre2mapimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4kpu = consumerchannel2mapmanoeuvre4kpu.recv()
            message2print = "{(Step 1B/33) received" + "; message2mapreceive4kpu-received-by-manoeuvre2mapimage4kpu: " + str(message2mapreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4kpu == '<copy4xi>copy2mapimage4kpu</copy4xi>':
                message2mapsend4kpu = '<roger4xi>roger2mapimage4kpu</roger4xi>'
            else:
                message2mapsend4kpu = '<negative4xi><negative2mapimage4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4kpu.send(message2mapsend4kpu)
            
            message2print = "{(Step 1C/33) sent" + "; message2mapsend4kpu-sent-by-manoeuvre2mapimage4kpu: " + str(message2mapsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '1C-'
        except:
            message2print = "{(Step 1Z/33) failed" + "; message2mapreceive4kpu-failed-by-manoeuvre2mapimage4kpu: " + str(message2mapreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '1Z-' 
    else:
        pass

                    
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE II - RECEIVE CONTROL AND COMMAND READY MESSAGE VIA PIPE AND SEND READY OR NEGATIVE BACK 
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4kpu == 2:
        message2print = "{(Step 2A/70) try" + "; receive-try-by-manoeuvre2mapimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        message2mapreceive4kpu = ''
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # RECEIVE CONTROL AND COMMAND MESSAGE VIA PIPE
            # ----------------------------------------------------------------------------------------------------------------         
            message2mapreceive4kpu = consumerchannel2mapmanoeuvre4kpu.recv()
            message2print = "{(Step 2B/33) received" + "; message2mapreceive4kpu-received-by-manoeuvre2mapimage4kpu: " + str(message2mapreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
                         
            if message2mapreceive4kpu == '<ready4xi>ready2mapimage4kpu</ready4xi>':
                message2mapsend4kpu = '<ready4xi>ready2mapimage4kpu</ready4xi>'
            else:
                message2mapsend4kpu = '<negative4xi><negative2ready4kpu</<negative4xi>'
            # ----------------------------------------------------------------------------------------------------------------
            # SEND ROGER OR NEGATIVE BACK 
            # ----------------------------------------------------------------------------------------------------------------                                
            publisherchannel2mapmanoeuvre4kpu.send(message2mapsend4kpu)
            
            message2print = "{(Step 2C/33) sent" + "; message2mapsend4kpu-sent-by-manoeuvre2mapimage4kpu: " + str(message2mapsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            # 
            
            code2ack  = code2ack + '2C-'
        except:
            message2print = "{(Step 2Z/33) failed" + "; message2mapreceive4kpu-failed-by-manoeuvre2mapimage4kpu: " + str(message2mapreceive4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '2Z-' 
    else:
        pass        
                
    # ------------------------------------------------------------------------------------------------------------------------
    # STATE III - GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE AND SEND ACK OR NACK VIA PIPE
    # ------------------------------------------------------------------------------------------------------------------------
    if state2map4kpu == 3:    
        info2image4kpu = None
        message2print = "{(Step 3A/70) try" + "; info2image4kpu-try-by-manoeuvre2mapimage4kpu"  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        try: 
            # ----------------------------------------------------------------------------------------------------------------
            # GET IMAGE INFORMATION SUCH AS IMAGE PATH VIA QUEUE
            # ----------------------------------------------------------------------------------------------------------------        
            info2image4kpu = queue2mapmanoeuvre4kpu.get()
            
            message2print = "{(Step 3B/33) got" + "; info2image4kpu-got-by-manoeuvre2mapimage4kpu: " + str(info2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # 
            if info2image4kpu is None:
                code2nack  = code2nack + '3Z1-'
                ack2mapinfo4kpu = False 
            else:    
                code2ack  = code2ack + '3C-'
                ack2mapinfo4kpu = True
        except:
            message2print = "{(Step 3Z1/33) failed" + "; queue2mapmanoeuvre4kpu-failed-by-manoeuvre2mapimage4kpu: " + str(queue2mapmanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '3Z1-'
            
        message2mapsend4kpu = ' '
        try:
            # ----------------------------------------------------------------------------------------------------------------
            # SEND CONTROL INFO ABOUT IMAGE OPERATION - EITHER ACK OR NACK  
            # ----------------------------------------------------------------------------------------------------------------         
            if ack2mapinfo4kpu is True:
                message2mapsend4kpu = '<ack4xi>ack2imageinfo4kpu</ack4xi>'
            else:
                message2mapsend4kpu = '<nack4xi>nack2imageinfo4kpu</nack4xi>'
            message2print = "{(Step 4B/33) set" + "; message2mapsend4kpu-set-by-manoeuvre2mapimage4kpu to: " + str(message2mapsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            #      
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4kpu-failed-by-manoeuvre2mapimage4kpu: " + str(message2mapsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            
            code2nack  = code2nack + '4Z1-'    
        try:                
            message2print = "{(Step 4C/70) using" + "; publisherchannel2mapmanoeuvre4kpu-calling-by-manoeuvre2mapimage4kpu: " + str(publisherchannel2mapmanoeuvre4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)    
            # 
            message2print = "{{(Step 4D/70) passing" + "; message2mapsend4kpu-passing-to-send-by-manoeuvre2mapimage4kpu: " + str(message2mapsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #              
        except:
            message2print = "{(Step 4Z1/33) failed" + "; message2mapsend4kpu-failed-by-manoeuvre2mapimage4kpu: " + str(message2mapsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z2-'                
        try:    
            publisherchannel2mapmanoeuvre4kpu.send(message2mapsend4kpu)
            
            message2print = "{(Step 4E/33) sent" + "; message2mapsend4kpu-sent-by-manoeuvre2mapimage4kpu: " + str(message2mapsend4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)          
            #          
            code2ack  = code2ack + '4E-'
        except:
            message2print = "{(Step 4Z3/33) failed" + "; message2mapsend4kpu-failed-by-manoeuvre2mapimage4kpu: " + str(message2mapsend4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            code2nack  = code2nack + '4Z3-'                
    else:
        pass
        

            
    # ------------------------------------------------------------------------------------------------------------------------
    # RETURN 
    # ------------------------------------------------------------------------------------------------------------------------
    response2manouvre4map['ACK2INFO4KPU']        = ack2mapinfo4kpu
    response2manouvre4map['CODE2ACK4KPU']        = code2ack
    response2manouvre4map['CODE2NACK4KPU']       = code2nack
    response2manouvre4map['MESSAGE2GET4KPU']     = message2mapget4kpu
    response2manouvre4map['MESSAGE2PUT4KPU']     = message2mapput4kpu
    response2manouvre4map['MESSAGE2RECEIVE4KPU'] = message2mapreceive4kpu
    response2manouvre4map['MESSAGE2SEND4KPU']    = message2mapsend4kpu
    response2manouvre4map['STATE2MANOEUVRE4KPU'] = state2map4kpu
                          
    message2print = "{(Step 33/33) returning" + "; response2manouvre4map-returning-by-manoeuvre2mapimage4kpu: " + str(response2manouvre4map) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #1 {'MESSAGE2SEND4KPU': '<roger4xi>roger2mapimage4kpu</roger4xi>', 
    #  'ACK2INFO4KPU': False, 
    #  'MESSAGE2GET4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'MESSAGE2RECEIVE4KPU': '<copy4xi>copy2mapimage4kpu</copy4xi>', 
    #  'CODE2ACK4KPU': ' 0D-1C-', 
    #  'MESSAGE2PUT4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'CODE2NACK4KPU': ' ', 
    #  'STATE2MANOEUVRE4KPU': 1}
     
    #2 {'MESSAGE2SEND4KPU': '<ready4xi>ready2mapimage4kpu</ready4xi>', 
    #  'ACK2INFO4KPU': False, 
    #  'MESSAGE2GET4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2mapimage4kpu</ready4xi>', 
    #  'CODE2ACK4KPU': ' 0D-2C-', 
    #  'MESSAGE2PUT4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'CODE2NACK4KPU': ' ', 
    #  'STATE2MANOEUVRE4KPU': 2}; time: 06:49:48 10/04/11 Pacific Daylight Time}
     
    #3 {'MESSAGE2SEND4KPU': '<ack4xi>ack2imageinfo4kpu</ack4xi>', 
    #  'ACK2INFO4KPU': True, 
    #  'MESSAGE2GET4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'MESSAGE2RECEIVE4KPU': '<ready4xi>ready2mapimage4kpu</ready4xi>', 
    #  'CODE2ACK4KPU': ' 0D-3C-4E-', 
    #  'MESSAGE2PUT4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'CODE2NACK4KPU': ' ', 
    #  'STATE2MANOEUVRE4KPU': 3}; time: 06:49:48 10/04/11 Pacific Daylight Time}
    
    #4 {'MESSAGE2SEND4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'ACK2INFO4KPU': False, 
    #  'MESSAGE2GET4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'MESSAGE2RECEIVE4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'CODE2ACK4KPU': ' 0D-', 
    #  'MESSAGE2PUT4KPU': '<default4xi>default2map4kpu</default4xi>', 
    #  'CODE2NACK4KPU': ' ', 'STATE2MANOEUVRE4KPU': 0}; time: 06:49:48 10/04/11 Pacific Daylight Time}

    return response2manouvre4map           
          
#</function4xi>

# -------------------------------------------------------------------------------------------------
#<function4xi function = "write2image4kpu" usage = "
#                                                                                                              
#   # ---------------------------------------------------------------------------------------------
#   # (16) ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4KPU                      
#   # ---------------------------------------------------------------------------------------------    
#   print('(Step 16A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4KPU ', time4xi())    
#   respon4write2image = {} 
#   task4write2image = None
#   try:                               
#       response4write2image = write2image4kpu(task4write2image)                                                                                                          
#       print('(Step 16B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL & COMMAND WITH READ2IMAGE4KPU : ', response4write2image)
#       queue2write4kpu      = response4write2image['QUEUE']
#       channel2publish4kpu = response4write2image['PUBLISHERCHANNEL']
#       channel2consume4kpu = response4write2image['CONSUMERCHANNEL']
#   except:
#       print('(Step 16Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL & COMMAND WITH READ2IMAGE4KPU ', time4xi()) 
#   # --------------------------------------------------------------------------------------------------
#   # (17) SEND CONTROL & COMMAND COPY VIA PIPE FOR MANOEUVRE2IMAGE4KPU TO RECEIVE                 
#   # --------------------------------------------------------------------------------------------------      
#   print('(Step 17A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:                               
#       message2writesend4kpu = "<copy4xi>copy2write4kpu</copy4xi>"
#       channel2publish4kpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4kpu.send(message2writesend4kpu)
#       print("(Step 17B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2publish4kpu, " MESSAGE2SEND4KPU: " , message2writesend4kpu)                                                                                                
#   except:
#       print('(Step 17Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())                                                
#   # ------------------------------------------------------------------------------------------
#   # (18) LET MANOEUVRE2IMAGE4KPU TO RECEIVE COPY AND AND SEND ROGER VIA PIPE                
#   # ------------------------------------------------------------------------------------------       
#   print("(Step 18C/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2write4kpu, channel2consume4kpu, channel2publish4kpu)
#       print("(Step 18D/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2readimage4kpu)
#   except:
#       print("(Step 18Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   else:
#       pass # trust activated thread    
#   # ---------------------------------------------------------------------------------------------
#   # (19) RECEIVE CONTROL & COMMAND ROGER VIA PIPE                   
#   # ---------------------------------------------------------------------------------------------          
#   print('(Step 19A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:
#       channel2consume4kpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4kpu = channel2consume4kpu.recv()
#       print("(Step 19B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2consume4kpu, " CONTROL & COMMAND: ", message2writereceive4kpu)                                                                                                  
#       # <roger4xi>roger2write4kpu</roger4xi>
#   except:
#       print('(Step 19Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4KPU ', time4xi())         
#   # ---------------------------------------------------------------------------------------------------
#   # (20) SEND CONTROL & COMMAND READY VIA PIPE FOR MANOEUVRE2IMAGE4KPU TO RECEIVE                
#   # ---------------------------------------------------------------------------------------------------      
#   print('(Step 20A/700) OF KPU2IMAGE4XI5TEST - TRYING TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:                               
#       message2writesend4kpu = "<write4xi>write2write4kpu</write4xi>"
#       channel2publish4kpu = response4write2image['PUBLISHERCHANNEL']
#       channel2publish4kpu.send(message2writesend4kpu)
#       print("(Step 20B/700) OF KPU2IMAGE4XI5TEST - SENT TO CHANNEL2PUBLISH4KPU: ", channel2publish4kpu, " MESSAGE2SEND4KPU: " , message2writesend4kpu)                                                                                                
#       # <write4xi>write2write4kpu</write4xi> 
#   except:
#        print('(Step 20Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO SEND CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())                                                
#   # -----------------------------------------------------------------------------------
#   # (21) LET MANOEUVRE2IMAGE4KPU TO RECEIVE READY AND TO SEND READY                
#   # -----------------------------------------------------------------------------------       
#   print("(Step 21A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2write4kpu, channel2consume4kpu, channel2publish4kpu)
#       print("(Step 21B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2readimage4kpu)
#   except:
#       print("(Step 21Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   # ---------------------------------------------------------------------------------------
#   # (22) RECEIVE CONTROL & COMMAND READY VIA PIPE                      
#   # ---------------------------------------------------------------------------------------          
#   print('(Step 22A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND VIA PIPE WITH READ2IMAGE4KPU ', time4xi())     
#   try:
#       channel2consume4kpu = response4write2image['CONSUMERCHANNEL']                              
#       message2writereceive4kpu = channel2consume4kpu.recv()
#       print("(Step 22B/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2consume4kpu, " CONTROL & COMMAND: ", message2writereceive4kpu)                                                                                                  
#       # <write4xi>write2write4kpu</write4xi> 
#   except:
#       print('(Step 22Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND VIA PIPE  WITH READ2IMAGE4KPU ', time4xi())          
#   # -----------------------------------------------------------------------------------------
#   # (24) PUT IMAGE PATH TO QUEUE
#   # -----------------------------------------------------------------------------------------                          
#   print("(Step 24A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PUT IMAGE PATH TO QUEUE ", time4xi())      
#   try:    
#       queue2write4dragon.put(path2dragon)             
#       print("(Step 24B/700) OF DRAGON2IMAGE4XI5TEST - PUT TO QUEUE IMAGE PATH: ", path2dragon) 
#       # ..//..//..//..//..//web//images//png//dragonxi.png  
#   except:
#       print("(Step 24Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PUT IMAGE PATH TO QUEUE ", time4xi()) 
#   # ------------------------------------------------------------------------------------------------------
#   # (25) LET MANOEUVRE2IMAGE4KPU TO GET FROM QUEUE AND TO SEND EITHER ACK OR NACK VIA PIPE               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 25A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2write4kpu, channel2consume4kpu, channel2publish4kpu)
#       print("(Step 25B/700) OF KPU2IMAGE4XI5TEST - PERFORMED TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", manoeuvre2readimage4kpu)
#   except:
#       print("(Step 25Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())     
#   # -----------------------------------------------------------------------------------------------
#   # (26) RECEIVE ACK OR NACK ABOUT IMAGE OPERATION VIA PIPE                
#   # -----------------------------------------------------------------------------------------------          
#   print('(Step 26A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())      
#   try: 
#       message2writereceive4kpu = channel2consume4kpu.recv()
#       print("(Step 26D/700) OF KPU2IMAGE4XI5TEST - RECEIVED FROM CHANNEL2CONSUME4KPU: ", channel2consume4kpu, " CONTROL & COMMAND: ", message2writereceive4kpu)                                                                                                   
#       # <ack4xi>ack2imageinfo4kpu</ack4xi>
#   except:
#       print('(Step 26Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RECEIVE CONTROL & COMMAND ABOUT IMAGE OPERTION VIA PIPE ', time4xi())               
#   # -------------------------------------------------------------------------------------------------------
#   # (27) LET MANOEUVRE2IMAGE4KPU GO TO STATE 0               
#   # -------------------------------------------------------------------------------------------------------       
#   print("(Step 27A/700) OF KPU2IMAGE4XI5TEST - TRYING TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi()) 
#   try:        
#       response2manoeuvre4kpu = manoeuvre2readimage4kpu(queue2write4kpu, channel2consume4kpu, channel2publish4kpu)
#       print("(Step 27B/700) OF KPU2IMAGE4XI5TEST - GOT RESPONSE2MANOEUVRE4KPU ", response2manoeuvre4kpu)
#   except:
#       print("(Step 27Z/700) OF KPU2IMAGE4XI5TEST *** FAILED TO PERFORM TARGET FUNCTION MANOEUVRE2IMAGE4KPU ", time4xi())   

#">

# -------------------------------------------------------------------------------------------------   
def write2image4kpu(task4imagewrite2kpu=None):

    '''
    writes 2D images for KPU
    '''
    global kpu2image4xi                                                     # 2D-IMAGES FOR KPU  
    global control2writeimage4kpu                                              # COMMAND AND CONTROL FOR KPU IMAGES 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS        
    # -----------------------------------------------------------------------------------------          
    class2print = "write2image4kpu"
    method2print = "write2image4kpu"

    message2print = "{(Step 1A/33) started" + "; write2image4kpu-started: " + str(write2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function write2image4kpu at 0x00000000038E8EC8>
    
    message2print = "{(Step 1B/33) passed" + "; task4imagewrite2kpu-to-write2image4kpu: " + str(task4imagewrite2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # None
            
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # CREATE KPU2IMAGE4XI IF NECESSARY       
    # ----------------------------------------------------------------------------------------- 
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                              # INITIALIZE IMAGE FOR KPU 
            message2print = "{(Step 1C/70) created" + "; kpu2image4xi-created-by-write2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        else:
            message2print = "{(Step 1D/70) skipped" + "; create2image4kpu-skipped-by-write2image4kpu: " + str(write2image4kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # create2image4kpu-skipped-by-write2image4kpu: <function write2image4kpu at 0x00000000038E8EC8>                                 
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 1E/70) set" + "; kpu2image4xi-set-by-write2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #             
        except:
            kpu2image4xi = {}
            message2print = "{(Step 1Z/70) yielded" + "; kpu2image4xi-yielded-by-write2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '1Z-' 
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - CHECK PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    try:             
        if task4imagewrite2kpu is None:
            # -----------------------------------------------------------------------------------------
            # NO PARAMETERS GIVEN BY CALLER FOR READ - BUILD TASK! 
            # -----------------------------------------------------------------------------------------    
            
            task4imagewrite2kpu = {}                                                                                                                                                                                       
            
            message2print = "{(Step 1A/70) initialized" + "; task4imagewrite2kpu-created-by-write2image4kpu to: " + str(task4imagewrite2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
            
            # ---------------------------------------------------------------------------------------------
            # (1B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                                 # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 1B1/70) initialized" + "; function2callback4kpu-created-by-write2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # <function trace2image4kpu at 0x00000000038E2A48>
            
            code2ack  = code2ack + '1B1-'
           
            args2callback = {}                                                                                      # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4kpu"                                                            # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4kpu"                                                            # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    

            message2print = "{(Step 1B2/70) initialized" + "; args2callback-created-by-write2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # {'MESSAGE': '{(Step 1Z1/33) failed; write2image4kpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'write2image4kpu', 
            #  'METHOD': 'write2image4kpu'}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            code2ack  = code2ack + '1B2-'
            
            # ---------------------------------------------------------------------------------------------
            # (1C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2kpu['DATA2CALLBACK'] = args2callback                                                    # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                        # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '1C-'
            message2print = "{(Step 1C/70) initialized" + "; task4imagewrite2kpu-created-by-write2image4kpu to: " + str(task4imagewrite2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            # {'FUNCTION2CALLBACK': <function trace2image4kpu at 0x00000000038E2A48>,
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; write2image4kpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}',
            #  'CLASS': 'write2image4kpu',
            #  'METHOD': 'write2image4kpu'}}; time: 02:32:29 10/05/11 Pacific Daylight Time}

            
            # ---------------------------------------------------------------------------------------------
            # (2) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 2A/70) initialized" + "; platform2xi-created-by-write2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # {}
            
            try:
                platform2xi = get2platformids4xi()                                                                  # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 2B/70) got" + "; platform2xi-got-by-write2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}; time: 02:32:29 10/05/11 Pacific Daylight Time}

                code2ack  = code2ack + '2B-'
            except:
                message2print = "{(Step 2Z/70) failed" + "; platform2xi-failed-by-write2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '2Z-'
                 
            task4imagewrite2kpu['PLATFORM'] = platform2xi                                                           # PLATFORM OF SYSTEM 
              
            
            # ---------------------------------------------------------------------------------------------
            # (3) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 3A/70) trying" + "; context2kpu-trying-by-write2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                     # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 3B/70) got" + "; context2kpu-got-by-write2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4kpu', ...
   
                code2ack  = code2ack + '3B-'         
            except:
                message2print = "{(Step 3Z/70) failed" + "; context2kpu-failed-by-write2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '3Z-'
                
            task4imagewrite2kpu ['CONTEXT'] = context2kpu           
            
            # ---------------------------------------------------------------------------------------------
            # (4) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 4A/70) trying" + "; device2kpu-trying-by-write2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                           # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 4B/70) got" + "; device2kpu-got-by-write2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568' ...
               
                code2ack  = code2ack + '4B-'  
            except:
                message2print = "{(Step 4Z/70) failed" + "; device2kpu-failed-by-write2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '4Z-'
            
            task4imagewrite2kpu ['DEVICE'] = device2kpu
            
                   
            # ---------------------------------------------------------------------------------------------
            # (5) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2kpu = {}
             
            message2print = "{(Step 5A/70) trying" + "; imageinfo2kpu-trying-by-write2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 5B/70) got" + "; imageinfo2kpu-got-by-write2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # {'SIZE2QUEUE': 2147483646, 
                #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
                #  'ORDER': 'CL_RGBA'}, 
                #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004494E10>, 
                #  'WIDTH': 800, 
                #  'CONTEXT': {'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2context4kpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2context4kpu', 
                #  'METHOD': 'create2context4kpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2context4kpu at 0x000000000387E348>, 
                #  'ACK2RETURN': ' 2A-2C-2D-2E-2B-3B-4B-5B-6B-7B-7B-', 
                #  'INFO2PRIVATE': None, 
                #  'PLATFORM': {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}, 
                #  'NACK2RETURN': ' ', 
                #  'SIZE2PRIVATE': 0, 
                #  'PROPERTIES': None}, 
                #  'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
                #  'COUNT2REFERENCE': 1, 
                #  'FUNCTION2CALLBACK': <function trace2image4kpu at 0x00000000038E2A48>, 
                #  'COMMANDQUEUE': ...

                code2ack  = code2ack + '5B-'  
            except:
                message2print = "{(Step 5Z/70) failed" + "; imageinfo2kpu-failed-by-write2image4kpu: " + str(imageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '5Z-'          
                      
            task4imagewrite2kpu ['IMAGE'] = imageinfo2kpu         
                        

            # ---------------------------------------------------------------------------------------------
            # (6) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 6A/70) trying" + "; queue2kpu-trying-by-write2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                              # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 6B/70) got" + "; queue2kpu-got-by-write2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # <queue4xi.Queue4Xi object at 0x0000000003D4FA90>
                
                code2ack  = code2ack + '6B-'  
            except:
                message2print = "{(Step 6Z/70) failed" + "; queue2kpu-failed-by-write2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '6Z-'
                
            task4imagewrite2kpu ['QUEUE'] = queue2kpu

            
            # ---------------------------------------------------------------------------------------------
            # (7) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 7A/70) trying" + "; pipe2kpu-trying-by-write2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                                # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 7B/70) got" + "; pipe2kpu-got-by-write2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # (<read-write PipeConnection, handle 568>, <read-write PipeConnection, handle 572>)
                
                code2ack  = code2ack + '7B-'  
            except:
                message2print = "{(Step 7Z/70) failed" + "; pipe2kpu-failed-by-write2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '7Z-'
                
            task4imagewrite2kpu ['PIPE'] = pipe2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (8) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 8A/70) trying" + "; commandqueueinfo2kpu-trying-by-write2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                                   # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 8B/70) got" + "; commandqueueinfo2kpu-got-by-write2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2commandqueue4kpu-failed ; time: 02:32:29 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2commandqueue4kpu', 
                #  'METHOD': 'create2commandqueue4kpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2commandqueue4kpu at 0x000000000388DA48> ...
                
                code2ack  = code2ack + '8B-'  
            except:
                message2print = "{(Step 8Z/70) failed" + "; commandqueueinfo2kpu-failed-by-write2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '8Z-'
        
            task4imagewrite2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
            
            
            
            # ---------------------------------------------------------------------------------------------
            # (9) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 9A/70) trying" + "; bufferinfo2kpu-trying-by-write2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                               # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 9B/70) got" + "; bufferinfo2kpu-got-by-write2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'CODE2ERROR': 0, 
                #  'COUNT2REFERENCE': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2buffer4kpu', 
                #  'METHOD': 'create2buffer4kpu'}, ...
                
                code2ack  = code2ack + '9B-' 
            except:
                message2print = "{(Step 9Z/70) failed" + "; bufferinfo2kpu-failed-by-write2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '9Z-'
        
            task4imagewrite2kpu ['BUFFER'] = bufferinfo2kpu 
            
            # ---------------------------------------------------------------------------------------------
            # (10) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 10A/70) trying" + "; eventinfo2kpu-trying-by-write2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 10B/70) got" + "; eventinfo2kpu-got-by-write2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #  {'CODE2ERROR': 0, 
                #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2event4kpu-failed ; time: 02:32:28 10/05/11 Pacific Daylight Time}', 
                #  'CLASS': 'create2event4kpu', 
                #  'METHOD': 'create2event4kpu'}, 
                #  'FUNCTION2CALLBACK': <function trace2event4kpu at 0x00000000038A3F48>, ...
                
                code2ack  = code2ack + '10B-' 
            except:
                message2print = "{(Step 10Z/70) failed" + "; eventinfo2kpu-failed-by-write2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '10Z-'
        
            task4imagewrite2kpu ['EVENT'] = eventinfo2kpu 
            
            
            # ---------------------------------------------------------------------------------------------
            # (11) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 11A/70) trying" + "; eventwaitlist2kpu-trying-by-write2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 11B/70) set" + "; eventwaitlist2kpu-set-by-write2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {} 
                
                code2ack  = code2ack + '11B-' 
            except:
                message2print = "{(Step 11Z/70) failed" + "; eventwaitlist2kpu-failed-by-write2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '11Z-'
        
            task4imagewrite2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     

            
            # ---------------------------------------------------------------------------------------------
            # (12) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
           
            message2print = "{(Step 12A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-write2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                   
                message2print = "{(Step 12B/70) set" + "; eventwaitlistquantity2kpu-set-by-write2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0 
                
                code2ack  = code2ack + '12B-' 
            except:
                message2print = "{(Step 12Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-write2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '12Z-'
        
            task4imagewrite2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                                 
            
            # ---------------------------------------------------------------------------------------------
            # (13) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2kpu = ' '                                                                   
     
            message2print = "{(Step 13A/70) trying" + "; blocking2kpu-trying-by-write2image4kpu: " + str(blocking2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2kpu = 'CL_FALSE'                                                                           # the write command is non-blocking                                                              
                message2print = "{(Step 13B/70) set" + "; blocking2kpu-set-by-write2image4kpu to: " + str(blocking2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # CL_FALSE 
                code2ack  = code2ack + '13B-' 
            except:
                message2print = "{(Step 13Z/70) failed" + "; blocking2kpu-failed-by-write2image4kpu: " + str(blocking2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '13Z-'
        
            task4imagewrite2kpu ['BLOCKING'] = blocking2kpu             
            
            # ---------------------------------------------------------------------------------------------
            # (14) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2kpu = {}
            
            message2print = "{(Step 14A/70) trying" + "; origin2kpu-trying-by-write2image4kpu: " + str(origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2kpu['X'] = 0 
                origin2kpu['Y'] = 0  
                origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 14B/70) set" + "; origin2kpu-set-by-write2image4kpu to: " + str(origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 0, 'X': 0, 'Z': 0} 
                code2ack  = code2ack + '14B-' 
            except:
                message2print = "{(Step 14Z/70) failed" + "; origin2kpu-failed-by-write2image4kpu: " + str(origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '14Z-'
        
            task4imagewrite2kpu ['ORIGIN'] = origin2kpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (15) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 15A/70) trying" + "; region2kpu-trying-by-write2image4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO READ 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO READ 
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO READ                                                                                                                          
                message2print = "{(Step 15B/70) set" + "; region2kpu-set-by-write2image4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # {'Y': 90, 'X': 160, 'Z': 0}
                code2ack  = code2ack + '15B-' 
            except:
                message2print = "{(Step 15Z/70) failed" + "; region2kpu-failed-by-write2image4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '15Z-'
        
            task4imagewrite2kpu ['REGION'] = region2kpu
            
            
            # ---------------------------------------------------------------------------------------------
            # (16) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.

            height2kpu = 800
            width2kpu = 800
            rowpitch2kpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 16A/70) trying" + "; rowpitch2kpu-trying-by-write2image4kpu: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            #  
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2kpu = int(width2kpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 16B/70) set" + "; rowpitch2kpu-set-by-write2image4kpu to: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 800                                                                                            
                 
                code2ack  = code2ack + '16B-' 
            except:
                message2print = "{(Step 16Z/70) failed" + "; rowpitch2kpu-failed-by-write2image4kpu: " + str(rowpitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '16Z-'
        
            task4imagewrite2kpu ['ROWPITCH'] = rowpitch2kpu     
            
            # ----------------------------------------------------------------------------------------------------
            # (17) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2kpu = None
            size2kpu = int(width2kpu * height2kpu)                                                                  # CALCULATED MEMORY
            if size2kpu == 0:
                size2kpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 17A/70) trying" + "; pointer2kpu-trying-by-write2image4kpu: " + str(pointer2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2kpu = Arena4Xi(size2kpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 17B/70) got" + "; arena2kpu-got-by-write2image4kpu: " + str(arena2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # <arena4xi.Arena4Xi object at 0x00000000041348D0>
                
                size2arena = arena2kpu.size                                                                         # 
                              
                message2print = "{(Step 17C/70) got" + "; size2arena-got-by-write2buffer4kpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 640000 
                
                buffer2arena = arena2kpu.buffer
                
                message2print = "{(Step 17D/70) got" + "; buffer2arena-got-by-write2buffer4kpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # <mmap.mmap object at 0x000000000421A450> 
                
                code2ack  = code2ack + '17D-' 
                
                if size2arena < size2kpu:
                    size2kpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 17E/70) set" + "; size2kpu-set-by-write2buffer4kpu to: " + str(size2kpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '17E-'
                else:
                    pass
                    
                pointer2kpu = buffer2arena
                message2print = "{(Step 17F/70) set" + "; pointer2kpu-set-by-write2buffer4kpu to: " + str(pointer2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # <mmap.mmap object at 0x000000000421A450>
                
                code2ack  = code2ack + '17F-'       
            except:
                message2print = "{(Step 17Z/70) failed" + "; pointer2kpu-failed-by-write2image4kpu: " + str(pointer2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '17Z-'
        
            task4imagewrite2kpu ['POINTER'] = pointer2kpu                            
            task4imagewrite2kpu ['SIZE']    = size2kpu                                                               # AUTOMATIC BY KPUXI
            
            
            # --------------------------------------------------------------------------------------------------------
            # (18) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2kpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 18A/70) trying" + "; slicepitch2kpu-trying-by-write2image4kpu: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2kpu = 0                                                          
                message2print = "{(Step 18B/70) set" + "; slicepitch2kpu-set-by-write2image4kpu to: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 0

                code2ack  = code2ack + '18B-'                
                 
            except:
                message2print = "{(Step 18Z/70) failed" + "; slicepitch2kpu-failed-by-write2image4kpu: " + str(slicepitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '18Z-'
            
            task4imagewrite2kpu ['SLICEPITCH'] = slicepitch2kpu                  
            
            # ---------------------------------------------------------------------------------------------
            # EOF PARAMETER PROVIDING                
            # ---------------------------------------------------------------------------------------------                                                                                                                               
        else:
            pass
            
        # ---------------------------------------------------------------------------------------------
        # CHECK PARAMETERS - WHETHER AUTOMATICALLY GENERATED OR USER GIVEN 
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2kpu.keys()
        message2print = "{(Step 20/70) using" + "; keys2task-using-by-write2image4kpu: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}

        if (('DATA2CALLBACK' in keys2task) and ('FUNCTION2CALLBACK' in keys2task)):
            pass
        else:                
            # ---------------------------------------------------------------------------------------------
            # (21B) TRACE - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            function2callback4kpu = trace2image4kpu                                                             # CALLBACK FUNCTION FOR KPU
            
            message2print = "{(Step 21B1/70) initialized" + "; function2callback4kpu-created-by-write2image4kpu to: " + str(function2callback4kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)            
            # 
            
            code2ack  = code2ack + '21B1-'
           
            args2callback = {}                                                                                  # CREATE ARGUMENT DICTIONARY FOR CALLBACK FUNCTION             
            args2callback['CLASS']   = "write2image4kpu"                                                        # MODULE OF KPUIZED OPENCL MODULE
            args2callback['METHOD']  = "write2image4kpu"                                                        # FUNCTION OF KPUIZED OPENCL MODULE
            args2callback['MESSAGE'] = "{(Step 1Z1/33) failed" + "; write2image4kpu-failed " + "; time: " + time4xi() + "}" # ERROR MESSAGE FOR KPUIZED OPENCL TO RETURN    
            message2print = "{(Step 21B2/70) initialized" + "; args2callback-created-by-write2image4kpu to: " + str(args2callback)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)               
            # 
    
            code2ack  = code2ack + '21B2-'
        
            # ---------------------------------------------------------------------------------------------
            # (21C) CALLBACK - AUTOMATIC                      
            # ---------------------------------------------------------------------------------------------              
            task4imagewrite2kpu['DATA2CALLBACK'] = args2callback                                                # USER DATA PASSED TO KPUIZED OPENCL - SEVERAL ARGS PASSED IN ONE DICTIONARY
            task4imagewrite2kpu['FUNCTION2CALLBACK'] = function2callback4kpu                                    # CALLBACK FUNCTION TO CALLED BY OPENCL IN EXCEPTIONS
            
            code2ack  = code2ack + '21C-'
            message2print = "{(Step 21C/70) initialized" + "; task4imagewrite2kpu-created-by-write2image4kpu to: " + str(task4imagewrite2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            #
             
        if 'PLATFORM' in keys2task:
            pass
        else:  
            # ---------------------------------------------------------------------------------------------
            # (22) PLATFORM - AUTOMATIC                   
            # ---------------------------------------------------------------------------------------------             
            platform2xi = {}
            message2print = "{(Step 22A/70) initialized" + "; platform2xi-created-by-write2image4kpu to: " + str(platform2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
                
            try:
                platform2xi = get2platformids4xi()                                                              # PLATFORM IS COMMON TO ALL DEVICES I.E TO DEVICE XI
                message2print = "{(Step 22B/70) got" + "; platform2xi-got-by-write2image4kpu: " + str(platform2xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                     
                code2ack  = code2ack + '22B-'
            except:
                message2print = "{(Step 22Z/70) failed" + "; platform2xi-failed-by-write2image4kpu: " + str(failed)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                                
                code2nack  = code2nack + '22Z-'
                     
            task4imagewrite2kpu['PLATFORM'] = platform2xi                                                       # PLATFORM OF SYSTEM 
              
        if 'CONTEXT' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (23) CONTEXT - AUTOMATIC                 
            # ---------------------------------------------------------------------------------------------    
            context2kpu = {}
            message2print = "{(Step 23A/70) trying" + "; context2kpu-trying-by-write2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                context2kpu = get2context4kpu()                                                                 # NO PARAMETERS - THE SYSTEM CREATES CONTEXT IF REQUIRED  
                
                message2print = "{(Step 23B/70) got" + "; context2kpu-got-by-write2image4kpu: " + str(context2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '23B-'         
            except:
                message2print = "{(Step 23Z/70) failed" + "; context2kpu-failed-by-write2image4kpu: " + str(context2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '23Z-'
                
            task4imagewrite2kpu ['CONTEXT'] = context2kpu           
        
        if 'DEVICE' in keys2task:
            pass
        else:                   
            # ---------------------------------------------------------------------------------------------
            # (24) DEVICE - AUTOMATIC                       
            # ---------------------------------------------------------------------------------------------    
            device2kpu = {}
            
            message2print = "{(Step 24A/70) trying" + "; device2kpu-trying-by-write2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                device2kpu = get2kpu4xi()                                                                       # THE SYSTEM CREATES DEVICE INFO, IF NECESSARY    
                
                message2print = "{(Step 24B/70) got" + "; device2kpu-got-by-write2image4kpu: " + str(device2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '24B-'  
            except:
                message2print = "{(Step 24Z/70) failed" + "; device2kpu-failed-by-write2image4kpu: " + str(device2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '24Z-'
            
            task4imagewrite2kpu ['DEVICE'] = device2kpu
        
        if 'IMAGE' in keys2task:
            pass
        else:                      
            # ---------------------------------------------------------------------------------------------
            # (25) IMAGE - REQUIRED                            
            # ---------------------------------------------------------------------------------------------         
            imageinfo2kpu = {}
             
            message2print = "{(Step 25A/70) trying" + "; imageinfo2kpu-trying-by-write2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            try:
                imageinfo2kpu = get2imageinfo4kpu()                                                                         
                
                message2print = "{(Step 25B/70) got" + "; imageinfo2kpu-got-by-write2image4kpu: " + str(imageinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                #                 
                code2ack  = code2ack + '25B-'  
            except:
                message2print = "{(Step 25Z/70) failed" + "; imageinfo2kpu-failed-by-write2image4kpu: " + str(imageinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '25Z-'          
                      
            task4imagewrite2kpu ['IMAGE'] = imageinfo2kpu         
                    
        if 'QUEUE' in keys2task:
            pass
        else:   
            # ---------------------------------------------------------------------------------------------
            # (26) QUEUE - REQUIRED BY KPUXI                          
            # ---------------------------------------------------------------------------------------------         
            queue2kpu = None
            
            message2print = "{(Step 26A/70) trying" + "; queue2kpu-trying-by-write2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                queue2kpu = Queue4Xi()                                                                          # THE SYSTEM CREATES QUEUE    
                
                message2print = "{(Step 26B/70) got" + "; queue2kpu-got-by-write2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '26B-'  
            except:
                message2print = "{(Step 26Z/70) failed" + "; queue2kpu-failed-by-write2image4kpu: " + str(queue2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '26Z-'
                
            task4imagewrite2kpu ['QUEUE'] = queue2kpu

        if 'PIPE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (27) PIPE - REQUIRED BY KPUXI                              
            # ---------------------------------------------------------------------------------------------         
            pipe2kpu = None
           
            message2print = "{(Step 27A/70) trying" + "; pipe2kpu-trying-by-write2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            try:
                pipe2kpu = Pipe4Xi()                                                                            # THE SYSTEM CREATES PIPE    
                
                message2print = "{(Step 27B/70) got" + "; pipe2kpu-got-by-write2image4kpu: " + str(pipe2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                                  
                # 
                
                code2ack  = code2ack + '27B-'  
            except:
                message2print = "{(Step 27Z/70) failed" + "; pipe2kpu-failed-by-write2image4kpu: " + str(pipe2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '27Z-'
                
            task4imagewrite2kpu ['PIPE'] = pipe2kpu 
            
        if 'COMMANDQUEUE' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (28) COMMAND QUEUE - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            commandqueueinfo2kpu = {}
           
            message2print = "{(Step 28A/70) trying" + "; commandqueueinfo2kpu-trying-by-write2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print)  
                        
            try:
                commandqueueinfo2kpu = get2commandqueueinfo4kpu()                                               # SYSTEM CREATES COMMAND QUEUE, IF NECESSARY   
                message2print = "{(Step 28B/70) got" + "; commandqueueinfo2kpu-got-by-write2image4kpu: " + str(commandqueueinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '28B-'  
            except:
                message2print = "{(Step 28Z/70) failed" + "; commandqueueinfo2kpu-failed-by-write2image4kpu: " + str(commandqueueinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '28Z-'
        
            task4imagewrite2kpu ['COMMANDQUEUE'] = commandqueueinfo2kpu
            
            
        if 'BUFFER' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (29) BUFFER - AUTOMATIC BY KPUXI                               
            # ---------------------------------------------------------------------------------------------             
            bufferinfo2kpu = {}
           
            message2print = "{(Step 29A/70) trying" + "; bufferinfo2kpu-trying-by-write2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                bufferinfo2kpu = get2bufferinfo4kpu()                                                           # SYSTEM CREATES BUFFER FOR KPU, IF NECESSARY
                message2print = "{(Step 29B/70) got" + "; bufferinfo2kpu-got-by-write2image4kpu: " + str(bufferinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '29B-' 
            except:
                message2print = "{(Step 29Z/70) failed" + "; bufferinfo2kpu-failed-by-write2image4kpu: " + str(bufferinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '29Z-'
        
            task4imagewrite2kpu ['BUFFER'] = bufferinfo2kpu
             
        if 'EVENT' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (30) EVENT - REQUIRED                            
            # ---------------------------------------------------------------------------------------------             
            eventinfo2kpu = {}
           
            message2print = "{(Step 30A/70) trying" + "; eventinfo2kpu-trying-by-write2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventinfo2kpu = get2eventinfo4kpu()                                                                    
                message2print = "{(Step 30B/70) got" + "; eventinfo2kpu-got-by-write2image4kpu: " + str(eventinfo2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '30B-' 
            except:
                message2print = "{(Step 30Z/70) failed" + "; eventinfo2kpu-failed-by-write2image4kpu: " + str(eventinfo2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '30Z-'
        
            task4imagewrite2kpu ['EVENT'] = eventinfo2kpu 
        
             
        if 'EVENTWAITLIST' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (31) EVENTWAITLIST - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            eventwaitlist2kpu = {}
           
            message2print = "{(Step 31A/70) trying" + "; eventwaitlist2kpu-trying-by-write2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                eventwaitlist2kpu = {}                                                                   
                message2print = "{(Step 31B/70) set" + "; eventwaitlist2kpu-set-by-write2image4kpu: " + str(eventwaitlist2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '31B-' 
            except:
                message2print = "{(Step 31Z/70) failed" + "; eventwaitlist2kpu-failed-by-write2image4kpu: " + str(eventwaitlist2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '31Z-'
        
            task4imagewrite2kpu ['EVENTWAITLIST'] = eventwaitlist2kpu                     

                 
        if 'EVENTWAITLISTQUANTITY' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (32) EVENTWAITLISTQUANTITY - REQUIRED                               
            # ---------------------------------------------------------------------------------------------             
            eventwaitlistquantity2kpu = 0
           
            message2print = "{(Step 32A/70) trying" + "; eventwaitlistquantity2kpu-trying-by-write2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                keys2event = eventwaitlist2kpu.keys()
                for key2event in keys2event:
                    eventwaitlistquantity2kpu = eventwaitlistquantity2kpu + 1                                                                     
                message2print = "{(Step 32B/70) set" + "; eventwaitlistquantity2kpu-set-by-write2image4kpu: " + str(eventwaitlistquantity2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                code2ack  = code2ack + '32B-' 
            except:
                message2print = "{(Step 32Z/70) failed" + "; eventwaitlistquantity2kpu-failed-by-write2image4kpu: " + str(eventwaitlistquantity2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '32Z-'
        
            task4imagewrite2kpu ['EVENTWAITLISTQUANTITY'] = eventwaitlistquantity2kpu       
                                                 
                 
        if 'BLOCKING' in keys2task:
            pass
        else:              
            # ---------------------------------------------------------------------------------------------
            # (33) BLOCKING - REQUIRED                        
            # ---------------------------------------------------------------------------------------------             
            blocking2kpu = ' '                                                                   
            message2print = "{(Step 33A/70) trying" + "; blocking2kpu-trying-by-write2image4kpu: " + str(blocking2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                blocking2kpu = 'CL_FALSE'                                                                       # the write command is non-blocking                                                              
                message2print = "{(Step 33B/70) set" + "; blocking2kpu-set-by-write2image4kpu to: " + str(blocking2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '33B-' 
            except:
                message2print = "{(Step 33Z/70) failed" + "; blocking2kpu-failed-by-write2image4kpu: " + str(blocking2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '33Z-'
        
            task4imagewrite2kpu ['BLOCKING'] = blocking2kpu             
                 
        if 'ORIGIN' in keys2task:
            pass
        else:                    
            # ---------------------------------------------------------------------------------------------
            # (34) ORIGIN - REQUIRED                             
            # ---------------------------------------------------------------------------------------------             
            origin2kpu = {}
            
            message2print = "{(Step 34A/70) trying" + "; origin2kpu-trying-by-write2image4kpu: " + str(origin2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                origin2kpu['X'] = 0 
                origin2kpu['Y'] = 0  
                origin2kpu['Z'] = 0                                                                                                                          
                message2print = "{(Step 34B/70) set" + "; origin2kpu-set-by-write2image4kpu to: " + str(origin2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '34B-' 
            except:
                message2print = "{(Step 34Z/70) failed" + "; origin2kpu-failed-by-write2image4kpu: " + str(origin2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '34Z-'
        
            task4imagewrite2kpu ['ORIGIN'] = origin2kpu
                
                 
        if 'REGION' in keys2task:
            pass
        else:            
            # ---------------------------------------------------------------------------------------------
            # (35) REGION - REQUIRED                         
            # ---------------------------------------------------------------------------------------------             
            region2kpu = {}
            
            message2print = "{(Step 35A/70) trying" + "; region2kpu-trying-by-write2image4kpu: " + str(region2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:                                                                                    
                region2kpu['X'] = 160                                                                               # WIDTH  IN PIXELS TO WRITE 
                region2kpu['Y'] = 90                                                                                # HEIGHT IN PIXELS TO WRITE
                region2kpu['Z'] = 0                                                                                 # DEPTH  IN PIXELS TO WRITE                                                                                                                          
                message2print = "{(Step 35B/70) set" + "; region2kpu-set-by-write2image4kpu to: " + str(region2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '35B-' 
            except:
                message2print = "{(Step 35Z/70) failed" + "; region2kpu-failed-by-write2image4kpu: " + str(region2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '35Z-'
        
            task4imagewrite2kpu ['REGION'] = region2kpu
            
                
                 
        if 'ROWPITCH' in keys2task:
            pass
        else:               
            # ---------------------------------------------------------------------------------------------
            # (36) ROWPITCH (STRIDE) - REQUIRED                              
            # ---------------------------------------------------------------------------------------------             
            # One can assume that memory representing an image or screen will have data laid out as one row of pixels followed directly by the next row of pixels. 
            # However many APIs (e.g. Direct3D) that return a pointer to image data add an extra block of data at the end of each row. 
            # int offset = x * bytesPerPixel + y * pitch;
            # When a video image is stored in memory, the memory buffer might contain extra padding bytes after each row of pixels. 
            # The padding bytes affect how the image is stored in memory, but do not affect how the image is displayed.
            # The stride is the number of bytes from one row of pixels in memory to the next row of pixels in memory. 
            # Stride is also called pitch. 
            # If padding bytes are present, the stride is wider than the width of the image, as shown in the following illustration.
     
            rowpitch2kpu = 0
            size2element = 1                                                                                        # VOXEL (X,Y,Z) = SHOT IN PIXELS, R=1 PIXELS
            voxelquantity2layer = 1                                                                                 # MAX NUMBER OF VOXELS IN EACH 2D LAYER 
            quantity2direction = 1                                                                                  # ONE DIRECTIONS IN 2D
            
            message2print = "{(Step 36A/70) trying" + "; rowpitch2kpu-trying-by-write2image4kpu: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:                                                                                                    # X-AXEL IS THE WORST CASE TO SCAN WITH Y=0, Z=0
                rowpitch2kpu = int(width2kpu * size2element * voxelquantity2layer / quantity2direction)             # (800 * 8 * 10) / 8 = 8K PIXELS MAX TO SCAN 
                                                                                                                    # SPHERE IS SPARSE HAVING FEW SHOTS PER SCAN LINE 
                                                                                                                                         
                message2print = "{(Step 36B/70) set" + "; rowpitch2kpu-set-by-write2image4kpu to: " + str(rowpitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                #                                                                                             
                 
                code2ack  = code2ack + '36B-' 
            except:
                message2print = "{(Step 36Z/70) failed" + "; rowpitch2kpu-failed-by-write2image4kpu: " + str(rowpitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '36Z-'
        
            task4imagewrite2kpu ['ROWPITCH'] = rowpitch2kpu     
                 
        if 'POINTER' in keys2task:
            pass
        else:                
            # ----------------------------------------------------------------------------------------------------
            # (37) POINTER - REQUIRED - THE POINTER TO A BUFFER IN HOST MEMORY WHERE IMAGE DATA IS TO BE READ FROM                          
            # ----------------------------------------------------------------------------------------------------             
            pointer2kpu = None
            size2kpu = int(width2kpu * height2kpu)                                                                  # CALCULATED MEMORY
            if size2kpu == 0:
                size2kpu = 4400                                                                                     # MINIMUM MEMORY
            
            message2print = "{(Step 37A/70) trying" + "; pointer2kpu-trying-by-write2image4kpu: " + str(pointer2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
            # 
                        
            try:
                arena2kpu = Arena4Xi(size2kpu)                                                                      # SYSTEM RESERVER MEMORY FOR IMAGE                             
                                                                                                                                         
                message2print = "{(Step 37B/70) got" + "; arena2kpu-got-by-write2image4kpu: " + str(arena2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                
                size2arena = arena2kpu.size                                                                         # 
                              
                message2print = "{(Step 37C/70) got" + "; size2arena-got-by-write2buffer4kpu: " + str(size2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                buffer2arena = arena2kpu.buffer
                
                message2print = "{(Step 37D/70) got" + "; buffer2arena-got-by-write2buffer4kpu: " + str(buffer2arena)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                code2ack  = code2ack + '37D-' 
                
                if size2arena < size2kpu:
                    size2kpu = size2arena                                                                           # AVAILABLE MEMORY FOR IMAGE
                    message2print = "{(Step 37E/70) set" + "; size2kpu-set-by-write2buffer4kpu to: " + str(size2kpu)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                    
                    code2ack  = code2ack + '37E-'
                else:
                    pass
                    
                pointer2kpu = buffer2arena
                message2print = "{(Step 37F/70) set" + "; pointer2kpu-set-by-write2buffer4kpu to: " + str(pointer2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                
                code2ack  = code2ack + '37F-'       
            except:
                message2print = "{(Step 37Z/70) failed" + "; pointer2kpu-failed-by-write2image4kpu: " + str(pointer2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                # 
                
                code2nack  = code2nack + '37Z-'
        
            task4imagewrite2kpu ['POINTER'] = pointer2kpu                            
            task4imagewrite2kpu ['SIZE']    = size2kpu                                                               # AUTOMATIC BY KPUXI
        
             
        if 'SLICEPITCH' in keys2task:
            pass
        else:               
            # --------------------------------------------------------------------------------------------------------
            # (38) SLICEPITCH - REQUIRED - THE SIZE IN BYTES OF THE 2D SLICE OF THE 3D REGION OF A 3D IMAGE BEING READ                         
            # --------------------------------------------------------------------------------------------------------             
            slicepitch2kpu = None                                                                                   # SLICEPITCH IS ZERO FOR 2D IMAGES                           
           
            message2print = "{(Step 38A/70) trying" + "; slicepitch2kpu-trying-by-write2image4kpu: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}" 
            print4xi(class2print, method2print, message2print) 
                        
            try:
                slicepitch2kpu = 0                                                          
                message2print = "{(Step 38B/70) set" + "; slicepitch2kpu-set-by-write2image4kpu to: " + str(slicepitch2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
                # 
                code2ack  = code2ack + '38B-'                
                 
            except:
                message2print = "{(Step 38Z/70) failed" + "; slicepitch2kpu-failed-by-write2image4kpu: " + str(slicepitch2kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)             
                code2nack  = code2nack + '38Z-'
            
            task4imagewrite2kpu ['SLICEPITCH'] = slicepitch2kpu                  

        # ---------------------------------------------------------------------------------------------
        # EOF PARAMETER CHECKING AND PROVIDING FOR MISSING ONES
        # ---------------------------------------------------------------------------------------------
        keys2task = task4imagewrite2kpu.keys()
        message2print = "{(Step 39/70) set" + "; keys2task-set-by-write2image4kpu to: " + str(sorted(keys2task))  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # ['BLOCKING',
        #  'BUFFER',
        #  'COMMANDQUEUE',
        #  'CONTEXT',
        #  'DATA2CALLBACK',
        #  'DEVICE',
        #  'EVENT',
        #  'EVENTWAITLIST',
        #  'EVENTWAITLISTQUANTITY',
        #  'FUNCTION2CALLBACK',
        #  'IMAGE',
        #  'ORIGIN',
        #  'PIPE',
        #  'PLATFORM',
        #  'POINTER',
        #  'QUEUE',
        #  'REGION',
        #  'ROWPITCH',
        #  'SIZE',
        #  'SLICEPITCH']; time: 02:45:31 10/05/11 Pacific Daylight Time}
             
    except: 
        message2print = "{(Step 39Z/70) yielded" + "; task4imagewrite2kpu-yielded-by-write2image4kpu: " + str(task4imagewrite2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)  
        # 
        
        code2nack  = code2nack + '39Z-' 

    # ---------------------------------------------------------------------------------------------
    # CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    control2writeimage4kpu = {}       
                                
    # ---------------------------------------------------------------------------------------------
    # CONSUMER AND PUBLISHER CHANNEL FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4channels2kpu = False
    channels2kpu = ()
    message2print = "{(Step 40A/70) trying" + "channels2kpu-trying-by-write2image4kpu: " + str(channels2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        # 
        consumerchannel2kpu, publisherchannel2kpu = task4imagewrite2kpu ['PIPE']

        message2print = "{(Step 40B/70) got" + "; consumerchannel2kpu-got-by-write2image4kpu: " + str(consumerchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 568>

        message2print = "{(Step 40C/70) got" + "; publisherchannel2kpu-got-by-write2image4kpu: " + str(publisherchannel2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <read-write PipeConnection, handle 572>                     
        ack4channels2kpu = True
        
        control2writeimage4kpu['CONSUMERCHANNEL']  = consumerchannel2kpu
        
        
        control2writeimage4kpu['PUBLISHERCHANNEL'] = publisherchannel2kpu
        
        code2ack  = code2ack + '40D-' 
        
    except:
        ack4channels2kpu = False
        code2nack  = code2nack + '40Z-'      
                        
    # ---------------------------------------------------------------------------------------------
    # QUEUE FOR CONTROL AND COMMAND
    # ---------------------------------------------------------------------------------------------
    ack4queue2kpu = False
    queue2kpu = None
    message2print = "{(Step 41A/70) trying" + "queue2kpu-trying-by-write2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)     
    try:
        queue2kpu = task4imagewrite2kpu ['QUEUE']

        message2print = "{(Step 41B/70) got" + "; queue2kpu-got-by-write2image4kpu: " + str(queue2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # <queue4xi.Queue4Xi object at 0x0000000003C83A90> 
              
        ack4queue2kpu = True
        control2writeimage4kpu['QUEUE']  = queue2kpu        
        code2ack  = code2ack + '41C-'
    except:
        ack4queue2kpu = False  
        code2nack  = code2nack + '41Z-'
    
    
    # -----------------------------------------------------------------------------------------    
    # DUMMY PROCESS OBJECT = THREAD CREATION
    # -----------------------------------------------------------------------------------------           
    # dummy2image4kpu = None
    # message2print = "{(Step 69A/70) trying" + "dummy2image4kpu-trying-by-write2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}"   
    # print4xi(class2print, method2print, message2print)     
    # try:                            
        # dummy2image4kpu = DummyProcess4Xi(target=manoeuvre2readimage4kpu,                                               # in this module to handle queue and pipe channels
        #                                   args =(queue2kpu,                                                         # caller puts and gets - does NOT call any functions
        #                                          consumerchannel2kpu, publisherchannel2kpu))                        # caller sends and receives - does NOT call any functions
        # message2print = "{(Step 69B/70) created" + "dummy2image4kpu-created-by-write2image4kpu: " + str(dummy2image4kpu)  + "; time: " + time4xi() + "}" 
        # print4xi(class2print, method2print, message2print)  
        # code2ack  = code2ack + '69B-'
        
    # except:
        # code2nack  = code2nack + '69Z-'
        
    # ---------------------------------------------------------------------------------------------
    # RETURN CONTROL AND COMMAND DICTIONARY
    # ---------------------------------------------------------------------------------------------          
    message2print = "{(Step 70A/70) set" + "; code2ack-set-by-write2image4kpu: " + str(code2ack) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)                                                             
    # 1B1-1B2-1C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-12B-13B-14B-15B-16B-17D-17F-18B-40D-41C-
     
    message2print = "{(Step 70B/70) set" + "; code2nack-set-by-write2image4kpu: " + str(code2nack) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    #  
        
    message2print = "{(Step 70C/70) returning" + "; control2writeimage4kpu-returning-by-write2image4kpu: " + str(control2writeimage4kpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000003C83A90>,
    #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 572>,
    #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 568>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

    return control2writeimage4kpu
                    
#</function4xi> 


 
# -------------------------------------------------------------------------------------------------
#<function4xi function = "release2image4kpu" usage = "
#
#   task4release2image  = {} 
#   task4release2image  ['COUNT2REFERENCE'] = 1 
#   response4release2image = {}
#    
#   print('(Step 12A/33) OF KPU2IMAGE4XI5TEST - TRYING TO RELEASE IMAGE OF KPU DEVICE ', time4xi())   
#   try:                               
#       response4release2image = release2image4kpu(task4release2image)                                                                                             
#       print('(Step 12B/33) OF KPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4release2image)                           
#   except:
#       print('(Step 12Z1/33) OF KPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF KPU DEVICE ', time4xi())  
#   
#                 
#">

# -------------------------------------------------------------------------------------------------   
def release2image4kpu(task4release2kpu=None):

    '''
    releases kpu2image4xi
    '''
    global kpu2image4xi  
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33) PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "release2image4kpu"
    method2print = "release2image4kpu" 

    message2print = "{(Step 1A/33) started" + "; release2image4kpu-started: " + str(release2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function release2image4kpu at 0x0000000003A9E8C8>
    
    message2print = "{(Step 1B/33) passed" + "; task4release2kpu-to-release2image4kpu: " + str(task4release2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1} 

             
    code2ack  = ' '
    code2nack = ' '

    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()                                                                       # INITIALIZE IMAGE                 
    except:
        try:
            kpu2image4xi = create2image4kpu()
        except:
            kpu2image4xi = {}
            message2print = "{(Step 2Z0/70) yielded" + "; kpu2image4xi-yielded-by-release2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '2Z0-' 
                
    message2print = "{(Step 1A/70) try" + "; release-try-by-release2image4kpu: None"  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    
    if task4release2kpu is None:
        task4release2kpu = {}
        task4release2kpu['COUNT2REFERENCE'] = 1
           
    try:
        # -----------------------------------------------------------------------------------------
        # RELEASE
        # -----------------------------------------------------------------------------------------               
        kpu2image4xi['COUNT2REFERENCE'] = kpu2image4xi['COUNT2REFERENCE'] - task4release2kpu['COUNT2REFERENCE']
        
        message2print = "{(Step 1A/70) calculated" + "; count2reference-calculated-by-release2image4kpu: " + str(kpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
         
        if kpu2image4xi['COUNT2REFERENCE'] < 0:
            kpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 1B/70) warning" + "; count2reference-warning-by-release2image4kpu: " + " ** " + str(kpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            # { release2image4kpu : release2image4kpu ; (Step 8Z1/70) warning; count2reference-warning-by-release2image4kpu:  ** 0 ** ; time: 23:41:37 09/06/11 Pacific Daylight Time}    
            code2nack  = code2nack + '1B-'
        else:
            code2ack  = code2ack + '1B-'
    except:
        try:
            kpu2image4xi['COUNT2REFERENCE'] = 0
            
            message2print = "{(Step 8Z1/70) warning" + "; count2reference-warning-by-release2image4kpu: " + " ** " + str(kpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            # 
            code2nack  = code2nack + '8Z1-'
            
        except:
            message2print = "{(Step 8Z2/70) yielded" + "; kpu2image4xi-yielded-by-release2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '8Z2-'
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70)           
    # -----------------------------------------------------------------------------------------
    kpu2image4xi ['ACK2RETURN']  = code2ack
    kpu2image4xi ['NACK2RETURN'] = code2nack  
    
    message2print = "{(Step 70/70) returning" + "; kpu2image4xi-returning-by-release2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0,
    #  'ACK2RETURN': ' ',
    #  'ROWPITCH': 800,
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}',
    #  'CLASS': 'create2image4kpu',
    #  'METHOD': 'create2image4kpu'},
    #  'WIDTH': 800,
    #  'BUFFER': {'CODE2ERROR': 0, 
    #  'COUNT2REFERENCE': 0,
    
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 23:41:37 09/06/11 Pacific Daylight Time}', 'CLASS': 'create2buffer4kpu', 'METHOD': 'create2buffer4kpu'},
    #  'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x0000000003A97348>,
    #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CDE298>,
    #  'FLAGS': 0, 'CONTEXT': {'COUNT2REFERENCE': 0, ...
    
    
    return kpu2image4xi

#</function4xi> 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "retain2image4kpu" usage = "
#   task4retain2image  = {}
#   task4retain2image  ['DIMENSION'] = 2 
#   task4retain2image  ['COUNT2REFERENCE'] = 1 
#   response4retain2Dimage = {}
#   print('(Step 13A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF KPU DEVICE ', time4xi())  
#   try:                               
#       response4retain2Dimage = retain2image4kpu(task4retain2image)                                                                                                 
#       print('(Step 13B/700) OF KPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', response4retain2Dimage)                          
#       # 
#   except:
#       print('(Step 13Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF KPU DEVICE ', time4xi()) 
#
#">

# -------------------------------------------------------------------------------------------------   
def retain2image4kpu(task4retain2kpu=None):

    '''
    retains either kpu2image4xi
    '''
    global kpu2image4xi 
    
     
    global code2ack
    global code2nack
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/70) - PARAMETERS           
    # -----------------------------------------------------------------------------------------          
    class2print = "retain2image4kpu"
    method2print = "retain2image4kpu"

    message2print = "{(Step 1A/33) started" + "; retain2image4kpu-started: " + str(retain2image4kpu)  + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                                     #  
    # <function retain2image4kpu at 0x0000000003A9E948>   
    
    message2print = "{(Step 1B/33) passed" + "; task4retain2kpu-to-retain2image4kpu: " + str(task4retain2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)                                                              
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 2}
    # {'COUNT2REFERENCE': 1, 'DIMENSION': 3}
    code2ack  = ' '
    code2nack = ' '
    
    # -----------------------------------------------------------------------------------------
    # (Step 2/70) - PROVIDE TASKS, IF NECESSARY AND CHECK GIVEN TASKS 
    # -----------------------------------------------------------------------------------------    
    if task4retain2kpu is None:
        task4retain2kpu = {}
        task4retain2kpu['DIMENSION'] = 2
        task4retain2kpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 2A/33) set" + "; task4retain2kpu-set-by-retain2image4kpu to: " + str(task4retain2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
    else:
        pass
        
    if 'DIMENSION' in task4retain2kpu.keys():
        if task4retain2kpu['DIMENSION'] == 2:
            pass
        else:
            task4retain2kpu['DIMENSION'] = 2 
            message2print = "{(Step 2B/33) set" + "; dimension-set-by-retain2image4kpu to: " + str(task4retain2kpu['DIMENSION'])  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                       
    else:
        task4retain2kpu['DIMENSION'] = 2
    
    # -----------------------------------------------------------------------------------------
    # (Step 3/70) - CHECK COUNT2REFERENCE
    # -----------------------------------------------------------------------------------------         
    if 'COUNT2REFERENCE' in task4retain2kpu.keys():
        pass
    else:
        task4retain2kpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3A/33) set" + "; count2reference-set-by-retain2image4kpu to: " + str(task4retain2kpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # 
                                   
    if task4retain2kpu['COUNT2REFERENCE'] < 1:
        task4retain2kpu['COUNT2REFERENCE'] = 1
        message2print = "{(Step 3B/33) set" + "; count2reference-set-by-retain2image4kpu to: " + str(task4retain2kpu['COUNT2REFERENCE'])  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #           
        
    # -----------------------------------------------------------------------------------------
    # (Step 4/70) - INITIALIZE 2D IMAGE IF NECESSARY
    # -----------------------------------------------------------------------------------------
    try:
        if kpu2image4xi is None:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 4A/70) created" + "; kpu2image4xi-created-by-retain2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                                                                                          
    except:
        try:
            kpu2image4xi = create2image4kpu()
            message2print = "{(Step 4B/70) created" + "; kpu2image4xi-created-by-retain2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                 
        except:
            kpu2image4xi = {}
            message2print = "{(Step 4Z/70) yielded" + "; kpu2image4xi-yielded-by-retain2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '4Z-' 


    # -----------------------------------------------------------------------------------------
    # (Step 5/70) - RETAIN
    # -----------------------------------------------------------------------------------------             
    try:
        kpu2image4xi['COUNT2REFERENCE'] = kpu2image4xi['COUNT2REFERENCE'] + task4retain2kpu['COUNT2REFERENCE']
        message2print = "{(Step 6A/70) calculated" + "; count2reference-calculated-by-retain2image4kpu: " + str(kpu2image4xi['COUNT2REFERENCE']) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        #              
        if kpu2image4xi['COUNT2REFERENCE'] <= 0:
            kpu2image4xi['COUNT2REFERENCE'] = 1
            message2print = "{(Step 6Z1/70) warning" + "; count2reference-warning-by-retain2image4kpu: " + " ** " + str(kpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)      
            code2nack  = code2nack + '6Z1-'
        else:
            code2ack  = code2ack + '6C-'

    except:
        try:
            kpu2image4xi['COUNT2REFERENCE'] = 1
        
            message2print = "{(Step 7Z1/70) warning" + "; count2reference-warning-by-retain2image4kpu: " + " ** " + str(kpu2image4xi['COUNT2REFERENCE'])  + " ** " + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)   
            code2nack  = code2nack + '7Z1-'
        
        except:
            message2print = "{(Step 7Z2/70) yielded" + "; kpu2image4xi-yielded-by-retain2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
            code2nack  = code2nack + '7Z2-'
        
                                                                                          
            
    # -----------------------------------------------------------------------------------------
    # (Step 70/70) - RETURN           
    # -----------------------------------------------------------------------------------------
    
    kpu2image4xi ['ACK2RETURN']  = code2ack
    kpu2image4xi ['NACK2RETURN'] = code2nack     
    message2print = "{(Step 70A/70) returning" + "; kpu2image4xi-returning-by-retain2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)                                                             
    # {'CODE2ERROR': 0, 'ACK2RETURN': ' 6C-', 'ROWPITCH': 800, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 23:25:02 10/05/11 Pacific Daylight Time}', 'CLASS': 'create2image4kpu', 'METHOD': 'create2image4kpu'}, 'WIDTH': 800, 'BUFFER': {'CODE2ERROR': 0, 
    #                   'COUNT2REFERENCE': 0, ...
    # 'COUNT2REFERENCE': 1, 
    # 'FLAGS': 0, 'CONTEXT': ...
    
    return kpu2image4xi

#</function4xi> 


# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2imageinfo4kpu" usage = "
#   #1
#   print('(Step 3A/33) OF KPU2IMAGE4XI5TEST - TRYING TO GET 2D IMAGE INFO FOR KPU', time4xi())  
#   try:
#       imageinfo2kpu = get2imageinfo4kpu()                                                                       
#       print('(Step 3B/33) OF KPU2IMAGE4XI5TEST - GOT 2D IMAGE INFO FOR KPU: ', imageinfo2kpu) 
#   except:
#       print('(Step 3Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO GET 2D IMAGE INFO FOR KPU', time4xi())
#   #2
#   task4getinfo2image = {}
#   task4getinfo2image ['DIMENSION'] = 2
#   task4getinfo2image ['WIDTH'] = 0
#   task4getinfo2image ['HEIGHT'] = 0
#   response4getinfo2image = {}
#   print('(Step 12C/33) OF KPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 3D IMAGE INFO OF KPU DEVICE ', time4xi())    
# 
#   try:                               
#       response4getinfo2image = get2imageinfo4kpu (task4getinfo2image)                                                                                             
#       print('(Step 12D/33) OF KPU2IMAGE4XI5TEST - RESPONSE4RELEASE2IMAGE : ', response4getinfo2image)                           
#       #
#   except:
#       print('(Step 12Z2/33) OF KPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 3D IMAGE INFO OF KPU DEVICE ', time4xi()) 
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2imageinfo4kpu(task4getinfo2image=None):

    '''
    gets kpu2image4xi
    '''
    
    global kpu2image4xi
    
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2imageinfo4kpu"
    method2print = "get2imageinfo4kpu"

    message2print = "{(Step 0A/33) started" + "; get2imageinfo4kpu-started: " + str(get2imageinfo4kpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # <function get2imageinfo4kpu at 0x0000000003A9E9C8>                                                              
    
    message2print = "{(Step 0A/33) passed" + "; task4getinfo2image-passed-to-get2imageinfo4kpu: " + str(task4getinfo2image) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # {'WIDTH': 0, 'DIMENSION': 2, 'HEIGHT': 0}
    # {'WIDTH': 0, 'DEPTH': 0, 'DIMENSION': 3, 'HEIGHT': 0}
    # {'DOMAIN': 'ALL', 'DIMENSION': 2}
    
    image2kpu = {}
    try: 
        if kpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE 2D IMAGE FOR KPU           
            # -------------------------------------------------------------------------------------           
            image2kpu = create2image4kpu()
            message2print = "{(Step 1B2/33) initialized" + "; image2kpu-initialized-by-get2imageinfo4kpu to: " + str(image2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
        else:
            message2print = "{(Step 1B2/33) using" + "; kpu2image4xi-using-by-get2imageinfo4kpu: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            # {'CODE2ERROR': 0, 
            #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-', 
            #  'ROWPITCH': 800, 
            #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
            #  'CLASS': 'create2image4kpu', 
            #  'METHOD': 'create2image4kpu'}, 
            #  'WIDTH': 800, 
            #  'BUFFER': ...
            
            
        # -------------------------------------------------------------------------------------
        # RETURN THE WHOLE OR PART OF THE IMAGE INFO OF KPU          
        # -------------------------------------------------------------------------------------     
        dimension2xi = 0
        
        if task4getinfo2image is None:
            task4getinfo2image = {}
            task4getinfo2image ['DIMENSION'] = 2 
            task4getinfo2image ['DOMAIN'] = 'ALL'
            image2kpu = kpu2image4xi
            message2print = "{(Step 1C/33) using" + "; image2kpu-got-by-get2imageinfo4kpu: " + str(image2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # {'CODE2ERROR': 0, 
            #  'DOMAIN': 'ALL', 
            #  'CONTEXT': ...
            
        else:
            pass    

        keys2task4imageinfo = task4getinfo2image.keys()
        if 'DIMENSION' in keys2task4imageinfo:
            dimension2xi = task4getinfo2image['DIMENSION']
            if dimension2xi == 2:
                pass
            else:
                dimension2xi = 2    
        else:
            pass
            
        if 'DOMAIN' in keys2task4imageinfo:
            domain2xi = task4getinfo2image['DOMAIN']
            if domain2xi == 'ALL':
                pass
            else:
                domain2xi = 'ALL'

            keys2domain = kpu2image4xi.keys()
            for key2domain in keys2domain:
                task4getinfo2image[key2domain] = kpu2image4xi[key2domain]
                    
            message2print = "{(Step 1C/33) using" + "; task4getinfo2image-using-by-get2imageinfo4kpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                          
        else:
            pass            
            
                    

        keys2task = task4getinfo2image.keys()
        for key2task in keys2task:
            try:
                message2print = "{(Step 2D2/33) using" + "; key2task-using-by-get2image4kpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # WIDTH; DIMENSION; HEIGHT  
                image2kpu [key2task] = kpu2image4xi[key2task]
                
                message2print = "{(Step 2D2/33) set" + "; info-of-image2kpu-set-by-get2imageinfo4kpu to: " + str(image2kpu [key2task])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 800; 800  
            except:
                message2print = "{(Step 2Z/70) yielded" + "; key2task-yielded-by-get2image4kpu: " + str(key2task) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                               
                # DIMENSION;
                                   
                message2print = "{(Step 2Z2/70) using" + "; task4getinfo2image-using-by-get2image4kpu: " + str(task4getinfo2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)  
                # 
                
                message2print = "{(Step 2Z3/70) using" + "; kpu2image4xi-using-by-get2image4kpu: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #                           
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2kpu = create2image4kpu()
            message2print = "{(Step 1F/33) initialized" + "; image2kpu-initialized-by-get2imageinfo4kpu to: " + str(image2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
            if task4getinfo2image is None:
                image2kpu = kpu2image4xi
                message2print = "{(Step 1G/33) got" + "; image2kpu-got-by-get2imageinfo4kpu: " + str(image2kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            else:
                keys2task = task4getinfo2image.keys()
                for key2task in keys2task:
                    try:
                        message2print = "{(Step HD2/33) using" + "; key2task-using-by-get2image4kpu: " + str(key2task)  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)  
                        # 
                        image2kpu [key2task] = kpu2image4xi[key2task]
                    
                        message2print = "{(Step HD2/33) set" + "; info-of-image2kpu-set-by-get2imageinfo4kpu to: " + str(image2kpu [key2task])  + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        #   
                    except:
                        message2print = "{(Step HZ/70) yielded" + "; key2task-yielded-by-get2image4kpu: " + str(key2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)                            
        except:
            image2kpu = {}
            message2print = "{(Step 1Z3/70) yielded" + "; image2kpu-yielded-by-get2image4kpu: " + str(image2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; image2kpu-returning-by-get2imageinfo4kpu: " + str(image2kpu) + "; time: " + time4xi() + "}"  
    print4xi(class2print, method2print, message2print)
    # 2D: {'WIDTH': 800, 'HEIGHT': 800}
    # 3D: {'WIDTH': 800, 'DEPTH': 800, 'HEIGHT': 800}
    # 2D ALL: {'CODE2ERROR': 0, 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create2image4kpu', 
    #  'METHOD': 'create2image4kpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ...
    
    # 3D ALL: {'PIPE': (<read-write PipeConnection, handle 520>, <read-write PipeConnection, handle 524>), 
    #  'ROWPITCH': 800, 
    #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create3image4kpu-failed ; time: 09:05:31 10/05/11 Pacific Daylight Time}', 
    #  'CLASS': 'create3image4kpu', 
    #  'METHOD': 'create3image4kpu'}, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'BUFFER': ....
    #  'HEIGHT': 800, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004504400>, 
    #  'WIDTH': 800, 
    #  'DEPTH': 800, 
    #  'FLAGS': 0, 
    #  'CONTEXT': ...
    
    return image2kpu
            
#</function4xi> 
 

# -------------------------------------------------------------------------------------------------
#<function4xi function = "get2supportedimageformats4kpu" usage = "
#
#   print('(Step 3A/33) OF KPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR KPU', time4xi())  
#   try:
#       imageformats = get2supportedimageformats4kpu()                                                                       
#       print('(Step 3B/33) OF KPU2IMAGE4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR KPU: ', imageformats) 
#   except:
#       print('(Step 3Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR KPU', time4xi())
#            
#">

# -------------------------------------------------------------------------------------------------   
def get2supportedimageformats4kpu(task4supportedimageformats2kpu=None):  

    '''
    gets supported image formats for kpu
    '''
    
    global kpu2image4xi

    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "get2supportedimageformats4kpu"
    method2print = "get2supportedimageformats4kpu"

    message2print = "{(Step 0A/33) started" + "; get2supportedimageformats4kpu-started: " + str(get2supportedimageformats4kpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print) 
    # <function get2supportedimageformats4kpu at 0x0000000003BBE8C8>                                                          
    
    message2print = "{(Step 0B/33) passed" + "; task4supportedimageformats2kpu-passed-to-get2supportedimageformats4kpu: " + str(task4supportedimageformats2kpu) + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)   
    # None; {'DIMENSION': 3} 
    
    
    if task4supportedimageformats2kpu is None:
        task4supportedimageformats2kpu = {} 
        task4supportedimageformats2kpu['DIMENSION'] = 2                                                             # 2D image is the default
    
    dimension2xi = 2
    try:
        keys2supportedimageformats = task4supportedimageformats2kpu.keys()    
        if 'DIMENSION' in keys2supportedimageformats:
            dimension2xi = task4supportedimageformats2kpu['DIMENSION']
            if ((dimension2xi <=0) or (dimension2xi > 2)): 
                dimension2xi = 2
        else:
            dimension2xi = 2
    except:
        pass        
    
    message2print = "{(Step 1A/33) using" + "; dimension2xi-using-by-dimension2xi: " + str(dimension2xi)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)    
    # 2; 3
                        
    supportedimageformats2kpu = {} 
    try: 
        if kpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2kpu = create2image4kpu()                                                                          # create kpu2image4xi
            message2print = "{(Step 1B/33) initialized" + "; image2kpu-initialized-by-get2supportedimageformats4kpu to: " + str(image2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #                
        else:
            pass

                        
        # -------------------------------------------------------------------------------------
        # SUPPORTED IMAGE FORMATS FOR KPU       
        # -------------------------------------------------------------------------------------     
        supportedimageformats2kpu = kpu2image4xi['FORMAT']
        message2print = "{(Step 1E/33) got" + "; supportedimageformats2kpu-got-by-get2supportedimageformats4kpu: " + str(supportedimageformats2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)           
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
            
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            image2kpu = create2image4kpu()
            message2print = "{(Step 1F1/33) initialized" + "; image2kpu-initialized-by-get2supportedimageformats4kpu to: " + str(image2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
     
            
            # -------------------------------------------------------------------------------------
            # SUPPORTED IMAGE FORMATS FOR KPU       
            # -------------------------------------------------------------------------------------     
            supportedimageformats2kpu = kpu2image4xi['FORMAT']
            message2print = "{(Step 1H/33) got" + "; supportedimageformats2kpu-got-by-get2supportedimageformats4kpu: " + str(supportedimageformats2kpu)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)           
            #  
                                              
        except:
            message2print = "{(Step 1Z/70) yielded" + "; supportedimageformats2kpu-yielded-by-get2image4kpu: " + str(supportedimageformats2kpu) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; supportedimageformats2kpu-returning-by-get2supportedimageformats4kpu: " + str(supportedimageformats2kpu) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # 2: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    # 3: {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
    
    return supportedimageformats2kpu
            
#</function4xi> 

 



# -------------------------------------------------------------------------------------------------
#<function4xi function = "wait2image4kpu" usage = "
#   print('=' *33)          
#   print('(Step 16A/700) OF KPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE FOR KPU ', time4xi())   
#   print('=' *33)      
#   try:
#       response4wait2kpu = wait2image4kpu()
#       keys2response = response4wait2kpu.keys()
#       print('(Step 16B/700) OF KPU2IMAGE4XI5TEST - WAITED 2D IMAGE FOR KPU ', time4xi()) 
#       for key2response in sorted(keys2response):
#           print(key2response, ' : ', response4wait2kpu[key2response])
#           # ACK2RETURN  :   6C-
#           # BUFFER  :  {'CODE2ERROR': 0 ...
#           # CALLBACK  :  <function trace2image4kpu at 0x0000000004E93B48>
#           # CODE2ERROR  :  0
#           # COMMANDQUEUE  :  {'CODE2ERROR': 0, ...
#           # CONTEXT  :  {'COUNT2REFERENCE': 0 ...
#           # COUNT2REFERENCE  :  1
#           # DATA2CALLBACK  :  {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 02:00:43 10/06/11 Pacific Daylight Time}', 'CLASS': 'create2image4kpu', 'METHOD': 'create2image4kpu'}
#           # DEPTH  :  0
#           # DEVICE  :  {'RESOLUTION2TIMER4PROFILING': 318, 'ID2PLATFORM': '0000000001509568', 'PROFILING2QUEUE': 'ACK',...
#           # EMPTY2QUEUE  :  True
#           # EVENT  :  {'CODE2ERROR': 0, 'DATA2CALLBACK':...
#           # FLAGS  :  0
#           # FORMAT  :  {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'}
#           # FULL2QUEUE  :  False
#           # FUNCTION2CALLBACK  :  <function trace2image4kpu at 0x0000000004E93B48>
#           # HEIGHT  :  800
#           # NACK2RETURN  :   
#           # PIPE  :  (<read-write PipeConnection, handle 556>, <read-write PipeConnection, handle 560>)
#           # PLATFORM  :  {'list2platforms': ['DragonXi'], 'ack2return': ' 2C-2E-2F-2G-', 'nack2return': ' ', 'available2platform': 1, 'max2entries': 1}
#           # POINTER  :  <mmap.mmap object at 0x0000000004351500>
#           # QUEUE  :  <queue4xi.Queue4Xi object at 0x00000000044E95C0>
#           # ROWPITCH  :  800
#           # SIZE2QUEUE  :  2147483646 
#   except:
#       print('(Step 16Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO WAIT 2D IMAGE FOR KPU: ', wait2kpu)            
#">

# -------------------------------------------------------------------------------------------------   
def wait2image4kpu(task4wait2image=None):

    '''
    waits kpu2image4kpu
    '''
    
    global kpu2image4xi
    
    # -----------------------------------------------------------------------------------------
    # (Step 1/33)           
    # -----------------------------------------------------------------------------------------          
    class2print = "wait2image4kpu"
    method2print = "wait2image4kpu"

    message2print = "{(Step 1A1/33) started" + "; wait2image4kpu-started: " + str(wait2image4kpu)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    #  
    
    message2print = "{(Step 1A2/33) passed" + "; task4wait2image-passed-by-wait2image4kpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
    print4xi(class2print, method2print, message2print)
    # 
    
    try: 
        if kpu2image4xi is None:
            # -------------------------------------------------------------------------------------
            # INITIALIZE - KEEP SYSTEM RUNNING           
            # -------------------------------------------------------------------------------------           
            kpu2image4xi = create2image4kpu()
            
            message2print = "{(Step 1B/33) created" + "; kpu2image4xi-created-by-wait2image4kpu: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                                                
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4kpu = None
            try:
                queue2image4kpu = kpu2image4xi['QUEUE']
                message2print = "{(Step 1C/33) got" + "; queue2image4kpu-got-by-wait2image4kpu to: " + str(queue2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z1/70) yielded" + "; kpu2xi-yielded-by-wait2image4kpu: " + str(kpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            size2image4kpu = 0
            try:                    
                size2image4kpu = queue2image4kpu.qsize()
                message2print = "{(Step 1D/33) got" + "; size2image4kpu-got-by-wait2image4kpu to: " + str(size2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
                               
                
            except:     
                message2print = "{(Step 1Z2/70) yielded" + "; kpu2xi-yielded-by-wait2image4kpu: " + str(kpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
             
            empty2image4kpu = True       
            try:                    
                empty2image4kpu= queue2image4kpu.empty()
                message2print = "{(Step 1E/33) got" + "; empty2image4kpu-got-by-wait2image4kpu to: " + str(empty2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z3/70) yielded" + "; empty2image4kpu-yielded-by-wait2image4kpu: " + str(empty2image4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
             
            full2image4kpu = False       
            try:                                
                full2image4kpu = queue2image4kpu.full()
                message2print = "{(Step 1F/33) got" + "; full2image4kpu-got-by-wait2image4kpu to: " + str(full2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z4/70) yielded" + "; full2image4kpu-yielded-by-wait2image4kpu: " + str(full2image4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)   
                            
            message2print = "{(Step 1G/33) finished" + "; kpu2image4xi-finished-by-wait2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #
                            
        else:
            # -------------------------------------------------------------------------------------
            # USE THE EXISTING IMAGE FOR KPU          
            # -------------------------------------------------------------------------------------     
            
            message2print = "{(Step 1H/33) used" + "; kpu2image4xi-used-by-wait2image4kpu: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4kpu = None
            try:
                queue2image4kpu = kpu2image4xi['QUEUE']
                message2print = "{(Step 1I/33) got" + "; queue2image4kpu-got-by-wait2image4kpu to: " + str(queue2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
            except:     
                message2print = "{(Step 1Z5/70) yielded" + "; kpu2xi-yielded-by-wait2image4kpu: " + str(kpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                message2print = "{(Step 1J1/33) passed" + "; task4wait2image-passed-by-wait2image4kpu to: " + str(task4wait2image)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                # 
                queue2task = {}    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1J2/33) set" + "; queue2task-set-by-wait2image4kpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    # 
                except:     
                    message2print = "{(Step 1Z61/70) yielded" + "; queue2task-yielded-by-wait2image4kpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                
                try:                        
                    if queue2task == queue2image4kpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1K/33) accepted" + "; queue2task-accepted-by-wait2image4kpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        try:
                            message2print = "{(Step 1L/33) warned" + "; queue2task-warned-by-wait2image4kpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print) 
                            # 
                        
                            message2print = "{(Step 1M/33) warned" + "; queue2image4kpu-warned-by-wait2image4kpu: " + " ** " + str(queue2image4kpu)  + " ** " + "; time: " + time4xi() + "}"
                            print4xi(class2print, method2print, message2print)    
                            # 
                        except:
                            pass
                                                     
                except:     
                    message2print = "{(Step 1Z62/70) yielded" + "; queue2task-yielded-by-wait2image4kpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)                   
            
            size2queue4kpu = 0        
            try:                    
                size2queue4kpu = queue2image4kpu.qsize()
                message2print = "{(Step 1N1/33) got" + "; size2queue4kpu-got-by-wait2image4kpu to: " + str(size2queue4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                kpu2image4xi ['SIZE2QUEUE'] = size2queue4kpu
                message2print = "{(Step 1N2/33) set" + "; size2queue4image-got-by-wait2image4kpu to: " + str(kpu2image4xi ['SIZE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                  
                # 
            except:     
                message2print = "{(Step 1Z7/70) yielded" + "; size2queue4kpu-yielded-by-wait2image4kpu: " + str(size2queue4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            empty2queue4kpu = False
            try:                    
                empty2queue4kpu = queue2image4kpu.empty()
                message2print = "{(Step 1O1/33) got" + "; empty2queue4kpu-got-by-wait2image4kpu to: " + str(empty2queue4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                
                kpu2image4xi ['EMPTY2QUEUE'] = empty2queue4kpu
                message2print = "{(Step 1O2/33) set" + "; empty2queue4image-got-by-wait2image4kpu to: " + str(kpu2image4xi ['EMPTY2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #               
            except:     
                message2print = "{(Step 1Z8/70) yielded" + "; empty2queue4kpu-yielded-by-wait2image4kpu: " + str(empty2queue4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            full2queue4kpu = False
            try:                                
                full2queue4kpu = queue2image4kpu.full()
                message2print = "{(Step 1P1/33) got" + "; full2queue4kpu-got-by-wait2image4kpu to: " + str(full2queue4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                kpu2image4xi ['FULL2QUEUE'] = full2queue4kpu
                message2print = "{(Step 1P2/33) set" + "; full2queue4image-got-by-wait2image4kpu to: " + str(kpu2image4xi ['FULL2QUEUE'])  + "; time: " + time4xi() + "}" 
                print4xi(class2print, method2print, message2print)
                #                          
            except:     
                message2print = "{(Step 1Z9/70) yielded" + "; full2queue4kpu-yielded-by-wait2image4kpu: " + str(full2queue4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
            
            state2queue4kpu = None
            try:                                
                state2queue4kpu = queue2image4kpu.__getstate__()
                message2print = "{(Step 1P3/33) got" + "; state2queue4kpu-got-by-wait2image4kpu to: " + str(state2queue4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                # 
                 
                kpu2image4xi ['STATE2QUEUE'] = state2queue4kpu
                message2print = "{(Step 1P4/33) set" + "; state2queue4image-set-by-wait2image4kpu to: " + str(kpu2image4xi ['STATE2QUEUE'])  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #           
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; state2queue4kpu-yielded-by-wait2image4kpu: " + str(state2queue4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)               
                
                                                            
            message2print = "{(Step 1Q/33) finished" + "; kpu2image4xi-finished-by-wait2image4kpu: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            # 
    except:
        try:
            # -------------------------------------------------------------------------------------
            # INITIALIZE           
            # -------------------------------------------------------------------------------------           
            kpu2image4kpu = create2image4kpu()
            message2print = "{(Step 1R/33) got" + "; kpu2image4xi-got-by-wait2image4kpu to: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)
            #  
            
            # -------------------------------------------------------------------------------------
            # WAIT         
            # -------------------------------------------------------------------------------------             
            queue2image4kpu = None
            try:
                queue2image4kpu = kpu2image4xi['QUEUE']
                message2print = "{(Step 1S/33) got" + "; queue2image4kpu-got-by-wait2image4kpu to: " + str(queue2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                #
            except:     
                message2print = "{(Step 1Z10/70) yielded" + "; queue2image4kpu-yielded-by-wait2image4kpu: " + str(queue2image4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)
                
            if task4wait2image is None:
                pass
            else:
                # -------------------------------------------------------------------------------------
                # IMAGE QUEUE GIVEN AS PARAMETER          
                # -------------------------------------------------------------------------------------               
                queue2task = None    
                try:
                    queue2task = task4wait2image ['QUEUE']
                    message2print = "{(Step 1T/33) passed" + "; queue2task-passed-by-wait2image4kpu to: " + str(queue2task)  + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print)
                    #
                    
                    if queue2task == queue2image4kpu:
                        # -------------------------------------------------------------------------------------
                        # ACCEPTED - GIVEN QUEUE IS THE SAME          
                        # -------------------------------------------------------------------------------------                     
                        message2print = "{(Step 1U/33) accepted" + "; queue2task-accepted-by-wait2image4kpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print)
                        # 
                    else:
                        message2print = "{(Step 1X/33) warned" + "; queue2task-warned-by-wait2image4kpu: " + " ** " + str(queue2task)  + " ** " + "; time: " + time4xi() + "}"
                        print4xi(class2print, method2print, message2print) 
                        # 
                        
                        message2print = "{(Step 1Y1/33) warned" + "; queue2image4kpu-warned-by-wait2image4kpu: " + " ** " + str(queue2image4kpu)  + " ** " + "; time: " + time4xi() + "}" 
                        print4xi(class2print, method2print, message2print)    
                        #                      
                except:     
                    message2print = "{(Step 1Z11/70) yielded" + "; queue2task-yielded-by-wait2image4kpu: " + str(queue2task) + "; time: " + time4xi() + "}"
                    print4xi(class2print, method2print, message2print) 
            
            size2image4kpu = 0                             
            try:                    
                size2image4kpu = queue2image4kpu.qsize()
                message2print = "{(Step 1Y2/33) got" + "; size2image4kpu-got-by-wait2image4kpu to: " + str(size2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z12/70) yielded" + "; size2image4kpu-yielded-by-wait2image4kpu: " + str(size2image4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            empty2image4kpu = True          
            try:                    
                empty2image4kpu = queue2image4kpu.empty()
                message2print = "{(Step 1Y3/33) got" + "; empty2image4kpu-got-by-wait2image4kpu to: " + str(empty2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #
            except:     
                message2print = "{(Step 1Z13/70) yielded" + "; kpu2xi-yielded-by-wait2image4kpu: " + str(kpu2xi) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print) 
             
            full2image4kpu = False          
            try:                                
                full2image4kpu= queue2image4kpu.full()
                message2print = "{(Step 1Y4/33) got" + "; full2image4kpu-got-by-wait2image4kpu to: " + str(full2image4kpu)  + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)                
                #                
            except:     
                message2print = "{(Step 1Z14/70) yielded" + "; full2image4kpu-yielded-by-wait2image4kpu: " + str(full2image4kpu) + "; time: " + time4xi() + "}"
                print4xi(class2print, method2print, message2print)    
                                                        
            message2print = "{(Step 1Y5/33) finished" + "; kpu2image4xi-finished-by-wait2image4kpu: " + str(kpu2image4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)                                    
        except:
            kpu2image4kpu= {}
            message2print = "{(Step 1Z15/70) yielded" + "; kpu2image4xi-yielded-by-wait2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)  
                        
    message2print = "{(Step 33/33) returning" + "; kpu2image4xi-returning-by-wait2image4kpu: " + str(kpu2image4xi) + "; time: " + time4xi() + "}" 
    print4xi(class2print, method2print, message2print)
    # {'SIZE2QUEUE': 2147483646, 
    #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 
    #  'ORDER': 'CL_RGBA'}, 
    #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004420E10>, 
    #  'WIDTH': 800, 
    #  'CONTEXT': ...
    
    return kpu2image4xi
    
   
#</function4xi> 
                                               
# =================================================================================================
#<class4xi class = "KPU2Image4Xi" base = "object" child = "None"
#   synopsis = "offers methods for KPU 2D images" usage = "2D Image4Xi of KPU"
#>

# =================================================================================================
class KPU2Image4Xi(object): 
    ''' 
    KPU2Image4Xi offers methods for KPU 2D images
    '''            

    #                                        
    # ---------------------------------------------------------------------------------------------
    # sharable class variables
    # ---------------------------------------------------------------------------------------------      
    #<share4xi share = " 
    classvariable2kpu = cpu_corecount4xi()
    #"
    #/>
        
    # ---------------------------------------------------------------------------------------------
    # Public API of this class
    # ---------------------------------------------------------------------------------------------    
    #<expose4xi expose = "
    _exposed_ = (
            'create2image',                    # ACK 5.31   p76    
            'copy2image',                      # ACK 5.31   p88
            'copy2imagetobuffer',              # ACK 5.3.4  p90
            'copy2buffertoimage',              # ACK 5.3.4  p92
            'read2image',                      # ACK 5.3.3  p84
            'write2image',                     # ACK 5.3.3  p84            
            'map2image',                       # ACK 5.3.5  p94            
            'get2imageinfo',                   # ACK 5.3.6  p97
            'get2supportedimageformats',       # ACK 5.3.2  p82 nb: a separate method to get format whereas using get2imageinfo you can access any element by key such as 'FORMAT'   
            'retain2image',                    # ACK 5.4.1  p99 nb: for compability to OpenCL - you can get any element of dictionary by get2imageinfo
            'release2image',                   # ACK 5.4.1  p99
            'trace2image',                     # ACK modernized 
            'set2imagecallback',               # ACK modernized
            'set2image',                       # ACK modernized nb: with memorymap features you can process 2D images as files!                    
            'wait2image',                      # ACK modernized nb: checks Queue object and thereby generates delay   
            'sense',                           # ACK modernized  
            'reason',                          # ACK modernized  
            'ace')                             # ACK modernized           
    #"
    #/>
 
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "init-of-KPU2Image4Xi" usage = "
    #   print('(Step 5A/33) OF KPU2IMAGE4XI5TEST - CREATING IMAGE4KPU OBJECT ', time4xi())     
    #   try:
    #       imageobject2kpu = KPU2Image4Xi()
    #       print('(Step 5B/33) OF KPU2IMAGE4XI5TEST - CREATED IMAGE4KPU OBJECT: ', imageobject2kpu)                                  
    #   except:
    #       print('(Step 5Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO CREATE IMAGE4KPU OBJECT')
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def __init__(self, task4imagecreate2kpu = None):
        '''
        _init__ method initializes KPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------
                    
        class2print = "KPU2Image4Xi"
        method2print = "__init__"
                     
        message2print = "{(Step 1A/33) started" + "; init-of-KPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 23:41:37 09/06/11 Pacific Daylight Time 

        message2print = "{(Step 1B/33) borning" + "; kpu2image2xi-of-KPU2Image4Xi-borning: None" + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
                
        message2print = "{(Step 1C/33) inheriting" + "; object-inheriting-by-kpu2image2xi-of-KPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'>
        
        message2print = "{(Step 1D/33) sharing" + "; classvariable2kpu-sharing-by-kpu2image2xi-of-KPU2Image4Xi: " + str(KPU2Image4Xi.classvariable2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4             
                                
        message2print = "{(Step 1E/33) passed" + "; task4imagecreate2kpu-passed-to-init-of-KPU2Image4Xi: " + str(task4imagecreate2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None                            
        
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)             
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) passing" + "; task4imagecreate2kpu-passing-to-create2image4kpu-by-init-of-KPU2Image4Xi: " + str(task4imagecreate2kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None  
        
        kpu2image2xi = create2image4kpu(task4imagecreate2kpu)
        
        message2print = "{(Step 2B/33) performed" + "; create2image4kpu-performed-by-init-of-KPU2Image4Xi: " + str(create2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <function create2image4kpu at 0x0000000003A9E7C8>
                        
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)             
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 3A/33) activated" + "; kpu2image2xi-of-KPU2Image4Xi-activated: " + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # { KPU2Image4Xi : __init__ ; (Step 3A/33) activated; kpu2image2xi-of-KPU2Image4Xi-activated: ; time: 00:14:05 09/07/11 Pacific Daylight Time} 
                
        message2print = "{(Step 3B/33) inherited" + "; object-inherited-by-kpu2image2xi-of-KPU2Image4Xi: " + str(object)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # <class 'object'> 
        
        message2print = "{(Step 3C/33) shared" + "; classvariable2kpu-shared-by-kpu2image2xi-of-KPU2Image4Xi: " + str(KPU2Image4Xi.classvariable2kpu)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 4           
       
        message2print = "{(Step 3D/33) returning" + "; init-of-KPU2Image4Xi-returning: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # 00:14:05 09/07/11 Pacific Daylight Time
        

    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "create2image-of-KPU2Image4Xi" usage = "
    #   print('(Step 23A/33) OF KPU2IMAGE4XI5TEST - TRYING TO CREATE KPU IMAGE', time4xi()) 
    #   try:
    #       xi2kpu = KPU2Image4Xi()                            
    #       image2kpu = xi2kpu.create2image()                                  
    #       print('(Step 23B/33) OF KPU2IMAGE4XI5TEST - CREATED KPU IMAGE: ', image2kpu)
    #   except:
    #       print('(Step 23Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO CREATE KPU IMAGE ***', time4xi())                  
    #                     
    #">   
    
    # ---------------------------------------------------------------------------------------------   
    def create2image(self, task4image2create = None):
        '''
        create2image method creates 2D image objects of KPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "create2image"
                     
        message2print = "{(Step 1A/33) started" + "; create2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4image2create-passed-to-create-of-KPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                      
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4image2create-passing-to-create2image4kpu-by-create2image-of-KPU2Image4Xi: " + str(task4image2create) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)            
        # None      
  
        kpu2image2xi = create2image4kpu(task4image2create)

        message2print = "{(Step 2B/33) performed" + "; create2image4kpu-performed-by-create2image-of-KPU2Image4Xi: " + str(create2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function create2image4kpu at 0x00000000039DF7C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
        message2print = "{(Step 3/33) returning" + "; kpu2image2xi-returning-by-create2image-of-KPU2Image4Xi: " + str(kpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          # 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 00:53:55 10/05/11 Pacific Daylight Time}',
        #  'CLASS': 'create2image4kpu',
        #  'METHOD': 'create2image4kpu'},
        #  'WIDTH': 800 ...

        return kpu2image2xi
                                
    #                        
    #</methodize4xi>
    

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2image-of-KPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF KPU2IMAGE4XI - TRYING TO SET KPU IMAGE')
    #   try:
    #       cc2xi = KPU2Image4Xi()                            
    #       image2kpu = cc2xi.set2image()                                  
    #       print('(Step 1B/33) OF KPU4XI5TEST - SET KPU IMAGE: ', image2kpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF KPU4XI5TEST *** FAILED TO SET KPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2image(self, task4set2image = None):
        '''
        sets 2D image KPU objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "set2image"
                     
        message2print = "{(Step 1A/33) started" + "; set2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4set2image-passed-to-set2image-of-KPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'CODE2ERROR': 0,
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2image-passing-to-set2image4kpu-by-set2image-of-KPU2Image4Xi: " + str(task4set2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'CODE2ERROR': 0, 
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004191E10>, 
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...

        kpu2image2xi = set2image4kpu(task4set2image)

        message2print = "{(Step 2B/33) performed" + "; set2image4kpu-performed-by-set2image-of-KPU2Image4Xi: " + str(set2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function set2image4kpu at 0x0000000003AB36C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; kpu2image2xi-returning-by-set2image-of-KPU2Image4Xi: " + str(kpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 23B-24B-25B-26B-27B-28B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4kpu', 'METHOD': 'create2image4kpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 01:02:22 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4kpu',
        #  'METHOD': 'create2buffer4kpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x0000000003AA81C8>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x0000000003CC92F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                    
                
        return kpu2image2xi
                                
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2imageinfo-of-KPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF KPU2IMAGE4XI - TRYING TO GET KPU 2D IMAGE')
    #   try:
    #       cc2xi = KPU2Image4Xi()                            
    #       image2kpu = cc2xi.get2imageinfo()                                  
    #       print('(Step 1B/33) OF KPU4XI5TEST - GOT KPU IMAGE: ', image2kpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF KPU4XI5TEST *** FAILED TO GET KPU 2D IMAGE ***')   
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2imageinfo(self, task4getinfo2image=None):
        '''
        get2imageinfo method gets 2D objects of KPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "get2imageinfo"
                     
        message2print = "{(Step 1A/33) started" + "; get2imageinfo-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        kpu2image2xi = get2imageinfo4kpu(task4getinfo2image)
        
        message2print = "{(Step 2A/33) returning" + "; kpu2image2xi-returning-by-get2imageinfo-of-KPU2Image4Xi: " + str(kpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0,
        #  'ACK2RETURN': ' 1D-1E-2C-2B-3B-4B-5B-6B-7B-8B-9B-10B-11B-14B-15B-16B-17B-17D-18B-18C-21A1-22B1-22C1-23B-24B-25B-26B-27B-28B-29B-30B-31B-34H-35H-36B-37B-38B-',
        #  'ROWPITCH': 800,
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 'CLASS': 'create2image4kpu', 'METHOD': 'create2image4kpu'},
        #  'WIDTH': 800,
        #  'BUFFER': {'CODE2ERROR': 0, 'COUNT2REFERENCE': 0, 'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2buffer4kpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}',
        #  'CLASS': 'create2buffer4kpu',
        #  'METHOD': 'create2buffer4kpu'},
        #  'FUNCTION2CALLBACK': <function trace2buffer4kpu at 0x00000000039D8348>,
        #  'POINTER2BUFFER': <mmap.mmap object at 0x00000000037D32F0>,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0, ...
        # or when only two elemets required
        # {'WIDTH': 800, 'HEIGHT': 800}
        
        return kpu2image2xi
                            
    #</methodize4xi>

    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "get2supportedimageformats-of-KPU2Image4Xi" usage = "
    #  print('(Step 6B1/33) OF KPU2IMAGE4XI5TEST - TRYING TO GET SUPPORTED IMAGE FORMATS FOR KPU')
    #  try:
    #      cc2xi = KPU2Image4Xi()                            
    #      supportedimageformats2kpu = cc2xi.get2supportedimageformats()                                  
    #      print('(Step 6B2/33) OF KPU4XI5TEST - GOT SUPPORTED IMAGE FORMATS FOR KPU: ', supportedimageformats2kpu)
    #                                                   
    #  except:
    #      print('(Step 2Z/33) OF KPU4XI5TEST *** FAILED TO GET SUPPORTED IMAGE FORMATS FOR KPU***')     
    # ">
    
    # ---------------------------------------------------------------------------------------------   
    def get2supportedimageformats(self):
        '''
        get2supportedimageformats method gets supported imageformats by objects of KPU2Image4Xi
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "get2supportedimageformats"
                     
        message2print = "{(Step 1A/33) started" + "; get2supportedimageformats-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:43:00 09/07/11 Pacific Daylight Time

        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # ----------------------------------------------------------------------------------------- 
        supportedimageformats2kpu = get2supportedimageformats4kpu()
        
        message2print = "{(Step 2A/33) returning" + "; supportedimageformats2kpu-returning-by-get2imageinfo-of-KPU2Image4Xi: " + str(supportedimageformats2kpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'} 
        
        return supportedimageformats2kpu
                            
    #</methodize4xi>    
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "read2image-of-KPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi()) 
    #   response4read2kpu = {}
    #   try:
    #       xi2kpu = KPU2Image4Xi()    
    #       response4read2kpu = xi2kpu.read2image()
    #       print('(Step 127B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH READ IMAGE OBJECT, RESPONSE4READ2KPU: ', response4read2kpu) 
    #   except:
    #       print('(Step 127Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH READ IMAGE OBJECT ', time4xi())
    #   ... continue as with read2image4kpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def read2image(self, task4read2image = None):
        '''
        reads KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "read2image"
                     
        message2print = "{(Step 1A/33) started" + "; read2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:20:02 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4read2image-passed-to-read2image-of-KPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4read2image-passing-to-read2image4kpu-by-read2image-of-KPU2Image4Xi: " + str(task4read2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4readimage2kpu = read2image4kpu(task4read2image)

        message2print = "{(Step 2B/33) performed" + "; read2image4kpu-performed-by-read2image-of-KPU2Image4Xi: " + str(read2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function read2image4kpu at 0x000000000385DA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4readimage2kpu-returning-by-read2image-of-KPU2Image4Xi: " + str(response4readimage2kpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000417DD68>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 936>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 932>}; time: 00:20:02 10/05/11 Pacific Daylight Time}
  
        return response4readimage2kpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "map2image-of-KPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi()) 
    #   xi2kpu = KPU2Image4Xi() 
    #   ...
    #   response4map2kpu = {}
    #   try:
    #       response4map2kpu = xi2kpu.map2image()
    #       print('(Step 127B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH MAP IMAGE OBJECT, RESPONSE4READ2KPU: ', response4map2kpu) 
    #   except:
    #       print('(Step 127Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH MAP IMAGE OBJECT ', time4xi())
    #   ... continue as with map2image4kpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def map2image(self, task4map2image = None):
        '''
        maps KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "map2image"
                     
        message2print = "{(Step 1A/33) started" + "; map2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4map2image-passed-to-map2image-of-KPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4map2image-passing-to-map2image4kpu-by-map2image-of-KPU2Image4Xi: " + str(task4map2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4mapimage2kpu = map2image4kpu(task4map2image)

        message2print = "{(Step 2B/33) performed" + "; map2image4kpu-performed-by-map2image-of-KPU2Image4Xi: " + str(map2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function map2image4kpu at 0x000000000390BAC8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4mapimage2kpu-returning-by-map2image-of-KPU2Image4Xi: " + str(response4mapimage2kpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x00000000050E9C18>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1100>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1096>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

        
        return response4mapimage2kpu
                                                           
    #</methodize4xi>
            
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "release2image-of-KPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RELEASE 2D IMAGE BY DECREMENTING REFERENCE COUNT WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4release2image  = {} 
    #   task4release2image  ['COUNT2REFERENCE'] = 1 
    #   response4release2image = {}
    #   print('(Step 133A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RELEASE 2D IMAGE OF KPU DEVICE ', time4xi())    
    # 
    #   try:                               
    #       response4release2Dimage = xi2kpu.release2image(task4release2image)                                                                                             
    #       print('(Step 133B/700) OF KPU2IMAGE4XI5TEST - RELEASED 2D IMAGE OF KPU DEVICE ', time4xi())                           
    #       keys2response = response4release2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4release2Dimage[key2response])
    #           #
    #   except:
    #       print('(Step 133Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RELEASE IMAGE OF KPU DEVICE ', time4xi())  
    #   
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 133C/700) OF KPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF KPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2kpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 133D/700) OF KPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF KPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 133Z2/700) OF KPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF KPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def release2image(self, task4release2image = None):
        '''
        releases KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "release2image"
                     
        message2print = "{(Step 1A/33) started" + "; release2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 
            
        message2print = "{(Step 1B/33) passed" + "; task4release2image-passed-to-release2image-of-KPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4release2image-passing-to-release2image4kpu-by-release2image-of-KPU2Image4Xi: " + str(task4release2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # 
  
        kpu2image2xi = release2image4kpu(task4release2image)

        message2print = "{(Step 2B/33) performed" + "; release2image4kpu-performed-by-release2image-of-KPU2Image4Xi: " + str(release2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; kpu2image2xi-returning-by-release2image-of-KPU2Image4Xi: " + str(kpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        #                          
                
        return kpu2image2xi
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "retain2image-of-KPU2Image4Xi" usage = "
    #
    # ---------------------------------------------------------------------------------------------
    # RETAIN 2D IMAGE BY DECREASING REFERENCE WITH 1             
    # ---------------------------------------------------------------------------------------------    
    #   task4retain2image  = {}
    #   task4retain2image  ['COUNT2REFERENCE'] = 1 
    #   response4retain2Dimage = {}
    #   print('(Step 134A/700) OF KPU2IMAGE4XI5TEST - TRYING TO RETAIN 2D IMAGE OF KPU DEVICE ', time4xi())   
    #   try:                               
    #       response4retain2Dimage = xi2kpu.retain2image(task4retain2image)                                                                                                 
    #       print('(Step 134B/700) OF KPU2IMAGE4XI5TEST - RESPONSE4RETAIN2DIMAGE : ', time4xi())                         
    #       keys2response = response4retain2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4retain2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO RETAIN 2D IMAGE OF KPU DEVICE ', time4xi())
    #   task4getinfo2image = {}
    #   task4getinfo2image ['COUNT2REFERENCE'] = 0
    #   response4getinfo2image = {}
    #   print('(Step 134C/700) OF KPU2IMAGE4XI5TEST - TRYING TO GET SELECTED 2D IMAGE INFO OF KPU DEVICE ', time4xi())     
    #   # 
    #   try:                               
    #       response4getinfo2Dimage = xi2kpu.get2imageinfo (task4getinfo2image)                                                                                             
    #       keys2response = response4getinfo2Dimage.keys()
    #       print('(Step 134D/700) OF KPU2IMAGE4XI5TEST - GOT SELECTED 2D IMAGE INFO OF KPU DEVICE ', time4xi())   
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4getinfo2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 134Z2/700) OF KPU2IMAGE4XI5TEST *** FAILED TO GET SELECTED 2D IMAGE INFO OF KPU DEVICE ', time4xi())  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def retain2image(self, task4retain2image = None):
        '''
        retains KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "retain2image"
                     
        message2print = "{(Step 1A/33) started" + "; retain2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4retain-passed-to-retain2image-of-KPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # {'COUNT2REFERENCE': 1}
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4retain-passing-to-retain2image4kpu-by-retain-of-KPU2Image4Xi: " + str(task4retain2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # {'COUNT2REFERENCE': 1}
  
        kpu2image2xi = retain2image4kpu(task4retain2image)

        message2print = "{(Step 2B/33) performed" + "; retain2image4kpu-performed-by-retain2image-of-KPU2Image4Xi: " + str(retain2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function retain2image4kpu at 0x00000000039DF948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; kpu2image2xi-returning-by-retain2image-of-KPU2Image4Xi: " + str(kpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'CODE2ERROR': 0, 
        #  'ACK2RETURN': ' 1D-', 
        #  'ROWPITCH': 800, 
        #  'DATA2CALLBACK': {'MESSAGE': '{(Step 1Z1/33) failed; create2image4kpu-failed ; time: 00:14:05 09/07/11 Pacific Daylight Time}', 
        #  'CLASS': 'create2image4kpu', 
        #  'METHOD': 'create2image4kpu'}, 
        #  'WIDTH': 800, 
        #  'BUFFER': {'CODE2ERROR': 0, 
        #  ...
        #  'COUNT2REFERENCE': 1,
        #  'FLAGS': 0,
        #  'CONTEXT': {'COUNT2REFERENCE': 0 ...
                         
                
        return kpu2image2xi
                                                           
    #</methodize4xi>   
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "wait2image-of-KPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 126A/700) OF KPU2IMAGE4XI5TEST - TRYING TO WAIT 2D IMAGE OPERATION FOR KPU ', time4xi()) 
    #   print('=' *33)      
    #   wait2kpu = {}
    #   try:
    #       response4wait2Dimage = xi2kpu.wait2image()
    #       print('(Step 122B/700) OF KPU2IMAGE4XI5TEST - WAITED 2D IMAGE OPERATION FOR KPU ', time4xi()) 
    #       keys2response = response4wait2Dimage.keys()
    #       for key2response in sorted(keys2response):
    #           print(key2response, ' : ', response4wait2Dimage[key2response])
    #           # 
    #   except:
    #       print('(Step 126Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO 2D IMAGE OPERATION FOR KPU ', time4xi())    
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def wait2image(self, task4wait2image = None):
        '''
        waits KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "wait2image"
                     
        message2print = "{(Step 1A/33) started" + "; wait2image-of-KPU2Image4Xi-started: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4wait2image-passed-to-wait2image-of-KPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4wait-passing-to-wait2image4kpu-by-wait2image-of-KPU2Image4Xi: " + str(task4wait2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        kpu2image2xi = wait2image4kpu(task4wait2image)

        message2print = "{(Step 2B/33) performed" + "; wait2image4kpu-performed-by-wait2image-of-KPU2Image4Xi: " + str(wait2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # semaphore and lock analysis by multiprocessing4xi
        # <function wait2image4kpu at 0x00000000039DFA48> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; kpu2image2xi-returning-by-wait2image-of-KPU2Image4Xi: " + str(kpu2image2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'SIZE2QUEUE': 2147483646,
        #  'FORMAT': {'TYPE': 'CL_UNORM_INT8', 'ORDER': 'CL_RGBA'},
        #  'QUEUE': <queue4xi.Queue4Xi object at 0x000000000434DDA0>,
        #  'WIDTH': 800,
        #  'CONTEXT': ...
                
        return kpu2image2xi
                                                           
    #</methodize4xi> 
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "write2image-of-KPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4write2kpu = {}
    #   try:
    #       response4write2kpu = xi2kpu.write2image()
    #       print('(Step 127B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2KPU: ', response4write2kpu) 
    #   except:
    #       print('(Step 127Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with write2image4kpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def write2image(self, task4write2image = None):
        '''
        writes KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "write2image"
                     
        message2print = "{(Step 1A/33) started" + "; write2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 02:45:31 10/05/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4write2image-passed-to-write2image-of-KPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4write2image-passing-to-write2image4kpu-by-write2image-of-KPU2Image4Xi: " + str(task4write2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4writeimage2kpu = write2image4kpu(task4write2image)

        message2print = "{(Step 2B/33) performed" + "; write2image4kpu-performed-by-write2image-of-KPU2Image4Xi: " + str(write2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function write2image4kpu at 0x000000000380AEC8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4writeimage2kpu-returning-by-write2image-of-KPU2Image4Xi: " + str(response4writeimage2kpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004D10E10>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 968>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 964>}; time: 02:45:31 10/05/11 Pacific Daylight Time}

                     
                
        return response4writeimage2kpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2image-of-KPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #   response4copy2kpu = {}
    #   try:
    #       response4copy2kpu = xi2kpu.copy2image()
    #       print('(Step 127B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH WRITE IMAGE OBJECT, RESPONSE4WRITE2KPU: ', response4copy2kpu) 
    #   except:
    #       print('(Step 127Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH WRITE IMAGE OBJECT', time4xi()) 
    #
    #   ... continue as with copy2image4kpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2image(self, task4copy2image = None):
        '''
        copies KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "copy2image"
                     
        message2print = "{(Step 1A/33) started" + "; copy2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2image-passed-to-copy2image-of-KPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None 
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2image-passing-to-copy2image4kpu-by-copy2image-of-KPU2Image4Xi: " + str(task4copy2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None 
  
        response4copyimage2kpu = copy2image4kpu(task4copy2image)

        message2print = "{(Step 2B/33) performed" + "; copy2image4kpu-performed-by-copy2image-of-KPU2Image4Xi: " + str(copy2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2image4kpu at 0x000000000390B8C8>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2kpu-returning-by-copy2image-of-KPU2Image4Xi: " + str(response4copyimage2kpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004EC4128>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1000>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 996>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
        
        return response4copyimage2kpu
                                                           
    #</methodize4xi>
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2imagetobuffer-of-KPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER OBJECT', time4xi()) 
    #   response4copyimagetobuffer = {}
    #   try:
    #       response4copyimagetobuffer = xi2kpu.copy2imagetobuffer()
    #       print('(Step 127B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER  OBJECT, RESPONSE4WRITE2KPU: ', response4copyimagetobuffer) 
    #   except:
    #       print('(Step 127Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY IMAGE TO BUFFER ', time4xi()) 
    #
    #   ... continue as with copy2imagetobuffer4kpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2imagetobuffer(self, task4copy2imagetobuffer = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "copy2imagetobuffer"
                     
        message2print = "{(Step 1A/33) started" + "; copy2imagetobuffer-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2imagetobuffer-passed-to-copy2imagetobuffer-of-KPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2imagetobuffer-passing-to-copy2imagetobuffer4kpu-by-copy2imagetobuffer-of-KPU2Image4Xi: " + str(task4copy2imagetobuffer) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2kpu = copy2imagetobuffer4kpu(task4copy2imagetobuffer)

        message2print = "{(Step 2B/33) performed" + "; copy2imagetobuffer4kpu-performed-by-copy2imagetobuffer-of-KPU2Image4Xi: " + str(copy2imagetobuffer4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2imagetobuffer4kpu at 0x000000000390B948>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2kpu-returning-by-copy2imagetobuffer-of-KPU2Image4Xi: " + str(response4copyimage2kpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x0000000004FE5FD0>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1036>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1032>}; time: 01:28:44 10/05/11 Pacific Daylight Time}
          
        return response4copyimage2kpu
                                                           
    #</methodize4xi>
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "copy2buffertoimage-of-KPU2Image4Xi" usage = "
    #   print('(Step 127A/700) OF KPU2IMAGE4XI5TEST - TRYING TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE OBJECT', time4xi()) 
    #   response4copybuffertoimage = {}
    #   try:
    #       response4copybuffertoimage = xi2kpu.copy2buffertoimage()
    #       print('(Step 127B/700) OF KPU2IMAGE4XI5TEST - ESTABLISHED CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE  OBJECT, RESPONSE4WRITE2KPU: ', response4copybuffertoimage) 
    #   except:
    #       print('(Step 127Z1/700) OF KPU2IMAGE4XI5TEST *** FAILED TO ESTABLISH CONTROL AND COMMAND WITH COPY BUFFER TO IMAGE ', time4xi()) 
    #
    #   ... continue as with copy2buffertoimage4kpu function
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def copy2buffertoimage(self, task4copy2buffertoimage = None):
        '''
        copies 2D images to buffer
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "copy2buffertoimage"
                     
        message2print = "{(Step 1A/33) started" + "; copy2buffertoimage-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:28:44 10/05/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4copy2buffertoimage-passed-to-copy2buffertoimage-of-KPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4copy2buffertoimage-passing-to-copy2buffertoimage4kpu-by-copy2buffertoimage-of-KPU2Image4Xi: " + str(task4copy2buffertoimage) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
  
        response4copyimage2kpu = copy2buffertoimage4kpu(task4copy2buffertoimage)

        message2print = "{(Step 2B/33) performed" + "; copy2buffertoimage4kpu-performed-by-copy2buffertoimage-of-KPU2Image4Xi: " + str(copy2buffertoimage4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function copy2buffertoimage4kpu at 0x000000000390B9C8> 
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; response4copyimage2kpu-returning-by-copy2buffertoimage-of-KPU2Image4Xi: " + str(response4copyimage2kpu) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'QUEUE': <queue4xi.Queue4Xi object at 0x000000000505A6D8>,
        #  'PUBLISHERCHANNEL': <read-write PipeConnection, handle 1068>,
        #  'CONSUMERCHANNEL': <read-write PipeConnection, handle 1064>}; time: 01:28:44 10/05/11 Pacific Daylight Time}

                
        return response4copyimage2kpu
                                                           
    #</methodize4xi>    
                    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "trace2image-of-KPU2Image4Xi" usage = "
    #   print('=' *33)          
    #   print('(Step 15A/25) OF KPU2IMAGE4XI5TEST - TRYING TO TRACE KPU: ', time4xi())      
    #   print('=' *33)      
    #   trace2kpu = None
    #   try:
    #       trace2kpu = trace2image4kpu() 
    #       print('(Step 15B/25) OF KPU2IMAGE4XI5TEST - TRACE2KPU: ', trace2kpu)  
    #       # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 01:39:15 10/06/11 Pacific Daylight Time}', 'CLASS': 'trace2image4kpu', 'METHOD': 'trace2image4kpu'}
    #    except:
    #       print('(Step 15Z1/25) OF KPU2IMAGE4XI5TEST *** FAILED TO TRACE KPU: ', trace2kpu)     
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def trace2image(self, task4trace2image = None):
        '''
        traces KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "trace2image"
                     
        message2print = "{(Step 1A/33) started" + "; trace2image-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time 
            
        message2print = "{(Step 1B/33) passed" + "; task4trace2image-passed-to-trace-of-KPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None   
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4trace2image-passing-to-trace2image4kpu-by-trace2image-of-KPU2Image4Xi: " + str(task4trace2image) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None  
  
        trace2xi = trace2image4kpu(task4trace2image)

        message2print = "{(Step 2B/33) performed" + "; trace2image4kpu-performed-by-trace2image-of-KPU2Image4Xi: " + str(trace2image4kpu) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)             
        # <function trace2image4kpu at 0x00000000039DF348>
                                
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; trace2xi-returning-by-trace2image-of-KPU2Image4Xi: " + str(trace2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # {'MESSAGE': '{(Step 2Z0/33) failed; trace2image4kpu-failed {}; time: 01:05:20 09/08/11 Pacific Daylight Time}',
        #  'CLASS': 'trace2image4kpu',
        #  'METHOD': 'trace2image4kpu'}; time: 01:05:20 09/08/11 Pacific Daylight Time}
                 
                
        return trace2xi
                                                           
    #</methodize4xi>       


    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "set2imagecallback-of-KPU2Image4Xi" usage = "
    #   print('(Step 1A/33) OF KPU2IMAGE4XI - TRYING TO SET KPU IMAGE')
    #   try:
    #       cc2xi = KPU2Image4Xi()                            
    #       image2kpu = cc2xi.set2imagecallback()                                  
    #       print('(Step 1B/33) OF KPU4XI5TEST - SET KPU IMAGE: ', image2kpu)                                                  
    #   except:
    #       print('(Step 1Z/33) OF KPU4XI5TEST *** FAILED TO SET KPU IMAGE ***')  
    #">
    
    # ---------------------------------------------------------------------------------------------   
    def set2imagecallback(self, task4set2imagecallback = None): 
        '''
        set2imagecallback of KPU2Image4Xi objects
        ''' 
        
        # -----------------------------------------------------------------------------------------
        # (Step 1/33)           
        # -----------------------------------------------------------------------------------------              
        class2print = "KPU2Image4Xi"
        method2print = "set2imagecallback"
                     
        message2print = "{(Step 1A/33) started" + "; set2imagecallback-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 00:14:05 09/07/11 Pacific Daylight Time
            
        message2print = "{(Step 1B/33) passed" + "; task4set2imagecallback-passed-to-set2imagecallback-of-KPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                          
        # None    
          
        # -----------------------------------------------------------------------------------------
        # (Step 2/33)           
        # -----------------------------------------------------------------------------------------
        message2print = "{(Step 2A/33) passing" + "; task4set2imagecallback-passing-to-set2imagecallback -by-set2imagecallback-of-KPU2Image4Xi: " + str(task4set2imagecallback) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)                                                               
        # None
        
        callback2kpu4image = {} 
        try:
            callback2kpu4image = set2imagecallback4kpu (task4set2imagecallback)

            message2print = "{(Step 2B/33) callback2kpu4image" + "; callback2kpu4image-set-by-set2imagecallback-of-KPU2Image4Xi to: " + str(callback2kpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print)             
            # <function trace2image4kpu at 0x00000000039DF348>
        except:
            message2print = "{(Step 4A/33) failed" + "; callback2kpu4image-failed-by-set2imagecallback-of-KPU2Image4Xi: " + str(callback2kpu4image) + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
                                            
        # -----------------------------------------------------------------------------------------
        # (Step 3/33)           
        # ----------------------------------------------------------------------------------------- 
       
        message2print = "{(Step 3A/33) returning" + "; callback2kpu4image-returning-by-set2imagecallback-of-KPU2Image4Xi: " + str(callback2kpu4image) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print) 
        # <function trace2image4kpu at 0x0000000003AB31C8> 
                                             
        return callback2kpu4image
                                                           
    #</methodize4xi>     
    
                                                              
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "sense-of-KPU2Image4Xi" usage = "
    #   try:
    #       print(xi2kpu.sense())
    #       print('(Step 24B/33) OF KPU2IMAGE4XI5TEST - SENSED ', time4xi()) 
    #   except:
    #       print('(Step 24Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO SENSE ', time4xi())  
    # ">
    
    # ---------------------------------------------------------------------------------------------
    def sense(self):
        '''
        method sense: senses KPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'KPU2Image4Xi'
        method2print = 'sense'
        
                     
        message2print = "{(Step 1A/33) started" + "; sense-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------  
        
        message2print = "{(Step 2A/33) sensing" + "; KPU2Image4Xi-sensing-by-sense-of-KPU2Image4Xi: " + str(KPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'kpu2image4xi.KPU2Image4Xi'>         
        
        # -----------------------------------------------------------------------------------------
        # get sensed knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb2xi = sense_object4xi(self)
        
        message2print = "{(Step 3A/33) got" + "; kb2xi-got-by-sense-of-KPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object': <kpu2image4xi.KPU2Image4Xi object at 0x0000000004188400>}   
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb2xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------         
        try:
            kb2xi['classvariable2kpu-of-KPU2Image4Xi'] = KPU2Image4Xi.classvariable2kpu
        except:
            message2print = "{(Step 4A/33) yielded" + "; kb2xi-yielded-by-sense-of-KPU2Image4Xi: " + str(kb2xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
            
      
        message2print = "{(Step 5A/33) sensed" + "; KPU2Image4Xi-sensed-by-sense-of-KPU2Image4Xi: " + str(KPU2Image4Xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)    
        # <class 'kpu2image4xi.KPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        # 
        # -----------------------------------------------------------------------------------------         
        
        message2print = "{(Step 6A/33) returning" + "; kb2xi-returning-by-sense-of-KPU2Image4Xi: " + str(kb2xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object': <kpu2image4xi.KPU2Image4Xi object at 0x0000000004188400>, 'classvariable2kpu-of-KPU2Image4Xi': 4}             
          
        return kb2xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
    #</methodize4xi>
    
        
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "reason-of-KPU2Image4Xi" usage = "
    #   print('(Step 25A/33) OF KPU2IMAGE4XI5TEST - REASONING ', time4xi())
    #   try:
    #       print(xi2kpu.reason())
    #       print('(Step 25B/33) OF KPU2IMAGE4XI5TEST - REASONED ', time4xi())
    #   except:
    #       print('(Step 25Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO REASON ', time4xi())   
    #">
    
    # ---------------------------------------------------------------------------------------------
    def reason(self):
        '''
        method reason: reasons KPU2Image4Xi class objects
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # ----------------------------------------------------------------------------------------- 
        class2print = 'KPU2Image4Xi'
        method2print = 'reason'

                     
        message2print = "{(Step 1A/33) started" + "; reason-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time 
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) reasoning" + "; KPU2Image4Xi-reasoning-by-reason-of-KPU2Image4Xi: " + str(KPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print) 
        # <class 'kpu2image4xi.KPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        # get reasoned knowledge for this object
        # -----------------------------------------------------------------------------------------         
        
        kb4xi = reason_object4xi(self)
        
        
        message2print = "{(Step 3A/33) got" + "; kb4xi-got-by-reason-of-KPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object-reason-object4xi': <kpu2image4xi.KPU2Image4Xi object at 0x0000000004188400>}      
        
        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb4xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------          
        try:
            kb4xi['time-reason-of-KPU2Image4Xi'] = time4xi() #adding time
        except:
            message2print = "{(Step 4Z/33) yielded" + "; kb4xi-yielded-by-reason-of-KPU2Image4Xi: " + str(kb4xi)  + "; time: " + time4xi() + "}"
            print4xi(class2print, method2print, message2print) 
        
          
        
        message2print = "{(Step 5A/33) reasoned" + "; KPU2Image4Xi-reasoned-by-reason-of-KPU2Image4Xi: " + str(KPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)        
        # <class 'kpu2image4xi.KPU2Image4Xi'>
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        
        message2print = "{(Step 6A/33) returning" + "; kb4xi-returning-by-reason-of-KPU2Image4Xi: " + str(kb4xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'object-reason-object4xi': <kpu2image4xi.KPU2Image4Xi object at 0x0000000004188400>, 'time-reason-of-KPU2Image4Xi': '01:02:22 09/07/11 Pacific Daylight Time'}                
          
        return kb4xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
    
    
    # ---------------------------------------------------------------------------------------------
    #<methodize4xi method = "ace-of-KPU2Image4Xi" usage = "
    #   print('(Step 26A/33) OF KPU2IMAGE4XI5TEST - ACING ', time4xi())   
    #   try:
    #       print(xi2kpu.ace())
    #       print('(Step 26B/33) OF KPU2IMAGE4XI5TEST - ACED ', time4xi())                                                 
    #   except:
    #       print('(Step 26Z/33) OF KPU2IMAGE4XI5TEST *** FAILED TO ACE ', time4xi())                                       
    #    
    #">
    
    # ---------------------------------------------------------------------------------------------
    def ace(self):
        '''
        method ace: aces KPU2Image4Xi class objects 
        '''
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------       
        
        
        class2print = 'KPU2Image4Xi'
        method2print = 'ace'
        
                     
        message2print = "{(Step 1A/33) started" + "; ace-of-KPU2Image4Xi-started: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)   
        # 01:02:22 09/07/11 Pacific Daylight Time   
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------
        
        message2print = "{(Step 2A/33) acing" + "; KPU2Image4Xi-acing-by-ace-of-KPU2Image4Xi: " + str(KPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'kpu2image4xi.KPU2Image4Xi'>           
        
        # -----------------------------------------------------------------------------------------
        # get aced knowledge for this object
        # -----------------------------------------------------------------------------------------        
        
        kb8xi = ace_object4xi(self)  
        
        message2print = "{(Step 3A/33) got" + "; kb8xi-got-by-ace-of-KPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)
        # {'object8xi': <kpu2image4xi.KPU2Image4Xi object at 0x0000000004188400>}   


        # -----------------------------------------------------------------------------------------
        # add knowledge specific to this object
        # format example: kb8xi['time2xi'] = time4xi() #adding time 
        # -----------------------------------------------------------------------------------------              
        try:
             kb8xi['cores-of-KPU2Image4Xi'] = KPU2Image4Xi.classvariable2kpu
        except:
             message2print = "{(Step 4A/33) yielded" + "; kb8xi-yielded-by-ace-of-KPU2Image4Xi: " + str(kb8xi)  + "; time: " + time4xi() + "}"
             print4xi(class2print, method2print, message2print) 
  
        message2print = "{(Step 5A/33) aced" + "; KPU2Image4Xi-aced-by-ace-of-KPU2Image4Xi: " + str(KPU2Image4Xi) + "; time: " + time4xi() + "}"
        print4xi(class2print, method2print, message2print)         
        # <class 'kpu2image4xi.KPU2Image4Xi'>  
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------        
        message2print = "{(Step 6A/33) returning" + "; kb8xi-returning-by-ace-of-KPU2Image4Xi: " + str(kb8xi) + "; time: " + time4xi() + "}" 
        print4xi(class2print, method2print, message2print)                                                          
        # {'cores-of-KPU2Image4Xi': 4, 'object8xi': <kpu2image4xi.KPU2Image4Xi object at 0x0000000004188400>}
        
        return kb8xi
        
        # -----------------------------------------------------------------------------------------
        #
        # -----------------------------------------------------------------------------------------            
    #</methodize4xi>
#</class4xi>
#</program4xi>

# -------------------------------------------------------------------------------------------------
# Recorded History                             
# ------------------------------------------------------------------------------------------------- 
#<record4xi record = "
#   kpu2image4xi5test.txt
#   sense-kpu2image4xi5test.xml
#   reason-kpu2image4xi5test.xml
#   ace-kpu2image4xi5test.xml
#"
#>

#<acknowledge4xi acknowledgement = "33" tester = "Small & Smart"/>

# -------------------------------------------------------------------------------------------------
# to-be-done                              
# -------------------------------------------------------------------------------------------------    
#<do4xi do = "
# nothing
#"
#/>

#<note4xi note = "
# parse4xi.py - to parse this module
# "/>

# -------------------------------------------------------------------------------------------------
# Legal Notice                             
# ------------------------------------------------------------------------------------------------- 
#<protect4xi protection = "copyright"
#   owner = "Small & Smart"
#   rights = "All rights reserved"
#   state = "California"
#   country = "United States of America"
#   codes = "Codes established in California and applicable to corporations in California"
#   compliance = "The California Law"
#   court = "Supreme Court of California, 350 McAllister Street, San Francisco, CA 94102-4783"
#/>

#<end4xi end = "
# -------------------------------------------------------------------------------------------------
# end of kpu2image4xi.py                              
# ------------------------------------------------------------------------------------------------- 
#"
#/>
